;@Id: boot.tpl#117 @
;=============================================================================
;  FILENAME:   boot.asm
;  VERSION:    4.06
;  DATE:       8 March 2005
;
;  DESCRIPTION:
;  M8C Boot Code for CY7C63800 microcontroller family.
;  This file also contains the Interrupt Service Routines for enCoRe II GPIO
;  interrupts: INT0, INT1, INT2, as well as the GPIO Port interrupts for
;  port 0, port 1, port 2, and port 3.
;
;  Copyright (C) Cypress Semiconductor 2004, 2005. All rights reserved.
;
; NOTES:
; PSoC Designer's Device Editor uses a template file, BOOT.TPL, located in
; the project's root directory to create BOOT.ASM. Any changes made to 
; BOOT.ASM will be  overwritten every time the project is generated; therfore
; changes should be made to BOOT.TPL not BOOT.ASM. Care must be taken when
; modifying BOOT.TPL so that replacement strings (such as @PROJECT_NAME)
; are not accidentally modified.
;
;=============================================================================

include ".\lib\GlobalParams.inc"
include "m8c.inc"
include "m8ssc.inc"

;-----------------------------------------------------------------------------
; Optimization flags
;-----------------------------------------------------------------------------
C_LANGUAGE_SUPPORT: equ 1   ;Set to 0 to optimize for ASM only

;-----------------------------------------------------------------------------
; Export Declarations
;-----------------------------------------------------------------------------

export __Start
export __Exit
export __bss_start

export __lit_start
export __idata_start
export __data_start
export __func_lit_start
export __text_start
export __usermodules_start
export __psoc_config_start

;-----------------------------------------------------------------------------
; Interrupt Vector Table
;-----------------------------------------------------------------------------
;
; Interrupt vector table entries are 4 bytes long and contain the code that
; services the interrupt (or causes it to be serviced).
;
;-----------------------------------------------------------------------------

    AREA    TOP(ROM,ABS,CON)

    org 0                   ;Reset Interrupt Vector
    ljmp bootLoaderVerify

    org 04h                 ;Supply Monitor Interrupt Vector
    jmp userSupplyMonitor

    org 08h                 ;INT0 Interrupt Vector
    jmp userInt0
    reti

    org 0Ch                 ;SPI TX Empty Interrupt Vector
    jmp userSpiTxEmpty
    reti

    org 10h                 ;SPI RX Full Interrupt Vector
    jmp userSpiRxFull
    reti

    org 14h                 ;GPIO Port 0 Interrupt Vector
    jmp userGpioPort0
    reti
    
    org 18h                 ;GPIO Port 1 Interrupt Vector
    jmp userGpioPort1
    reti
    
    org 1Ch                 ;INT1 Interrupt Vector
    jmp userInt1
    reti
    
    org 20h                 ;USB Endpoint 0 Interrupt Vector
    ljmp bootEp0Isr
    reti

    org 24h                 ;USB Endpoint 1 Interrupt Vector
    ljmp bootEp1Isr
    reti

    org 28h                 ;USB Endpoint 2 Interrupt Vector
    ljmp bootEp2Isr
    reti

    org 2Ch                 ;USB Bus Reset Interrupt Vector
    ljmp bootResetIsr
    reti

    org 30h                 ;USB Bus Activity Interrupt Vector
    ljmp bootUsbActivity
    reti

    org 34h                 ;One Millisecond Interval Timer Interrupt Vector
    jmp userMsTimer
    reti

    org 38h                 ;Programmable Interval Timer Interrupt Vector
    jmp userPiTimer
    reti

    org 3Ch                 ;Timer Capture 0 Interrupt Vector
    jmp userTcap0
    reti
	
    org 40h                 ;Timer Capture 1 Interrupt Vector
    jmp userTcap1
    reti

    org 44h                 ;Free Running Counter Wrap Interrupt Vector
    jmp userCounterWrap
    reti

    org 48h                 ;INT 2 Interrupt Vector
    jmp userInt2
    reti

    org 4Ch                 ;PS2 Data Low
    jmp userPs2Low
    reti

    org 50h                 ;GPIO Port 2 Interrupt Vector
    jmp userGpioPort2
    reti

    org 54h                 ;GPIO Port 3 Interrupt Vector
    jmp userGpioPort3
    reti

    org 58h                 ;Reserved
    jmp userRes1
    reti

    org 5Ch                 ;Reserved
    jmp userRes2
    reti

    org 60h                 ;Reserved
    jmp userRes3
    reti

    org 64h                 ;Sleep Timer Interrupt Vector
    jmp userSleepTimer
    reti


;*******************************************************;
;*** Blocks 2 and 3 0080 to 00FF Writable Boot table ***;
;*******************************************************;

    org 80h                 ;Supply Monitor Interrupt Vector
    userSupplyMonitor::
    halt                    ;Stop execution if power falls too low

    org 84h                 ;INT0 Interrupt Vector
    userInt0::
    ljmp    INT0_ISR
    reti

    org 88h                 ;SPI TX Empty Interrupt Vector
    userSpiTxEmpty::
    `@INTERRUPT_3`
    reti

    org 8Ch                 ;SPI RX Full Interrupt Vector
    userSpiRxFull::
    `@INTERRUPT_4`
    reti

    org 90h                 ;GPIO Port 0 Interrupt Vector
    userGpioPort0::
    ljmp    PORT0_ISR
    reti
    
    org 94h                 ;GPIO Port 1 Interrupt Vector
    userGpioPort1::
    ljmp    PORT1_ISR
    reti
    
    org 98h                 ;INT1 Interrupt Vector
    userInt1::
    ljmp    INT1_ISR
    reti
    
    org 9Ch                 ;USB Endpoint 0 Interrupt Vector
    userUsbEp0::
    `@INTERRUPT_8`
    reti

    org A0h                 ;USB Endpoint 1 Interrupt Vector
    userUsbEp1::
    `@INTERRUPT_9`
    reti

    org A4h                 ;USB Endpoint 2 Interrupt Vector
    userUsbEp2::
    `@INTERRUPT_10`
    reti

    org A8h                 ;USB Bus Reset Interrupt Vector
    userUsbReset::
    `@INTERRUPT_11`
    reti

    org ACh                 ;USB Bus Activity Interrupt Vector
    userUsbActivity::
    `@INTERRUPT_12`
    reti

    org B0h                 ;One Millisecond Interval Timer Interrupt Vector
    userMsTimer::
    `@INTERRUPT_13`
    reti

    org B4h                 ;Programmable Interval Timer Interrupt Vector
    userPiTimer::
    `@INTERRUPT_14`
    reti

    org B8h                 ;Timer Capture 0 Interrupt Vector
    userTcap0::
    `@INTERRUPT_15`
    reti
	
    org BCh                 ;Timer Capture 1 Interrupt Vector
    userTcap1::
    `@INTERRUPT_16`
    reti

    org C0h                 ;Free Running Counter Wrap Interrupt Vector
    userCounterWrap::
    `@INTERRUPT_17`
    reti

    org C4h                 ;INT 2 Interrupt Vector
    userInt2::
    ljmp    INT2_ISR
    reti

    org C8h                 ;PS2 Data Low
    userPs2Low::
    `@INTERRUPT_19`
    reti

    org CCh                 ;GPIO Port 2 Interrupt Vector
    userGpioPort2::
    ljmp    PORT2_ISR
    reti

    org D0h                 ;GPIO Port 3 Interrupt Vector
    userGpioPort3::
    ljmp    PORT3_ISR
    reti

    org D4h                 ;Reserved
    userRes1::
    `@INTERRUPT_22`
    reti

    org D8h                 ;Reserved
    userRes2::
    `@INTERRUPT_23`
    reti

    org DCh                 ;Reserved
    userRes3::
    `@INTERRUPT_24`
    reti

    org E0h                 ;Sleep Timer Interrupt Vector
    userSleepTimer::
    `@INTERRUPT_25`
    reti

    org E4h                 ;Vector for start at known location for BL
    __Start:
    jmp __Startup


;-----------------------------------------------------------------------------
;  USB Descriptors
;  - located here at a fixed location for bootloader ~100h
;  - writable flash area for descriptor updates
;  - modify custom.lkp and org point for AREA called start to 
;    grow/shrink the descriptor area
;-----------------------------------------------------------------------------
    AREA usb_desc(ROM,ABS,CON)

;-----------------------------------------------------------------------------
;  Start of Execution
;  CPU is operating at 3 MHz, change to 12 MHz
;  IO Bank is Bank0
;-----------------------------------------------------------------------------
    AREA start(ROM,ABS,CON)
    org  300h
__Startup:

IF ( WATCHDOG_ENABLE )             ; WDT selected in Global Params
    M8C_EnableWatchDog
ENDIF
    ;------------------
    ; Set up the clocks
    ;------------------

    ; Configure the Clock Out
    OR   REG[CLKIOCR], (CLOCK_OUT_JUST)

    mov  [bSSC_KEY1],0             ; Lockout Flash and Supervisiory operations
    mov  [bSSC_KEYSP],0      

    ;------------------
    ; Set up the stack
    ;------------------
    mov   A, __ramareas_end        ; Set top of stack to end of used RAM
    swap  SP, A                    ; This is only temporary if going to LMM

    ;-------------------------------------------------------------------------
    ; All the user selections and UserModule selections are now loaded,
    ; except CPU frequency (CPU is runing at 12 MHz).  Load the PSoC 
    ; configuration with a 12 MHz CPU clock to keep config time short.
    ;-------------------------------------------------------------------------
    lcall LoadConfigInit           ; Configure PSoC blocks per Dev Editor


IF (C_LANGUAGE_SUPPORT)
    call InitCRunTime              ; Initialize for C language
ENDIF ;(C_LANGUAGE_SUPPORT)

    ;-------------------------------------------------------------------------
    ; Global Interrupt are NOT enabled, this should be done in main().
    ; LVD is set but will not occur unless Global Interrupts are enabled. 
    ; Global Interrupts should be as soon as possible in main().
    ;-------------------------------------------------------------------------
    mov  reg[INT_VC],0              ; Clear any pending interrupts which may
                                    ; have been set during the boot process. 
    lcall _main                     ; Call main

__Exit:
    jmp  __Exit                     ; Wait here till power is turned off



;-----------------------------------------------------------------------------
; C Runtime Environment Initialization
; The following code is conditionally assembled.
;-----------------------------------------------------------------------------

IF (C_LANGUAGE_SUPPORT)

InitCRunTime:
    ;-----------------------------
    ; clear bss segment
    ;-----------------------------
    mov  A,0
    mov  [__r0],<__bss_start
BssLoop:
    cmp  [__r0],<__bss_end
    jz   BssDone
    mvi  [__r0],A
    jmp  BssLoop
BssDone:
    ;----------------------------
    ; copy idata to data segment
    ;----------------------------
    mov  A,>__idata_start
    mov  X,<__idata_start
    mov  [__r0],<__data_start
IDataLoop:
    cmp  [__r0],<__data_end
    jz   IDataDone
    push A
    romx
    mvi  [__r0],A
    pop  A
    inc  X
    adc  A,0
    jmp  IDataLoop
IDataDone:
    ret

ENDIF ;(C_LANGUAGE_SUPPORT)


;-----------------------------------------------------------------------------
; RAM segments for C CONST, static & global items
;-----------------------------------------------------------------------------
    AREA lit
__lit_start:

    AREA idata
__idata_start:

    AREA func_lit
__func_lit_start:

    AREA psoc_config(ROM,REL,CON)
__psoc_config_start:

    AREA UserModules(ROM,REL,CON)
__usermodules_start:

    AREA gpio_isr(ROM,REL,CON)
__gpio_isr_start:

;---------------------------------------------
;         CODE segment for general use
;---------------------------------------------
    AREA text(ROM,REL,CON)
__text_start:

;---------------------------------------------
;         Begin RAM area usage
;---------------------------------------------
    AREA usbRAM            (RAM, REL)        ; common APP/BL USB RAM area
    AREA data              (RAM, REL, CON)   ; initialized RAM
__data_start:

    AREA virtual_registers (RAM, REL, CON)   ; Temp vars of C compiler
    AREA InterruptRAM      (RAM, REL, CON)   ; Interrupts, on Page 0
    AREA bss               (RAM, REL, CON)   ; general use
__bss_start:
;-----------------------------------------------------------------------------
; End of the boot code
;-----------------------------------------------------------------------------

;-----------------------------------------------------------------------------
; GPIO Interrupt Service Routines
;-----------------------------------------------------------------------------
 AREA gpio_isr(ROM,REL,CON)
;-----------------------------------------------------------------------------
;  FUNCTION NAME: INT0_ISR
;
;  DESCRIPTION:   This is the ISR for the the INT0 GPIO interrupt
;
;-----------------------------------------------------------------------------
INT0_ISR:
   ;@PSoC_UserCode_BODY_1@ (Do not change this line.)
   ;---------------------------------------------------
   ; Insert your custom code below this banner
   ;---------------------------------------------------
   ;   NOTE: interrupt service routines must preserve
   ;   the values of the A and X CPU registers.

   ;---------------------------------------------------
   ; Insert your custom code above this banner
   ;---------------------------------------------------
   ;@PSoC_UserCode_END@ (Do not change this line.)
   RETI
;-----------------------------------------------------------------------------
;  FUNCTION NAME: INT1_ISR
;
;  DESCRIPTION:   This is the ISR for the the INT1 GPIO interrupt
;
;-----------------------------------------------------------------------------
INT1_ISR:
   ;@PSoC_UserCode_BODY_2@ (Do not change this line.)
   ;---------------------------------------------------
   ; Insert your custom code below this banner
   ;---------------------------------------------------
   ;   NOTE: interrupt service routines must preserve
   ;   the values of the A and X CPU registers.

   ;---------------------------------------------------
   ; Insert your custom code above this banner
   ;---------------------------------------------------
   ;@PSoC_UserCode_END@ (Do not change this line.)
   RETI
;-----------------------------------------------------------------------------
;  FUNCTION NAME: INT2_ISR
;
;  DESCRIPTION:   This is the ISR for the the INT2 GPIO interrupt
;
;-----------------------------------------------------------------------------
INT2_ISR:
   ;@PSoC_UserCode_BODY_3@ (Do not change this line.)
   ;---------------------------------------------------
   ; Insert your custom code below this banner
   ;---------------------------------------------------
   ;   NOTE: interrupt service routines must preserve
   ;   the values of the A and X CPU registers.

   ;---------------------------------------------------
   ; Insert your custom code above this banner
   ;---------------------------------------------------
   ;@PSoC_UserCode_END@ (Do not change this line.)
   RETI
;-----------------------------------------------------------------------------
;  FUNCTION NAME: PORT0_ISR
;
;  DESCRIPTION:   This is the ISR for the the PORT0 GPIO interrupt
;                 Note: Interrupts for GPIO P0.2, P0.3, and P0.4 are
;                 not dispatched through this ISR.  Those interrupts
;                 are dipatched through INT0, INT1 and INT2 respectively.
;
;-----------------------------------------------------------------------------
PORT0_ISR:
   ;@PSoC_UserCode_BODY_4@ (Do not change this line.)
   ;---------------------------------------------------
   ; Insert your custom code below this banner
   ;---------------------------------------------------
   ;   NOTE: interrupt service routines must preserve
   ;   the values of the A and X CPU registers.

   ;---------------------------------------------------
   ; Insert your custom code above this banner
   ;---------------------------------------------------
   ;@PSoC_UserCode_END@ (Do not change this line.)
   RETI
;-----------------------------------------------------------------------------
;  FUNCTION NAME: PORT1_ISR
;
;  DESCRIPTION:   This is the ISR for the the PORT1 GPIO interrupt
;
;-----------------------------------------------------------------------------
PORT1_ISR:
   ;@PSoC_UserCode_BODY_5@ (Do not change this line.)
   ;---------------------------------------------------
   ; Insert your custom code below this banner
   ;---------------------------------------------------
   ;   NOTE: interrupt service routines must preserve
   ;   the values of the A and X CPU registers.

   ;---------------------------------------------------
   ; Insert your custom code above this banner
   ;---------------------------------------------------
   ;@PSoC_UserCode_END@ (Do not change this line.)
   RETI
;-----------------------------------------------------------------------------
;  FUNCTION NAME: PORT2_ISR
;
;  DESCRIPTION:   This is the ISR for the the PORT2 GPIO interrupt
;
;-----------------------------------------------------------------------------
PORT2_ISR:
   ;@PSoC_UserCode_BODY_5@ (Do not change this line.)
   ;---------------------------------------------------
   ; Insert your custom code below this banner
   ;---------------------------------------------------
   ;   NOTE: interrupt service routines must preserve
   ;   the values of the A and X CPU registers.

   ;---------------------------------------------------
   ; Insert your custom code above this banner
   ;---------------------------------------------------
   ;@PSoC_UserCode_END@ (Do not change this line.)
   RETI
;-----------------------------------------------------------------------------
;  FUNCTION NAME: PORT3_ISR
;
;  DESCRIPTION:   This is the ISR for the the PORT3 GPIO interrupt
;
;-----------------------------------------------------------------------------
PORT3_ISR:
   ;@PSoC_UserCode_BODY_6@ (Do not change this line.)
   ;---------------------------------------------------
   ; Insert your custom code below this banner
   ;---------------------------------------------------
   ;   NOTE: interrupt service routines must preserve
   ;   the values of the A and X CPU registers.

   ;---------------------------------------------------
   ; Insert your custom code above this banner
   ;---------------------------------------------------
   ;@PSoC_UserCode_END@ (Do not change this line.)
   RETI
;-----------------------------------------------------------------------------
; End GPIO Interrupt Service Routines
;-----------------------------------------------------------------------------
;end of file
