<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:fo="http://www.w3.org/1999/XSL/Format">
  <xsl:include href="cGenPrim.xsl"/>
  <xsl:output method="text" omit-xml-declaration="yes" encoding="UTF-8"/>
  <xsl:param name="INSTANCE" select="/descendant::USB_ROOT/@INSTANCE_NAME"/>
  <xsl:template match="USB_ROOT">
    <xsl:call-template name="area">
      <xsl:with-param name="AREA_NAME" select="string('usb_desc')"/>
      <xsl:with-param name="TYPE" select="string('ROM')"/>
      <xsl:with-param name="LOC" select="string('ABS')"/>
      <xsl:with-param name="OVR" select="string('CON')"/>
    </xsl:call-template>
    <xsl:text>;----------------------------------------------------------------------
; Descriptor Vector Table
;   The following table provides pointers to key points in the descriptor 
;   tables for bootloader.  This table should be placed at address 100h.
;----------------------------------------------------------------------
EXPORT USB_VECTOR_TABLE
.LITERAL
USB_VECTOR_TABLE:
    DW </xsl:text>
    <xsl:value-of select="concat($INSTANCE, '_DEVICE_LOOKUP')"/><xsl:text>
    DW </xsl:text>
    <xsl:value-of select="concat($INSTANCE, '_DEVICE_DESCR_TABLE')"/><xsl:text>
    DW </xsl:text>
    <xsl:value-of select="concat($INSTANCE, '_StringTable')"/><xsl:text>
.ENDLITERAL

;----------------------------------------------------------------------
; Descriptor Dispatch Tables
;   The following tables provide the dispatching pointers and
;   descriptor lengths necessary for the request handler to
;   respond to USB requests.
;----------------------------------------------------------------------
</xsl:text>
    <xsl:call-template name="device-table"/>
    <xsl:text>;----------------------------------------------------------------------
</xsl:text>
    <xsl:call-template name="report-ram">
      <xsl:with-param name="NUM" select="0" />
    </xsl:call-template>
    <xsl:apply-templates/>
  </xsl:template>
  <xsl:template match="USB_DEVICE">
    <xsl:text>;----------------------------------------------------------------------
; Device Descriptor (</xsl:text>
    <xsl:value-of select="concat($INSTANCE,'_',@NAME)"/>)<xsl:text>
; This marks the beginning of the Device Descriptor.  This descriptor
; concatenates all of the descriptors in the following format:
;	Device Descriptor
;		Configuration Descriptor (1)
;	  	Interface 1 Descriptor
;				HID Descriptor 1
;			  	Endpoint Descriptor(s)
;			Interface 2 Descriptor
;				HID Descriptor 2
;					Endpoint Descriptor(s)
;		Configuration Descriptor (2)
;			Interface/HID/Endpoint...
;----------------------------------------------------------------------
</xsl:text>
    <xsl:call-template name="area"/>
    <xsl:call-template name="directive">
      <xsl:with-param name="DIRECTIVE" select="string('.LITERAL')"/>
    </xsl:call-template>
    <xsl:call-template name="label">
      <xsl:with-param name="LABEL" select="concat($INSTANCE,'_',@NAME,'_DESCR_START')"/>
      <xsl:with-param name="COMMENT" select="string(';')"/>
    </xsl:call-template>
    <xsl:call-template name="db">
      <xsl:with-param name="CODE" select="string('18')"/>
      <xsl:with-param name="COMMENT" select="string('; Device Descriptor Length (18)')"/>
    </xsl:call-template>
    <xsl:call-template name="db">
      <xsl:with-param name="CODE" select="string('1')"/>
      <xsl:with-param name="COMMENT" select="string('; bDescriptorType: DEVICE')"/>
    </xsl:call-template>
    <xsl:call-template name="dwl">
      <xsl:with-param name="CODE" select="string('0200')"/>
      <xsl:with-param name="COMMENT" select="string('; bcdUSB (ver 2.0)')"/>
      <xsl:with-param name="SUFFIX" select="string('H')"/>
    </xsl:call-template>
    <xsl:call-template name="db">
      <xsl:with-param name="CODE" select="@CLASS"/>
      <xsl:with-param name="COMMENT" select="string('; bDeviceClass')"/>
    </xsl:call-template>
    <xsl:call-template name="db">
      <xsl:with-param name="CODE" select="@SUBCLASS"/>
      <xsl:with-param name="COMMENT" select="string('; bDeviceSubClass ')"/>
    </xsl:call-template>
    <xsl:call-template name="db">
      <xsl:with-param name="CODE" select="@PROTOCOL"/>
      <xsl:with-param name="COMMENT" select="string('; bDeviceProtocol ')"/>
    </xsl:call-template>
    <xsl:call-template name="db">
      <xsl:with-param name="CODE" select="@MAX_PACKET_SIZE"/>
      <xsl:with-param name="COMMENT" select="string('; bMaxPacketSize0')"/>
    </xsl:call-template>
    <xsl:call-template name="dwl">
      <xsl:with-param name="CODE" select="@VENDOR_ID"/>
      <xsl:with-param name="COMMENT" select="string('; idVendor')"/>
      <xsl:with-param name="SUFFIX" select="string('H')"/>
    </xsl:call-template>
    <xsl:call-template name="dwl">
      <xsl:with-param name="CODE" select="@PRODUCT_ID"/>
      <xsl:with-param name="COMMENT" select="string('; idProduct')"/>
      <xsl:with-param name="SUFFIX" select="string('H')"/>
    </xsl:call-template>
    <xsl:call-template name="dwl">
      <xsl:with-param name="CODE" select="@DEVICE_RELEASE"/>
      <xsl:with-param name="COMMENT" select="string('; bcdDevice')"/>
      <xsl:with-param name="SUFFIX" select="string('H')"/>
    </xsl:call-template>
    <xsl:call-template name="db">
      <xsl:with-param name="CODE" select="concat('STR_HASH_', @VENDOR_STRING)"/>
      <xsl:with-param name="COMMENT" select="string('; iManufacturer')"/>
    </xsl:call-template>
    <xsl:call-template name="db">
      <xsl:with-param name="CODE" select="concat('STR_HASH_', @PRODUCT_STRING)"/>
      <xsl:with-param name="COMMENT" select="string('; iProduct')"/>
    </xsl:call-template>
    <xsl:call-template name="db">
      <xsl:with-param name="CODE" select="concat('STR_HASH_', @SN_STRING)"/>
      <xsl:with-param name="COMMENT" select="string('; iSerialNumber')"/>
    </xsl:call-template>
    <xsl:call-template name="db">
      <xsl:with-param name="CODE" select="count(child::USB_CONFIGURATION)"/>
      <xsl:with-param name="COMMENT" select="string('; bNumConfigurations')"/>
    </xsl:call-template>
    <xsl:call-template name="label">
      <xsl:with-param name="LABEL" select="concat($INSTANCE,'_',@NAME,'_DESCR_END')"/>
      <xsl:with-param name="COMMENT" select="string(';')"/>
    </xsl:call-template>
    <xsl:call-template name="equ">
      <xsl:with-param name="LABEL" select="concat($INSTANCE,'_',@NAME,'_DESCR_SIZE')"/>
      <xsl:with-param name="SIZE" select="concat($INSTANCE,'_',@NAME,'_DESCR_END',' - ',$INSTANCE,'_',@NAME,'_DESCR_START')"/>
      <xsl:with-param name="COMMENT" select="string('')"/>
    </xsl:call-template>
    <xsl:apply-templates/>
    <xsl:call-template name="directive">
      <xsl:with-param name="DIRECTIVE" select="string('.ENDLITERAL')"/>
    </xsl:call-template>
  </xsl:template>
  <xsl:template match="USB_CONFIGURATION">
;----------------------------------------------------------------------
; Configuration Descriptor (<xsl:value-of select="concat($INSTANCE,'_',../@NAME,'_',@NAME)"/>)
; This marks the beginning of the Configuration Descriptor.  This descriptor
; concatenates all of the descriptors in the following format:
; Configuration Descriptor
;	Interface 1 Descriptor
;		HID Descriptor 1
;			Endpoint Descriptor(s)
;	Interface 2 Descriptor
;		HID Descriptor 2
;			Endpoint Descriptor(s)
;----------------------------------------------------------------------
<xsl:call-template name="label">
      <xsl:with-param name="LABEL" select="concat($INSTANCE,'_',../@NAME, '_',@NAME,'_DESCR_START')"/>
      <xsl:with-param name="COMMENT" select="string(';')"/>
    </xsl:call-template>
    <xsl:call-template name="db">
      <xsl:with-param name="CODE" select="9"/>
      <xsl:with-param name="COMMENT" select="string('; Configuration Descriptor Length (9)')"/>
    </xsl:call-template>
    <xsl:call-template name="db">
      <xsl:with-param name="CODE" select="2"/>
      <xsl:with-param name="COMMENT" select="string('; bDescriptorType: CONFIGURATION')"/>
    </xsl:call-template>
    <xsl:call-template name="dwl">
      <xsl:with-param name="CODE" select="concat($INSTANCE,'_',../@NAME,'_',@NAME,'_DESCR_SIZE')"/>
      <xsl:with-param name="COMMENT" select="string('; wTotalLength')"/>
    </xsl:call-template>
    <xsl:call-template name="db">
      <xsl:with-param name="CODE" select="count(child::USB_INTERFACE)"/>
      <xsl:with-param name="COMMENT" select="string('; bNumInterfaces')"/>
    </xsl:call-template>
    <xsl:call-template name="db">
      <xsl:with-param name="CODE" select="position()"/>
      <xsl:with-param name="COMMENT" select="string('; bConfigurationValue')"/>
    </xsl:call-template>
    <xsl:call-template name="db">
      <xsl:with-param name="CODE" select="concat('STR_HASH_',@CONFIGURATION_STRING)"/>
      <xsl:with-param name="COMMENT" select="string('; iConfiguration')"/>
    </xsl:call-template>
    <xsl:call-template name="db">
      <xsl:with-param name="CODE" select="concat('80H | (',@SELF_POWERED, ' &lt;&lt; 6) | (',@REMOTE_WAKEUP,' &lt;&lt; 5)')"/>
      <xsl:with-param name="COMMENT" select="string('; bmAttributes >')"/>
    </xsl:call-template>
    <xsl:call-template name="db">
      <xsl:with-param name="CODE" select="number(@MAX_POWER div 2)"/>
      <xsl:with-param name="COMMENT" select="string('; bMaxPower')"/>
    </xsl:call-template>
    <xsl:apply-templates/>
    <xsl:text>;----------------------------------------------------------------------</xsl:text>
    <xsl:text>&#10;</xsl:text>
    <xsl:value-of select="$INSTANCE"/>_<xsl:value-of select="../@NAME"/>_<xsl:value-of select="@NAME"/>_DESCR_END:
<xsl:value-of select="$INSTANCE"/>_<xsl:value-of select="../@NAME"/>_<xsl:value-of select="@NAME"/>_DESCR_SIZE:	EQU	(<xsl:value-of select="$INSTANCE"/>_<xsl:value-of select="../@NAME"/>_<xsl:value-of select="@NAME"/>_DESCR_END - <xsl:value-of select="$INSTANCE"/>_<xsl:value-of select="../@NAME"/>_<xsl:value-of select="@NAME"/>_DESCR_START)
</xsl:template>
  <xsl:template match="USB_INTERFACE">;----------------------------------------------------------------------
; Interface Descriptor (<xsl:value-of select="concat($INSTANCE,'_',../../@NAME,'_',../@NAME,'_',@NAME)"/>)
;----------------------------------------------------------------------
<xsl:call-template name="db">
      <xsl:with-param name="CODE" select="9"/>
      <xsl:with-param name="COMMENT" select="string('; Interface Descriptor Length (9)')"/>
    </xsl:call-template>
    <xsl:call-template name="db">
      <xsl:with-param name="CODE" select="4"/>
      <xsl:with-param name="COMMENT" select="string('; bDescriptorType: INTERFACE')"/>
    </xsl:call-template>
    <xsl:call-template name="db">
      <xsl:with-param name="CODE" select="position()-1"/>
      <xsl:with-param name="COMMENT" select="string('; bInterfaceNumber (zero based)')"/>
    </xsl:call-template>
    <xsl:call-template name="db">
      <xsl:with-param name="CODE" select="0"/>
      <xsl:with-param name="COMMENT" select="string('; bAlternateSetting')"/>
    </xsl:call-template>
    <xsl:call-template name="db">
      <xsl:with-param name="CODE" select="count(descendant::USB_ENDPOINT)"/>
      <xsl:with-param name="COMMENT" select="string('; bNumEndpoints')"/>
    </xsl:call-template>
    <xsl:call-template name="db">
      <xsl:with-param name="CODE" select="@CLASS"/>
      <xsl:with-param name="COMMENT" select="string('; bInterfaceClass')"/>
    </xsl:call-template>
    <xsl:call-template name="db">
      <xsl:with-param name="CODE" select="@SUBCLASS"/>
      <xsl:with-param name="COMMENT" select="string('; bInterfaceSubClass')"/>
    </xsl:call-template>
    <xsl:call-template name="db">
      <xsl:with-param name="CODE" select="@PROTOCOL"/>
      <xsl:with-param name="COMMENT" select="string('; bInterfaceProtocol')"/>
    </xsl:call-template>
    <xsl:call-template name="db">
      <xsl:with-param name="CODE" select="concat('STR_HASH_',@INTERFACE_STRING)"/>
      <xsl:with-param name="COMMENT" select="string('; iInterface')"/>
    </xsl:call-template>
    <xsl:apply-templates/>
  </xsl:template>
  <xsl:template match="USB_ENDPOINT">;----------------------------------------------------------------------
; Endpoint Descriptor (<xsl:value-of select="concat($INSTANCE,'_',../../../@NAME,'_',../../@NAME,'_',../@NAME,'_',@NAME)"/>)
;----------------------------------------------------------------------
<xsl:call-template name="db">
      <xsl:with-param name="CODE" select="7"/>
      <xsl:with-param name="COMMENT" select="string('; Endpoint Descriptor Length (7)')"/>
    </xsl:call-template>
    <xsl:call-template name="db">
      <xsl:with-param name="CODE" select="5"/>
      <xsl:with-param name="COMMENT" select="string('; bDescriptorType: ENDPOINT')"/>
    </xsl:call-template>
    <xsl:call-template name="db">
      <xsl:with-param name="CODE" select="concat('(', @ADDRESS, ' | USB_DIR_', @DIRECTION, ')')"/>
      <xsl:with-param name="COMMENT" select="string('; bEndpointAddress')"/>
    </xsl:call-template>
    <xsl:call-template name="db">
      <xsl:with-param name="CODE" select="(@TRANSFER_TYPE + (@SYNC_TYPE * 4) + (@USAGE_TYPE * 4))"/>
      <xsl:with-param name="COMMENT" select="string('; bmAttributes')"/>
    </xsl:call-template>
    <xsl:call-template name="dwl">
      <xsl:with-param name="CODE" select="@MAX_PACKET_SIZE"/>
      <xsl:with-param name="COMMENT" select="string('; wMaxPacketSize')"/>
    </xsl:call-template>
    <xsl:call-template name="db">
      <xsl:with-param name="CODE" select="@INTERVAL"/>
      <xsl:with-param name="COMMENT" select="string('; bInterval')"/>
    </xsl:call-template>
    <xsl:apply-templates/>
  </xsl:template>
  <xsl:template match="USB_HID">;----------------------------------------------------------------------
; HID Class Descriptor (<xsl:value-of select="concat($INSTANCE,'_',../../../@NAME,'_',../../@NAME,'_',../@NAME)"/>)
;----------------------------------------------------------------------
<xsl:call-template name="label">
      <xsl:with-param name="LABEL" select="concat($INSTANCE,'_',../../../@NAME,'_', ../../@NAME,'_', ../@NAME,'_HID_DESCR_START')"/>
      <xsl:with-param name="COMMENT" select="string('')"/>
    </xsl:call-template>
    <xsl:call-template name="db">
      <xsl:with-param name="CODE" select="9"/>
      <xsl:with-param name="COMMENT" select="string('; bLength--HID Class Descriptor Length (9)')"/>
    </xsl:call-template>
    <xsl:call-template name="db">
      <xsl:with-param name="CODE" select="string('0x21')"/>
      <xsl:with-param name="COMMENT" select="string('; bDescriptorType: HID Class')"/>
    </xsl:call-template>
    <xsl:call-template name="dwl">
      <xsl:with-param name="CODE" select="string('0x0111')"/>
      <xsl:with-param name="COMMENT" select="string('; bcdHID')"/>
    </xsl:call-template>
    <xsl:call-template name="db">
      <xsl:with-param name="CODE" select="@COUNTRY_CODE"/>
      <xsl:with-param name="COMMENT" select="string('; bCountryCode')"/>
    </xsl:call-template>
    <xsl:call-template name="db">
      <xsl:with-param name="CODE" select="1"/>
      <xsl:with-param name="COMMENT" select="string('; bNumDescriptors')"/>
    </xsl:call-template>
    <xsl:call-template name="db">
      <xsl:with-param name="CODE" select="@HID_DESCR_TYPE"/>
      <xsl:with-param name="COMMENT" select="string('; bDescriptorType')"/>
    </xsl:call-template>
    <xsl:choose>
      <xsl:when test="@HID_REPORT > 0">
        <xsl:call-template name="dwl">
          <xsl:with-param name="CODE" select="concat($INSTANCE,'_HID_REPORT_', @HID_REPORT,'_DESCR_SIZE')"/>
          <xsl:with-param name="COMMENT" select="string('; wDescriptorLength')"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
        <xsl:call-template name="dwl">
          <xsl:with-param name="CODE" select="0"/>
          <xsl:with-param name="COMMENT" select="string('; wDescriptorLength (No report was selected in the setup wizard)')"/>
        </xsl:call-template>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:apply-templates/>
  </xsl:template>
  <!-- TRANSFER DESCRIPTOR TABLE BUILDERS -->
  <xsl:template name="dev-descr-table">
;----------------------------------------------------------------------
; Device Descriptor Table
;
;	This table provides transfer descriptors for each USB Device Descriptor
;----------------------------------------------------------------------
<xsl:call-template name="area">
  <xsl:with-param name="AREA_NAME" select="string('usb_desc')"/>
</xsl:call-template>
<xsl:call-template name="export-label">
      <xsl:with-param name="LABEL" select="concat($INSTANCE, '_DEVICE_DESCR_TABLE')"/>
      <xsl:with-param name="LITERAL" select="string('TRUE')"/>
    </xsl:call-template>
    <xsl:call-template name="td-header">
      <xsl:with-param name="COUNT" select="count(child::USB_DEVICE)"/>
      <xsl:with-param name="COMMENT" select="string('; Number of devices')"/>
    </xsl:call-template>
    <xsl:for-each select="USB_DEVICE">
      <xsl:call-template name="td-entry">
        <xsl:with-param name="MEM" select="string('USB_DS_ROM')"/>
        <xsl:with-param name="ADDR" select="concat($INSTANCE,'_',@NAME,'_DESCR_START')"/>
        <xsl:with-param name="SIZE" select="concat($INSTANCE,'_',@NAME,'_DESCR_SIZE')"/>
        <xsl:with-param name="PTR" select="string('NULL_PTR')"/>
      </xsl:call-template>
    </xsl:for-each>
    <xsl:call-template name="directive">
      <xsl:with-param name="DIRECTIVE" select="string('.ENDLITERAL')"/>
    </xsl:call-template>
  </xsl:template>
  <xsl:template name="cfg-descr-table">
;----------------------------------------------------------------------
; Configuration Descriptor Table
;  for (<xsl:value-of select="$INSTANCE"/>_<xsl:value-of select="@NAME"/>)
;
;	This table provides transfer descriptors for each USB Configuration
; Descriptor
;----------------------------------------------------------------------
<xsl:call-template name="area">
  <xsl:with-param name="AREA_NAME" select="string('usb_desc')"/>
</xsl:call-template>
<xsl:call-template name="label">
      <xsl:with-param name="LABEL" select="concat($INSTANCE,'_',@NAME, '_CONFIG_DESCR_TABLE')"/>
      <xsl:with-param name="COMMENT" select="string(';')"/>
      <xsl:with-param name="LITERAL" select="string('TRUE')"/>
    </xsl:call-template>
<xsl:call-template name="td-header">
      <xsl:with-param name="COUNT" select="count(child::USB_CONFIGURATION)"/>
      <xsl:with-param name="COMMENT" select="string('; Number of configurations')"/>
    </xsl:call-template>
    <xsl:for-each select="USB_CONFIGURATION">
      <xsl:call-template name="td-entry">
        <xsl:with-param name="MEM" select="string('USB_DS_ROM')"/>
        <xsl:with-param name="ADDR" select="concat($INSTANCE,'_',../@NAME,'_',@NAME,'_DESCR_START')"/>
        <xsl:with-param name="SIZE" select="concat($INSTANCE,'_',../@NAME,'_',@NAME,'_DESCR_SIZE')"/>
        <xsl:with-param name="PTR" select="string('NULL_PTR')"/>
      </xsl:call-template>
    </xsl:for-each>
    <xsl:call-template name="directive">
      <xsl:with-param name="DIRECTIVE" select="string('.ENDLITERAL')"/>
    </xsl:call-template>
  </xsl:template>
  <!-- DISPATCH TABLE BUILDERS -->
  <xsl:template name="device-table">
;----------------------------------------------------------------------
; Device Dispatch Table
;   for User Module: (<xsl:value-of select="$INSTANCE"/>)
;----------------------------------------------------------------------
<xsl:call-template name="area">
  <xsl:with-param name="AREA_NAME" select="string('usb_desc')"/>
</xsl:call-template>
<xsl:call-template name="export-label">
      <xsl:with-param name="LABEL" select="concat($INSTANCE,'_DEVICE_LOOKUP')"/>
      <xsl:with-param name="COMMENT" select="string(';')"/>
      <xsl:with-param name="LITERAL" select="string('TRUE')"/>
    </xsl:call-template>
    <xsl:call-template name="lt-header">
      <xsl:with-param name="COUNT" select="count(child::USB_DEVICE)"/>
      <xsl:with-param name="COMMENT" select="concat('; Number of devices defined for ', $INSTANCE)"/>
    </xsl:call-template>
    <xsl:for-each select="USB_DEVICE">
      <xsl:call-template name="lt-entry">
        <xsl:with-param name="ADDR" select="concat($INSTANCE, '_', @NAME, '_CONFIG_LOOKUP')"/>
        <xsl:with-param name="DPTR" select="concat($INSTANCE, '_', @NAME, '_CONFIG_DESCR_TABLE')"/>
      </xsl:call-template>
    </xsl:for-each>
    <xsl:call-template name="directive">
      <xsl:with-param name="DIRECTIVE" select="string('.ENDLITERAL')"/>
    </xsl:call-template>
    <xsl:for-each select="USB_DEVICE">
      <xsl:call-template name="cfg-table"/>
    </xsl:for-each>
    <xsl:call-template name="dev-descr-table"/>
  </xsl:template>
  <xsl:template name="cfg-table">
    <!--
	<xsl:for-each select="USB_CONFIGURATION">
		<xsl:call-template name="interface-table"/>
	</xsl:for-each>-->
;----------------------------------------------------------------------
; Configuration Dispatch Table
;   for Device: (<xsl:value-of select="$INSTANCE"/>_<xsl:value-of select="@NAME"/>)
;----------------------------------------------------------------------
<xsl:call-template name="area">
  <xsl:with-param name="AREA_NAME" select="string('usb_desc')"/>
</xsl:call-template>
<xsl:call-template name="export-label">
      <xsl:with-param name="LABEL" select="concat($INSTANCE,'_',@NAME,'_CONFIG_LOOKUP')"/>
      <xsl:with-param name="COMMENT" select="string(';')"/>
      <xsl:with-param name="LITERAL" select="string('TRUE')"/>
    </xsl:call-template>
    <xsl:call-template name="lt-header">
      <xsl:with-param name="COUNT" select="count(child::USB_CONFIGURATION)"/>
      <xsl:with-param name="COMMENT" select="string('; Number of configurations')"/>
    </xsl:call-template>
    <xsl:for-each select="USB_CONFIGURATION">
      <xsl:choose>
        <xsl:when test="count(child::USB_INTERFACE/USB_HID) > 0">
          <xsl:call-template name="lt-entry">
    	  <xsl:with-param name="ADDR" select="concat($INSTANCE, '_', ../@NAME, '_', @NAME, '_EP_SETUP')"/>
            <xsl:with-param name="DPTR" select="concat($INSTANCE, '_', ../@NAME, '_', @NAME, '_INTERFACE_LOOKUP')"/>
          </xsl:call-template>
        </xsl:when>
        <xsl:otherwise>
          <xsl:call-template name="lt-entry">
            <xsl:with-param name="COMMENT" select="string('; No HID Class Descriptors')"/>
          </xsl:call-template>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:for-each>
    <xsl:call-template name="directive">
      <xsl:with-param name="DIRECTIVE" select="string('.ENDLITERAL')"/>
    </xsl:call-template>
    <xsl:for-each select="USB_CONFIGURATION">
      <xsl:call-template name="hid-lt-table"/>
      <xsl:call-template name="ep-setup-table"/>
      <xsl:call-template name="ifc-lookup-table"/>
      <xsl:call-template name="hid-class-descr-td-table"/>
      <xsl:call-template name="hid-rpt-descr-td-table"/>
      <xsl:for-each select="USB_INTERFACE">
        <xsl:call-template name="ifc-rpt-table"/>
      </xsl:for-each>
    </xsl:for-each>
    <xsl:call-template name="cfg-descr-table"/>
  </xsl:template>

  <xsl:template name="ep-setup-table">
    <!-- Context: USB_CONFIGURATION -->
    <xsl:text>
;----------------------------------------------------------------------
; Endpoint Setup Table
; 
; This table provides the data to configure the endpoint mode registers
; for IN/OUT direction.
;----------------------------------------------------------------------
</xsl:text>
  <xsl:call-template name="area">
    <xsl:with-param name="AREA_NAME" select="string('usb_desc')"/>
  </xsl:call-template>
  <xsl:call-template name="label">
    <xsl:with-param name="LABEL" select="concat($INSTANCE, '_', ../@NAME, '_', @NAME, '_EP_SETUP')"/>
    <xsl:with-param name="COMMENT" select="string(';')"/>
    <xsl:with-param name="LITERAL" select="string('TRUE')"/>
  </xsl:call-template>
  <xsl:call-template name="ep-direction">
    <xsl:with-param name="EP" select="string('EP1')"/>
  </xsl:call-template>
  <xsl:call-template name="ep-direction">
    <xsl:with-param name="EP" select="string('EP2')"/>
  </xsl:call-template>
    <xsl:call-template name="directive">
      <xsl:with-param name="DIRECTIVE" select="string('.ENDLITERAL')"/>
    </xsl:call-template>
 </xsl:template>

  <xsl:template name="hid-lt-table">
    <!-- Context: USB_CONFIGURATION -->
    <xsl:if test="count(child::USB_INTERFACE/USB_HID[@HID_REPORT != 0]) > 0">
      <xsl:text>
;----------------------------------------------------------------------
; Interface Look Up Table
; 
; This look up table points to three tables.  Each of those tables is
; indexed by the selected interface number provided by the request.
;
; Item 0: HID Report GET_REPORT/SET_REPORT Lookup Table
; Item 1: HID Report Descriptor Transfer Descriptor Table
; Item 2: HID Class Descriptor Transfer Descriptor Table
;----------------------------------------------------------------------
</xsl:text>
<xsl:call-template name="area">
  <xsl:with-param name="AREA_NAME" select="string('usb_desc')"/>
</xsl:call-template>
      <xsl:call-template name="export-label">
        <xsl:with-param name="LABEL" select="concat($INSTANCE, '_', ../@NAME, '_', @NAME, '_INTERFACE_LOOKUP')"/>
        <xsl:with-param name="COMMENT" select="string(';')"/>
        <xsl:with-param name="LITERAL" select="string('TRUE')"/>
      </xsl:call-template>
      <xsl:call-template name="dw">
        <xsl:with-param name="CODE" select="concat($INSTANCE,'_',../@NAME,'_', @NAME, '_INTERFACE_RPT_LOOKUP')"/>
        <xsl:with-param name="COMMENT" select="string('; Interface GET_REPORT/SET_REPORT Lookup Table')"/>
      </xsl:call-template>
      <xsl:call-template name="dw">
        <xsl:with-param name="CODE" select="concat($INSTANCE,'_',../@NAME,'_', @NAME, '_HID_RPT_DESCR_TABLE')"/>
        <xsl:with-param name="COMMENT" select="string('; HID Report Descriptor Transfer Descriptor Table')"/>
      </xsl:call-template>
      <xsl:call-template name="dw">
        <xsl:with-param name="CODE" select="concat($INSTANCE,'_',../@NAME,'_', @NAME, '_HID_CLASS_DESCR_TABLE')"/>
        <xsl:with-param name="COMMENT" select="string('; HID Class Descriptor Transfer Descriptor Table')"/>
      </xsl:call-template>
      <xsl:call-template name="directive">
        <xsl:with-param name="DIRECTIVE" select="string('.ENDLITERAL')"/>
      </xsl:call-template>
    </xsl:if>
  </xsl:template>
  <xsl:template name="hid-rpt-descr-td-table">
    <!-- Context: USB_CONFIGURATION -->
    <xsl:if test="count(child::USB_INTERFACE/USB_HID[@HID_REPORT != 0]) > 0">
      <xsl:text>
;----------------------------------------------------------------------
; HID Report Descriptor transfer descriptor table
;----------------------------------------------------------------------
</xsl:text>
    <xsl:call-template name="area">
      <xsl:with-param name="AREA_NAME" select="string('usb_desc')"/>
    </xsl:call-template>
      <xsl:call-template name="export-label">
        <xsl:with-param name="LABEL" select="concat($INSTANCE,'_',../@NAME,'_', @NAME, '_HID_RPT_DESCR_TABLE')"/>
        <xsl:with-param name="COMMENT" select="string(';')"/>
        <xsl:with-param name="LITERAL" select="string('TRUE')"/>
      </xsl:call-template>
      <xsl:call-template name="td-header">
        <xsl:with-param name="COUNT" select="count(child::USB_INTERFACE)"/>
        <xsl:with-param name="COMMENT" select="string('; Number of interfaces/HID Report Descriptors')"/>
      </xsl:call-template>
      <xsl:for-each select="USB_INTERFACE">
        <xsl:choose>
          <xsl:when test="count(child::USB_HID) > 0">
            <xsl:call-template name="td-entry">
              <xsl:with-param name="MEM" select="string('USB_DS_ROM')"/>
              <xsl:with-param name="ADDR" select="concat($INSTANCE, '_HID_REPORT_',child::USB_HID/@HID_REPORT, '_DESCR_START')"/>
              <xsl:with-param name="SIZE" select="concat($INSTANCE,'_HID_REPORT_', child::USB_HID/@HID_REPORT,'_DESCR_SIZE')"/>
              <xsl:with-param name="PTR" select="string('NULL_PTR')"/>
            </xsl:call-template>
          </xsl:when>
          <xsl:otherwise>
            <xsl:call-template name="td-entry"/>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:for-each>
      <xsl:call-template name="directive">
        <xsl:with-param name="DIRECTIVE" select="string('.ENDLITERAL')"/>
      </xsl:call-template>
    </xsl:if>
  </xsl:template>
  <xsl:template name="hid-class-descr-td-table">
    <!-- Context: USB_CONFIGURATION -->
    <xsl:text>
;----------------------------------------------------------------------
; HID Class Descriptor transfer descriptor table
;----------------------------------------------------------------------
</xsl:text>
    <xsl:call-template name="area">
      <xsl:with-param name="AREA_NAME" select="string('usb_desc')"/>
    </xsl:call-template>
    <xsl:call-template name="export-label">
      <xsl:with-param name="LABEL" select="concat($INSTANCE,'_',../@NAME,'_', @NAME, '_HID_CLASS_DESCR_TABLE')"/>
      <xsl:with-param name="COMMENT" select="string(';')"/>
      <xsl:with-param name="LITERAL" select="string('TRUE')"/>
    </xsl:call-template>
    <xsl:call-template name="td-header">
      <xsl:with-param name="COUNT" select="count(child::USB_INTERFACE)"/>
      <xsl:with-param name="COMMENT" select="string('; Number of interfaces/HID Class Descriptors')"/>
    </xsl:call-template>
    <xsl:for-each select="USB_INTERFACE">
      <xsl:choose>
        <xsl:when test="count(child::USB_HID) > 0">
          <xsl:call-template name="td-entry">
            <xsl:with-param name="MEM" select="string('USB_DS_ROM')"/>
            <xsl:with-param name="ADDR" select="concat($INSTANCE,'_',../../@NAME,'_', ../@NAME,'_', @NAME,'_HID_DESCR_START')"/>
            <xsl:with-param name="SIZE" select="string('DESCR_SIZE_HID_CLASS')"/>
            <xsl:with-param name="PTR" select="string('NULL_PTR')"/>
          </xsl:call-template>
        </xsl:when>
        <xsl:otherwise>
          <xsl:call-template name="td-entry"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:for-each>
    <xsl:call-template name="directive">
      <xsl:with-param name="DIRECTIVE" select="string('.ENDLITERAL')"/>
    </xsl:call-template>
  </xsl:template>
  <xsl:template name="ifc-lookup-table">
    <!-- Context: USB_CONFIGURATION -->
    <xsl:text>
;----------------------------------------------------------------------
; Interface Lookup Table
; 
; This table is indexed by interface number.
;
;----------------------------------------------------------------------
</xsl:text>
    <xsl:call-template name="export-label">
      <xsl:with-param name="LABEL" select="concat($INSTANCE,'_',../@NAME,'_', @NAME, '_INTERFACE_RPT_LOOKUP')"/>
      <xsl:with-param name="COMMENT" select="string(';')"/>
      <xsl:with-param name="LITERAL" select="string('TRUE')"/>
    </xsl:call-template>
    <xsl:for-each select="USB_INTERFACE">
      <xsl:choose>
        <xsl:when test="count(child::USB_HID[@HID_REPORT != 0]) > 0">
          <xsl:for-each select="USB_HID">
            <xsl:call-template name="dw">
              <xsl:with-param name="CODE" select="concat($INSTANCE,'_',../../../@NAME,'_',../../@NAME,'_',../@NAME,       '_HID_RPT_LOOKUP')"/>
              <xsl:with-param name="COMMENT" select="string('')"/>
            </xsl:call-template>
          </xsl:for-each>
        </xsl:when>
        <xsl:otherwise>
          <xsl:call-template name="dw">
            <xsl:with-param name="CODE" select="0"/>
            <xsl:with-param name="COMMENT" select="string('; HID Class Reports not defined for this interface')"/>
          </xsl:call-template>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:for-each>
    <xsl:call-template name="directive">
      <xsl:with-param name="DIRECTIVE" select="string('.ENDLITERAL')"/>
    </xsl:call-template>
  </xsl:template>
  <xsl:template name="ifc-rpt-table">
    <xsl:if test="count(child::USB_HID[@HID_REPORT != 0]) > 0">
      <xsl:text>;----------------------------------------------------------------------
; HID Report Lookup Table for (</xsl:text>
      <xsl:value-of select="concat($INSTANCE,'_',../../@NAME,'_',../@NAME,'_',@NAME)"/>
      <xsl:text>)
;----------------------------------------------------------------------
</xsl:text>
<xsl:call-template name="area">
  <xsl:with-param name="AREA_NAME" select="string('usb_desc')"/>
</xsl:call-template>
      <xsl:call-template name="directive">
        <xsl:with-param name="DIRECTIVE" select="string('.LITERAL')"/>
      </xsl:call-template>
      <xsl:call-template name="label">
        <xsl:with-param name="LABEL" select="concat($INSTANCE,'_',../../@NAME,'_',../@NAME,'_',@NAME,'_HID_RPT_LOOKUP')"/>
      </xsl:call-template>
      <xsl:call-template name="lt-header">
        <xsl:with-param name="COUNT" select="4"/>
        <xsl:with-param name="COMMENT" select="string('; 4 Entries')"/>
      </xsl:call-template>
      <xsl:call-template name="rpt-lt-entry">
        <xsl:with-param name="RPT_TYPE" select="string('IN')"/>
        <xsl:with-param name="RPT_TAG" select="string('Input')"/>
      </xsl:call-template>
      <xsl:call-template name="rpt-lt-entry">
        <xsl:with-param name="RPT_TYPE" select="string('OUT')"/>
        <xsl:with-param name="RPT_TAG" select="string('Output')"/>
      </xsl:call-template>
      <xsl:call-template name="rpt-lt-entry">
        <xsl:with-param name="RPT_TYPE" select="string('FEATURE')"/>
        <xsl:with-param name="RPT_TAG" select="string('Feature')"/>
      </xsl:call-template>
      <xsl:call-template name="directive">
        <xsl:with-param name="DIRECTIVE" select="string('.ENDLITERAL')"/>
      </xsl:call-template>
      <xsl:text>;----------------------------------------------------------------------
</xsl:text>
      <xsl:for-each select="USB_HID">
        <xsl:if test="@HID_REPORT != 0">
          <xsl:call-template name="rpt-td-table">
            <xsl:with-param name="RPT_TYPE" select="string('IN')"/>
            <xsl:with-param name="RPT_TAG" select="string('Input')"/>
          </xsl:call-template>
          <xsl:call-template name="rpt-td-table">
            <xsl:with-param name="RPT_TYPE" select="string('OUT')"/>
            <xsl:with-param name="RPT_TAG" select="string('Input')"/>
          </xsl:call-template>
          <xsl:call-template name="rpt-td-table">
            <xsl:with-param name="RPT_TYPE" select="string('FEATURE')"/>
            <xsl:with-param name="RPT_TAG" select="string('Input')"/>
          </xsl:call-template>
        </xsl:if>
      </xsl:for-each>
    </xsl:if>
  </xsl:template>
  <!-- Context USB_INTERFACE -->
  <xsl:template name="rpt-lt-entry">
    <xsl:param name="RPT_TYPE" select="string('IN')"/>
    <xsl:param name="RPT_TAG" select="string('Input')"/>
<!-- Can we add a choose tag here to make a zero entry if there are no HID items fot RPT_TAG type?
-->
    <xsl:call-template name="lt-entry">
      <xsl:with-param name="ADDR" select="concat($INSTANCE,'_',../../@NAME,'_',../@NAME,'_',@NAME,'_',$RPT_TYPE, '_RPTS')"/>
      <xsl:with-param name="COMMENT" select="concat('; ', $RPT_TYPE,' Reports')"/>
    </xsl:call-template>
  </xsl:template>
  <!-- Context USB_HID -->
  <xsl:template name="rpt-td-table">
    <xsl:param name="RPT_TYPE" select="string('IN')"/>
    <xsl:param name="RPT_TAG" select="string('Input')"/>
    <xsl:text>; HID </xsl:text>
    <xsl:value-of select="$RPT_TYPE"/>
    <xsl:text> Report Transfer Descriptor Table for (</xsl:text>
    <xsl:value-of select="@NAME"/>
    <xsl:text>)
;----------------------------------------------------------------------
</xsl:text>
    <xsl:call-template name="directive">
      <xsl:with-param name="DIRECTIVE" select="string('IF WIZARD_DEFINED_REPORT_STORAGE')"/>
    </xsl:call-template>
    <xsl:call-template name="area">
      <xsl:with-param name="AREA_NAME" select="string('usb_desc')"/>
    </xsl:call-template>
    <xsl:call-template name="directive">
      <xsl:with-param name="DIRECTIVE" select="string('.LITERAL')"/>
    </xsl:call-template>
    <xsl:call-template name="label">
      <xsl:with-param name="LABEL" select="concat($INSTANCE,'_',../../../@NAME,'_',../../@NAME,'_',../@NAME,'_',$RPT_TYPE, '_RPTS')"/>
    </xsl:call-template>
    <xsl:call-template name="td-header">
      <xsl:with-param name="COUNT" select="1"/>
      <xsl:with-param name="COMMENT" select="string('; Only 1 Transfer Descriptor')"/>
    </xsl:call-template>
    <xsl:call-template name="td-entry">
      <xsl:with-param name="MEM" select="string('USB_DS_RAM')"/>
      <xsl:with-param name="ADDR" select="concat($INSTANCE,'_INTERFACE_',../@NUMBER,'_',$RPT_TYPE,'_RPT_DATA')"/>
      <xsl:with-param name="SIZE" select="concat($INSTANCE,'_HID_RPT_',@HID_REPORT,'_',$RPT_TYPE,'_RPT_SIZE')"/>
      <xsl:with-param name="PTR" select="string('NULL_PTR')"/>
    </xsl:call-template>
    <xsl:call-template name="directive">
      <xsl:with-param name="DIRECTIVE" select="string('.ENDLITERAL')"/>
    </xsl:call-template>
    <xsl:call-template name="directive">
      <xsl:with-param name="DIRECTIVE" select="string('ENDIF ; WIZARD_DEFINED_REPORT_STORAGE')"/>
    </xsl:call-template>
    <xsl:text>;----------------------------------------------------------------------
</xsl:text>
  </xsl:template>
  <!-- Context USB_ROOT -->
  <xsl:template name="report-ram">
    <xsl:param name="NUM" select="0"/>

    <xsl:if test="count(//USB_DEVICE/USB_CONFIGURATION/USB_INTERFACE[@NUMBER = string($NUM)]) > 0">
      <xsl:call-template name="interface-report-ram">
        <xsl:with-param name="NUM" select="$NUM"/>
      </xsl:call-template>
      <xsl:call-template name="report-ram">
        <xsl:with-param name="NUM" select="$NUM+1" />
      </xsl:call-template>
    </xsl:if>
  </xsl:template>
  <xsl:template name="interface-report-ram">
    <xsl:param name="NUM" select="0"/>
    <xsl:choose>
      <xsl:when test="count(//USB_DEVICE/USB_CONFIGURATION/USB_INTERFACE[@NUMBER = string($NUM)]/USB_HID) > 0">
        <xsl:text>;----------------------------------------------------------------------
; Interface </xsl:text>
        <xsl:value-of select="$NUM"/>
        <xsl:text> RAM Allocation (Reused across configurations)
;----------------------------------------------------------------------
</xsl:text>
    <xsl:call-template name="directive">
      <xsl:with-param name="DIRECTIVE" select="string('IF WIZARD_DEFINED_REPORT_STORAGE')"/>
    </xsl:call-template>
        <xsl:call-template name="area">
          <xsl:with-param name="AREA_NAME" select="string('InterruptRAM')"/>
          <xsl:with-param name="TYPE" select="string('RAM')"/>
          <xsl:with-param name="LOC" select="string('REL')"/>
          <xsl:with-param name="OVR" select="string('CON')"/>
        </xsl:call-template>
<xsl:text>;----------------------------------------------------------------------
</xsl:text>
        <xsl:call-template name="alloc-rpt-ram">
          <xsl:with-param name="RPT_TYPE" select="string('IN')"/>
          <xsl:with-param name="RPT_TAG" select="string('Input')"/>
          <xsl:with-param name="NUM" select="$NUM"/>
        </xsl:call-template>
        <xsl:call-template name="alloc-rpt-ram">
          <xsl:with-param name="RPT_TYPE" select="string('OUT')"/>
          <xsl:with-param name="RPT_TAG" select="string('Output')"/>
          <xsl:with-param name="NUM" select="$NUM"/>
        </xsl:call-template>
        <xsl:call-template name="alloc-rpt-ram">
          <xsl:with-param name="RPT_TYPE" select="string('FEATURE')"/>
          <xsl:with-param name="RPT_TAG" select="string('Feature')"/>
          <xsl:with-param name="NUM" select="$NUM"/>
        </xsl:call-template>
    <xsl:call-template name="directive">
      <xsl:with-param name="DIRECTIVE" select="string('ENDIF  ; WIZARD_DEFINED_REPORT_STORAGE')"/>
    </xsl:call-template>
       </xsl:when>
    </xsl:choose>
  </xsl:template>
  <xsl:template name="alloc-rpt-ram">
    <xsl:param name="RPT_TYPE" select="string('IN')"/>
    <xsl:param name="RPT_TAG" select="string('Input')"/>
    <xsl:param name="NUM" select="0"/>
    <xsl:call-template name="comment">
      <xsl:with-param name="COMMENT" select="concat('; RAM allocation for ',$RPT_TYPE,' Reports.')"/>
    </xsl:call-template>
    <xsl:text>;----------------------------------------------------------------------
</xsl:text>
    <xsl:call-template name="export">
      <xsl:with-param name="LABEL" select="concat('_',$INSTANCE,'_INTERFACE_',$NUM,'_',      $RPT_TYPE,'_RPT_DATA')"/>
    </xsl:call-template>
    <xsl:call-template name="export">
      <xsl:with-param name="LABEL" select="concat($INSTANCE,'_INTERFACE_',$NUM,'_',      $RPT_TYPE,'_RPT_DATA')"/>
    </xsl:call-template>
    <xsl:call-template name="label">
      <xsl:with-param name="LABEL" select="concat('_',$INSTANCE,'_INTERFACE_',$NUM,'_',      $RPT_TYPE,'_RPT_DATA')"/>
    </xsl:call-template>
    <xsl:call-template name="label">
      <xsl:with-param name="LABEL" select="concat($INSTANCE,'_INTERFACE_',$NUM,'_',      $RPT_TYPE,'_RPT_DATA')"/>
    </xsl:call-template>
    <xsl:call-template name="alloc-ram">
      <xsl:with-param name="SIZE" select="8"/>
      <xsl:with-param name="COMMENT" select="string(';')"/>
    </xsl:call-template>
    <xsl:text>
;----------------------------------------------------------------------
</xsl:text>
  </xsl:template>

<xsl:template name="ep-direction">
<xsl:param name="EP" select="string('EP1')"/>
      <xsl:choose>
        <xsl:when test="count(child::USB_INTERFACE/USB_ENDPOINT[@ADDRESS = $EP]) = 1">
          <xsl:for-each select="USB_INTERFACE/USB_ENDPOINT[@ADDRESS = $EP]">
            <xsl:call-template name="db">
              <xsl:with-param name="CODE" select="concat('USB_DIR_', @DIRECTION)"/>
              <xsl:with-param name="COMMENT" select="concat('; Endpoint ',$EP, '(',@DIRECTION,')')"/>
            </xsl:call-template>
          </xsl:for-each>
        </xsl:when>
        <xsl:when test="count(child::USB_INTERFACE/USB_ENDPOINT[@ADDRESS = $EP]) = 0">
          <xsl:call-template name="db">
            <xsl:with-param name="CODE" select="string('USB_DIR_UNUSED')"/>
            <xsl:with-param name="COMMENT" select="concat('; Endpoint ',$EP, ' not used for this configuration')"/>
          </xsl:call-template>
        </xsl:when>
        <xsl:otherwise>
          <xsl:call-template name="db">
            <xsl:with-param name="CODE" select="0"/>
            <xsl:with-param name="COMMENT" select="concat('; INVALID Number of Endpoint Descriptors ',' ')"/>
          </xsl:call-template>
        </xsl:otherwise>
      </xsl:choose>
</xsl:template>

</xsl:stylesheet>
