<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:fo="http://www.w3.org/1999/XSL/Format">
  <xsl:include href="cGenPrim.xsl"/>
  <xsl:output method="text" omit-xml-declaration="yes" encoding="UTF-8"/>
  <xsl:param name="INSTANCE" select="/descendant::HID_REPORT_ROOT/@NAME"/>
  <xsl:template match="/">
    <xsl:apply-templates/>
  </xsl:template>
  <xsl:template match="HID_REPORT_ROOT">
    <xsl:text>;----------------------------------------------------------------------
; HID Class Report Descriptors
;
; This section contains the HID Class Report Descriptors generated
; by the USB User Module Descriptor Generator
;
;----------------------------------------------------------------------
</xsl:text>
    <xsl:for-each select="HID_REPORT">
      <xsl:call-template name="report-descr"/>
    </xsl:for-each>
    <xsl:for-each select="HID_REPORT">
      <xsl:call-template name="report-size"/>
    </xsl:for-each>
  </xsl:template>



  <!-- Context HID_REPORT -->
  <xsl:template name="report-descr">
    <xsl:text>; HID Report Descriptor (HID_RPT_</xsl:text>
    <xsl:value-of select="@HASH"/>
    <xsl:text>}
;----------------------------------------------------------------------
</xsl:text>
    <xsl:call-template name="area"/>
    <xsl:call-template name="directive">
      <xsl:with-param name="DIRECTIVE" select="string('.LITERAL')"/>
    </xsl:call-template>
    <xsl:call-template name="label">
      <xsl:with-param name="LABEL" select="concat($INSTANCE, '_HID_REPORT_', @HASH, '_DESCR_START')"/>
      <xsl:with-param name="COMMENT" select="string(';')"/>
    </xsl:call-template>
    <xsl:apply-templates/>
    <xsl:call-template name="label">
      <xsl:with-param name="LABEL" select="concat($INSTANCE, '_HID_REPORT_', @HASH,  '_DESCR_END')"/>
      <xsl:with-param name="COMMENT" select="string(';')"/>
    </xsl:call-template>
    <xsl:call-template name="equ">
      <xsl:with-param name="LABEL" select="concat($INSTANCE,'_HID_REPORT_', @HASH,'_DESCR_SIZE')"/>
      <xsl:with-param name="SIZE" select="concat('(',$INSTANCE,'_HID_REPORT_', @HASH,'_DESCR_END - ',$INSTANCE, '_HID_REPORT_', @HASH, '_DESCR_START)')"/>
    </xsl:call-template>
    <xsl:call-template name="directive">
      <xsl:with-param name="DIRECTIVE" select="string('.ENDLITERAL')"/>
    </xsl:call-template>
    <xsl:text>;----------------------------------------------------------------------
; End HID Report Descriptor (HID_RPT_</xsl:text>
    <xsl:value-of select="@HASH"/>
    <xsl:text>}</xsl:text>
    <xsl:text>&#10;;----------------------------------------------------------------------&#10;</xsl:text>
  </xsl:template>

  <xsl:template match="HID_ITEM">
    <xsl:value-of select="string('    DB     ')"/>
    <xsl:call-template name="to-hex-byte">
      <xsl:with-param name="DEC" select="@ITEM_CODE + @SIZE"/>
    </xsl:call-template>
    <xsl:text>H</xsl:text>
    <xsl:apply-templates select="ITEM_DATA"/>
    <xsl:value-of select="substring($BLANKS,0,$CMT_COL - (count(ITEM_DATA)*5) - 14)"/>; <xsl:value-of select="@ITEM_TYPE"/>
    <xsl:text>&#10;</xsl:text>
    <xsl:apply-templates select="HID_ITEM"/>
  </xsl:template>
  <xsl:template match="ITEM_DATA">
    <xsl:text>, </xsl:text>
    <xsl:value-of select="@VALUE"/>
    <xsl:text>H</xsl:text>
  </xsl:template>

  <xsl:template name="rpt-ram-size">
    <xsl:param name="RPT_TYPE" select="string('IN')"/>
    <xsl:param name="RPT_TAG" select="string('Input')"/>
    <xsl:choose>
      <xsl:when test="count(//HID_ITEM[@ITEM_TYPE=$RPT_TAG])> 0">
        <xsl:call-template name="equ">
          <xsl:with-param name="LABEL" select="concat($INSTANCE,'_HID_RPT_',@HASH,'_',      $RPT_TYPE,'_RPT_SIZE')"/>
          <xsl:with-param name="SIZE" select="8"/>
          <xsl:with-param name="COMMENT" select="string('; TODO: Optimize report size')"/>
        </xsl:call-template>
      </xsl:when>
      <xsl:otherwise>
        <xsl:call-template name="equ">
          <xsl:with-param name="LABEL" select="concat($INSTANCE,'_HID_RPT_',@HASH,'_',      $RPT_TYPE,'_RPT_SIZE')"/>
          <xsl:with-param name="SIZE" select="0"/>
          <xsl:with-param name="COMMENT" select="concat('; No ',$RPT_TYPE,' Reports are defined.  Size set to 0.')"/>
        </xsl:call-template>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:text>;----------------------------------------------------------------------
</xsl:text>
  </xsl:template>

  <!-- Context HID_REPORT -->
  <xsl:template name="report-size">
    <xsl:text>; HID Report Sizes (HID_RPT_</xsl:text>
    <xsl:value-of select="@HASH"/>
    <xsl:text>}
;----------------------------------------------------------------------
</xsl:text>
    <xsl:call-template name="rpt-ram-size">
      <xsl:with-param name="RPT_TYPE" select="string('IN')"/>
      <xsl:with-param name="RPT_TAG" select="string('Input')"/>
    </xsl:call-template>
    <xsl:call-template name="rpt-ram-size">
      <xsl:with-param name="RPT_TYPE" select="string('OUT')"/>
      <xsl:with-param name="RPT_TAG" select="string('Output')"/>
    </xsl:call-template>
    <xsl:call-template name="rpt-ram-size">
      <xsl:with-param name="RPT_TYPE" select="string('FEATURE')"/>
      <xsl:with-param name="RPT_TAG" select="string('Feature')"/>
    </xsl:call-template>
</xsl:template>

</xsl:stylesheet>
