<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:fo="http://www.w3.org/1999/XSL/Format">
<xsl:include href="cGenPrim.xsl"/>
<xsl:output method="text" omit-xml-declaration="yes" encoding="UTF-8" />	
<xsl:param name="INSTANCE" select="/descendant::STRING_ROOT/@INSTANCE_NAME"/>
<xsl:template match="/">
<xsl:choose>
  <xsl:when test="count(child::STRING_ROOT/USB_STRING_DESCR) > 0">
<xsl:text>;----------------------------------------------------------------------
; USB String Descriptors
;
; This section contains the USB String Descriptors generated
; by the USB User Module Descriptor Generator
;
; Note:  The string labels are internally generated by the
; descriptor generator
;
; Descriptors that reference string descriptors, use a hashed
; symbol that is set in an EQU directive with each string
; descriptor.
;----------------------------------------------------------------------
</xsl:text>
<xsl:call-template name="equ">
<xsl:with-param name="LABEL" select="string('STR_HASH_0')"/>
<xsl:with-param name="SIZE"  select="0"/>
<xsl:with-param name="COMMENT" select="string('; String Hash for the null string')"/>
</xsl:call-template>
  <xsl:apply-templates/>
  </xsl:when>
  <xsl:otherwise>
<xsl:text>;----------------------------------------------------------------------
; USB String Descriptors
;
; No string descriptors were entered into the string descriptor
; section of the USB Setup wizard.
;----------------------------------------------------------------------
</xsl:text>
<xsl:call-template name="equ">
<xsl:with-param name="LABEL" select="string('STR_HASH_0')"/>
<xsl:with-param name="SIZE"  select="0"/>
<xsl:with-param name="COMMENT" select="string('; String Hash for the null string')"/>
</xsl:call-template>
  </xsl:otherwise>
</xsl:choose>
</xsl:template>
<xsl:template match="STRING_ROOT">
<xsl:call-template name="area">
<xsl:with-param name="AREA_NAME" select="string('usb_desc')"/>
<xsl:with-param name="TYPE" select="string('ROM')"/>
<xsl:with-param name="LOC"  select="string('ABS')"/>
<xsl:with-param name="OVR"  select="string('CON')"/>
</xsl:call-template>
<xsl:call-template name="export">
<xsl:with-param name="LABEL" select="concat($INSTANCE, '_StringTable')"/>
</xsl:call-template>
<xsl:call-template name="directive">
<xsl:with-param name="DIRECTIVE" select="string('.LITERAL')"/>
</xsl:call-template>
<xsl:call-template name="label">
<xsl:with-param name="LABEL" select="concat($INSTANCE, '_StringTable')"/>
</xsl:call-template>
<xsl:call-template name="td-header">
	<xsl:with-param name="COUNT" select="count(child::USB_STRING_DESCR) + 1"/>
	<xsl:with-param name="COMMENT" select="string('; Number of USB Strings')"/>
</xsl:call-template>
<xsl:call-template name="td-entry">
	<xsl:with-param name="MEM" select="string('USB_DS_ROM')"/>
	<xsl:with-param name="ADDR" select="string('LANGID_DESCR_START')"/>
	<xsl:with-param name="SIZE" select="string('LANGID_DESCR_SIZE')"/>
	<xsl:with-param name="PTR" select="string('NULL_PTR')"/>
</xsl:call-template>
<xsl:for-each select="USB_STRING_DESCR">
	<xsl:call-template name="td-entry">
		<xsl:with-param name="MEM" select="string('USB_DS_ROM')"/>
		<xsl:with-param name="ADDR" select="concat('STRING_',@HASH,'_DESCR_START')"/>
		<xsl:with-param name="SIZE" select="concat('STRING_',@HASH,'_DESCR_SIZE')"/>
		<xsl:with-param name="PTR" select="string('NULL_PTR')"/>
	</xsl:call-template>
</xsl:for-each>
<xsl:for-each select="USB_LANGID_DESCR">
<xsl:call-template name="USB_LANGID_DESCR"/>
</xsl:for-each>
<xsl:for-each select="USB_STRING_DESCR">
<xsl:call-template name="USB_STRING_DESCR"/>
</xsl:for-each>
<xsl:call-template name="directive">
<xsl:with-param name="DIRECTIVE" select="string('.ENDLITERAL')"/>
</xsl:call-template>
</xsl:template>

<xsl:template name="USB_LANGID_DESCR">
<xsl:text>;----------------------------------------------------------------------
; LANGID Descriptor
;----------------------------------------------------------------------
</xsl:text>
<xsl:call-template name="label">
<xsl:with-param name="LABEL" select="string('LANGID_DESCR_START')"/>
</xsl:call-template>
<xsl:call-template name="db">
  <xsl:with-param name="CODE" select="string('((1 * 2) + 2)')"/>
  <xsl:with-param name="COMMENT" select="string('; bLength (N+2)')"/>
</xsl:call-template>
<xsl:call-template name="db">
  <xsl:with-param name="CODE" select="string('3')"/>
  <xsl:with-param name="COMMENT" select="string('; bDescriptorType (STRING)')"/>
</xsl:call-template>
<xsl:call-template name="dwl">
  <xsl:with-param name="CODE" select="@LANGID_VALUE"/>
  <xsl:with-param name="COMMENT" select="string(';')"/>
</xsl:call-template>
<xsl:call-template name="label">
<xsl:with-param name="LABEL" select="string('LANGID_DESCR_END')"/>
</xsl:call-template>
<xsl:call-template name="equ">
<xsl:with-param name="LABEL" select="string('LANGID_DESCR_SIZE')"/>
<xsl:with-param name="SIZE"  select="concat('(LANGID_DESCR_END', ' - ','LANGID_DESCR_START)' )"/>
</xsl:call-template>
</xsl:template>

<xsl:template name="USB_STRING_DESCR">
<xsl:text>;----------------------------------------------------------------------
; String Desciptor
;----------------------------------------------------------------------
</xsl:text>
<xsl:call-template name="equ">
<xsl:with-param name="LABEL" select="concat('STR_HASH_',@HASH)"/>
<xsl:with-param name="SIZE"  select="position()"/>
<xsl:with-param name="COMMENT" select="string('; String Hash')"/>
</xsl:call-template>
<xsl:call-template name="label">
<xsl:with-param name="LABEL" select="concat('STRING_', @HASH, '_DESCR_START')"/>
</xsl:call-template>
<xsl:call-template name="db">
  <xsl:with-param name="CODE" select="concat('STRING_', @HASH, '_DESCR_SIZE')"/>
  <xsl:with-param name="COMMENT" select="string('; bLength')"/>
</xsl:call-template>
<xsl:call-template name="db">
  <xsl:with-param name="CODE" select="string('3')"/>
  <xsl:with-param name="COMMENT" select="string('; bDescriptorType (STRING)')"/>
</xsl:call-template>
<xsl:call-template name="dsu">
  <xsl:with-param name="STRING" select="@STRING_VALUE"/>
  <xsl:with-param name="COMMENT" select="string(';')"/>
</xsl:call-template>
<xsl:call-template name="label">
<xsl:with-param name="LABEL" select="concat('STRING_', @HASH, '_DESCR_END')"/>
</xsl:call-template>
<xsl:call-template name="equ">
<xsl:with-param name="LABEL" select="concat('STRING_', @HASH, '_DESCR_SIZE')"/>
<xsl:with-param name="SIZE"  select="concat('(STRING_', @HASH, '_DESCR_END', ' - ','STRING_', @HASH, '_DESCR_START)' )"/>
</xsl:call-template>
</xsl:template>

</xsl:stylesheet>
