<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:fo="http://www.w3.org/1999/XSL/Format">
  <xsl:output method="text" omit-xml-declaration="yes" encoding="UTF-8"/>
<xsl:param name="CMT_COL" select="50"/>
<xsl:param name="BLANKS"
select="string('                                                              ')"/>
<xsl:param name="SUFFIX" select="string(' ')"/>

<!-- Generate an area statement -->
<xsl:template name="area">
<xsl:param name="AREA_NAME" select="string('usb_desc')"/>
<xsl:param name="TYPE" select="string('ROM')"/>
<xsl:param name="LOC"  select="string('ABS')"/>
<xsl:param name="OVR"  select="string('CON')"/>AREA  <xsl:value-of select="$AREA_NAME"/>     (<xsl:value-of select="$TYPE"/>,<xsl:value-of select="$LOC"/>,<xsl:value-of select="$OVR"/>)
</xsl:template>

<!-- Allocate a block of RAM-->
<xsl:template name="alloc-ram">
<xsl:param name="SIZE" select="1"/>
<xsl:param name="COMMENT" select="string('')"/>
<xsl:value-of select="string('  BLK  ')"/><xsl:value-of select="$SIZE"/><xsl:value-of select="substring($BLANKS,0,$CMT_COL - string-length($SIZE)-7)"/><xsl:value-of select="$COMMENT"/>
</xsl:template>

<!-- Generate an exported label -->
<xsl:template name="export-label">
<xsl:param name="LABEL" select="string('')"/>
<xsl:param name="COMMENT" select="string('')"/>
<xsl:param name="LITERAL" select="string('FALSE')"/>
<xsl:text>EXPORT </xsl:text>
<xsl:value-of select="$LABEL"/>
<xsl:text>&#10;</xsl:text>
<xsl:if test="$LITERAL = string('TRUE')">
    <xsl:call-template name="directive">
      <xsl:with-param name="DIRECTIVE" select="string('.LITERAL')"/>
    </xsl:call-template>
</xsl:if>
<xsl:call-template name="label">
<xsl:with-param name="LABEL" select="$LABEL"/>
<xsl:with-param name="COMMENT" select="$COMMENT"/>
</xsl:call-template>
</xsl:template>

<!-- Generate an exported-->
<xsl:template name="export">
<xsl:param name="LABEL" select="string('')"/>
<xsl:text>EXPORT </xsl:text>
<xsl:value-of select="$LABEL"/>
<xsl:text>&#10;</xsl:text>
</xsl:template>
<!-- Generate an single directive -->
<xsl:template name="directive">
<xsl:param name="DIRECTIVE" select="string('')"/>
<xsl:value-of select="$DIRECTIVE"/>
<xsl:text>&#10;</xsl:text>
</xsl:template>

<xsl:template name="label">
<xsl:param name="LABEL" select="string('')"/>
<xsl:param name="COMMENT" select="string('')"/>
<xsl:param name="LITERAL" select="string('FALSE')"/>
<xsl:if test="$LITERAL = string('TRUE')">
    <xsl:call-template name="directive">
      <xsl:with-param name="DIRECTIVE" select="string('.LITERAL')"/>
    </xsl:call-template>
</xsl:if>
<xsl:value-of select="$LABEL"/>
<xsl:text>:</xsl:text>
<xsl:call-template name="ctab">
<xsl:with-param name="CUR_POS" select="string-length($LABEL) + 1"/>
</xsl:call-template>
<xsl:call-template name="comment">
<xsl:with-param name="COMMENT" select="$COMMENT"/>
</xsl:call-template>
</xsl:template>

<!-- Generate a transfer descriptor table header -->
<xsl:template name="td-header">
<xsl:param name="COUNT" select="string('0')"/>
<xsl:param name="COMMENT" select="string('')"/>
<xsl:value-of select="string('  TD_START_TABLE ')"/>
<xsl:value-of select="$COUNT"/>
<xsl:call-template name="ctab">
<xsl:with-param name="CUR_POS" select="string-length($COUNT) + string-length('  TD_START_TABLE ')"/>
</xsl:call-template>
<xsl:call-template name="comment">
<xsl:with-param name="COMMENT" select="$COMMENT"/>
</xsl:call-template>
</xsl:template>
<!-- Generate a transfer descriptor table entry -->
<xsl:template name="td-entry">
  <xsl:param name="MEM"  select="string('USB_DS_ROM')"/>
  <xsl:param name="ADDR" select="string('NULL_PTR')"/>
  <xsl:param name="SIZE" select="string('0')"/>
  <xsl:param name="PTR"  select="string('NULL_PTR')"/>
<xsl:value-of select="string('  TD_ENTRY       ')"/><xsl:value-of select="$MEM"/>, <xsl:value-of select="$SIZE"/>, <xsl:value-of select="$ADDR"/>, <xsl:value-of select="$PTR"/>
<xsl:text>&#10;</xsl:text>
</xsl:template>

<!-- Generate a look up table header -->
<xsl:template name="lt-header">
<xsl:param name="COUNT" select="string('0')"/>
<xsl:param name="COMMENT" select="string('')"/>
<xsl:value-of select="string('  LT_START    ')"/>
<xsl:value-of select="$COUNT"/>
<xsl:call-template name="ctab">
<xsl:with-param name="CUR_POS" select="string-length($COUNT) + string-length('  LT_START    ')"/>
</xsl:call-template>
<xsl:call-template name="comment">
<xsl:with-param name="COMMENT" select="$COMMENT"/>
</xsl:call-template>
</xsl:template>

<!-- Generate a look up table entry -->
<xsl:template name="lt-entry">
  <xsl:param name="ADDR" select="string('NULL_PTR')"/>
  <xsl:param name="DPTR" select="string('NULL_PTR')"/>
  <xsl:param name="COMMENT" select="string('')"/>
<xsl:value-of select="string('  LT_ENTRY    ')"/><xsl:value-of select="$ADDR"/>, <xsl:value-of select="$DPTR"/>
<xsl:call-template name="ctab">
<xsl:with-param name="CUR_POS" select="string-length('  LT_ENTRY    ') + string-length($ADDR) + string-length($DPTR) + 2"/>
</xsl:call-template>
<xsl:call-template name="comment">
<xsl:with-param name="COMMENT" select="$COMMENT"/>
</xsl:call-template>
</xsl:template>

<!-- Generate an blk -->
<xsl:template name="blk">
<xsl:param name="LABEL"/>
<xsl:param name="SIZE"/>
<xsl:param name="COMMENT" select="string('')"/>
<xsl:value-of select="$LABEL"/>
<xsl:value-of select="string(':  BLK    ')"/>
<xsl:value-of select="$SIZE"/>
<xsl:call-template name="ctab">
<xsl:with-param name="CUR_POS" select="string-length($LABEL) + string-length(':  BLK    ') + string-length($SIZE)"/>
</xsl:call-template>
<xsl:call-template name="comment">
<xsl:with-param name="COMMENT" select="$COMMENT"/>
</xsl:call-template>
</xsl:template>

<!-- Generate an equ -->
<xsl:template name="equ">
<xsl:param name="LABEL"/>
<xsl:param name="SIZE"/>
<xsl:param name="COMMENT" select="string('')"/>
<xsl:value-of select="$LABEL"/>
<xsl:value-of select="string(':  EQU    ')"/>
<xsl:value-of select="$SIZE"/>
<xsl:call-template name="ctab">
<xsl:with-param name="CUR_POS" select="string-length($LABEL) + string-length(':  EQU    ') + string-length($SIZE)"/>
</xsl:call-template>
<xsl:call-template name="comment">
<xsl:with-param name="COMMENT" select="$COMMENT"/>
</xsl:call-template>
</xsl:template>

<!-- Generate an DB -->
<xsl:template name="db">
  <xsl:param name="CODE"/>
  <xsl:param name="COMMENT" select="string('')"/>
  <xsl:param name="SUFFIX" select="string(' ')"/>
<xsl:value-of select="string('  DB    ')"/>
<xsl:value-of select="$CODE"/><xsl:value-of select="$SUFFIX"/>
<xsl:call-template name="ctab">
<xsl:with-param name="CUR_POS" select="string-length($CODE) + string-length('  DB    ') + 1"/>
</xsl:call-template>
<xsl:call-template name="comment">
<xsl:with-param name="COMMENT" select="$COMMENT"/>
</xsl:call-template>
</xsl:template>

<!-- Generate an DW -->
<xsl:template name="dw">
  <xsl:param name="CODE"/>
  <xsl:param name="COMMENT" select="string('')"/>
  <xsl:param name="SUFFIX" select="string(' ')"/>
<xsl:value-of select="string('  DW    ')"/>
<xsl:value-of select="$CODE"/><xsl:value-of select="$SUFFIX"/>
<xsl:call-template name="ctab">
<xsl:with-param name="CUR_POS" select="string-length($CODE) + string-length('  DW    ') + 1"/>
</xsl:call-template>
<xsl:call-template name="comment">
<xsl:with-param name="COMMENT" select="$COMMENT"/>
</xsl:call-template>
</xsl:template>

<!-- Generate an DWL -->
<xsl:template name="dwl">
  <xsl:param name="CODE"/>
  <xsl:param name="COMMENT" select="string('')"/>
  <xsl:param name="SUFFIX" select="string(' ')"/>
<xsl:value-of select="string('  DWL   ')"/>
<xsl:value-of select="$CODE"/><xsl:value-of select="$SUFFIX"/>
<xsl:call-template name="ctab">
<xsl:with-param name="CUR_POS" select="string-length($CODE) + string-length('  DWL   ') + 1"/>
</xsl:call-template>
<xsl:call-template name="comment">
<xsl:with-param name="COMMENT" select="$COMMENT"/>
</xsl:call-template>
</xsl:template>

<!-- Generate an DSU -->
<xsl:template name="dsu">
  <xsl:param name="STRING"/>
  <xsl:param name="COMMENT" select="string('')"/>
<xsl:value-of select="string('  DSU   ')"/>
<xsl:text>"</xsl:text>
<xsl:value-of select="$STRING"/>
<xsl:text>"</xsl:text>
<xsl:call-template name="ctab">
<xsl:with-param name="CUR_POS" select="string-length($STRING) + string-length('  DSU   ') + 2"/>
</xsl:call-template>
<xsl:call-template name="comment">
<xsl:with-param name="COMMENT" select="$COMMENT"/>
</xsl:call-template>
</xsl:template>


<!-- Generate a comment -->
<xsl:template name="comment">
  <xsl:param name="COMMENT" select="string(';')"/>
<xsl:value-of select="$COMMENT"/>
<xsl:text>&#10;</xsl:text>
</xsl:template>

<xsl:template name="to-hex-byte">
<xsl:param name="DEC"/>
<xsl:choose>
  <xsl:when test=" 16 >= $DEC">
    <xsl:value-of select="0"/>
    <xsl:call-template name="to-hex">
      <xsl:with-param name="DEC" select="$DEC mod 16"/>
    </xsl:call-template>
  </xsl:when>
  <xsl:otherwise>
  <xsl:call-template name="to-hex">
    <xsl:with-param name="DEC" select="$DEC"/>
  </xsl:call-template>
</xsl:otherwise>
</xsl:choose>
</xsl:template>

<xsl:template name="to-hex-word">
<xsl:param name="DEC"/>
<xsl:choose>
  <xsl:when test="16 >= $DEC">
    <xsl:value-of select="000"/>
    <xsl:call-template name="to-hex">
      <xsl:with-param name="DEC" select="$DEC mod 16"/>
    </xsl:call-template>
  </xsl:when>
  <xsl:when test="256 >= $DEC">
    <xsl:value-of select="000"/>
    <xsl:call-template name="to-hex">
      <xsl:with-param name="DEC" select="$DEC mod 16"/>
    </xsl:call-template>
  </xsl:when>
  <xsl:when test="2048 >= $DEC">
    <xsl:value-of select="000"/>
    <xsl:call-template name="to-hex">
      <xsl:with-param name="DEC" select="$DEC mod 16"/>
    </xsl:call-template>
  </xsl:when>
  <xsl:otherwise>
  <xsl:call-template name="to-hex">
    <xsl:with-param name="DEC" select="$DEC"/>
  </xsl:call-template>
</xsl:otherwise>
</xsl:choose>
</xsl:template>


<xsl:variable name="HEXCHARS" select="string('0123456789ABCDEF')"/>
<xsl:template name="to-hex">
<xsl:param name="DEC"/>
<xsl:choose>
  <xsl:when test="($DEC div 16) >= 1">
    <xsl:call-template name="to-hex">
      <xsl:with-param name="DEC" select="(($DEC - ($DEC mod 16)) div 16)"/>
    </xsl:call-template>
    <xsl:value-of select="substring($HEXCHARS,($DEC mod 16) + 1,1)"/>
  </xsl:when>
  <xsl:otherwise>
    <xsl:value-of select="substring($HEXCHARS,($DEC mod 16) + 1,1)"/>
</xsl:otherwise>
</xsl:choose>
</xsl:template>

<xsl:template name="ctab">
<xsl:param name="CUR_POS"/>
<xsl:param name="CTAB" select="$CMT_COL"/>
<xsl:if test="$CTAB &gt; $CUR_POS">
<xsl:value-of select="substring($BLANKS,0,$CTAB - $CUR_POS)"/>
</xsl:if>
</xsl:template>
</xsl:stylesheet>
