;;*****************************************************************************
;;*****************************************************************************
;;  FILENAME: `@INSTANCE_NAME`_cls_hid.asm
;;   Version: 1.5, Updated on 2006/06/19 at 11:43:56
;;  `@PSOC_VERSION`
;;
;;  DESCRIPTION: USB Human Interface Device (HID) Class request implemenatation
;;               for the enCoRe II family of devices
;;
;;  NOTE: User Module APIs conform to the fastcall convention for marshalling
;;        arguments and observe the associated "Registers are volatile" policy.
;;        This means it is the caller's responsibility to preserve any values
;;        in the X and A registers that are still needed after the API
;;        function returns. Even though these registers may be preserved now,
;;        there is no guarantee they will be preserved in future releases.
;;-----------------------------------------------------------------------------
;;  Copyright (c) Cypress Semiconductor 2004. All Rights Reserved.
;;*****************************************************************************
;;*****************************************************************************

include "m8c.inc"
include "`@INSTANCE_NAME`_macros.inc"
include "`@INSTANCE_NAME`.inc"

;-----------------------------------------------
;  Global Symbols
;-----------------------------------------------
EXPORT `@INSTANCE_NAME`_UpdateHIDTimer
EXPORT _`@INSTANCE_NAME`_UpdateHIDTimer
EXPORT `@INSTANCE_NAME`_bGetProtocol
EXPORT _`@INSTANCE_NAME`_bGetProtocol

AREA usbRAM (RAM,REL)
;-----------------------------------------------
;  Constant Definitions
;-----------------------------------------------
;-----------------------------------------------
; Variable Allocation
;-----------------------------------------------
;----------------------------------------------------------------------------
; Interface Setting
;----------------------------------------------------------------------------
 `@INSTANCE_NAME`_IdleReload:`@CTAB40` BLK   `@DSHID_bNumInterfaces`h    ; Idle Timer Reload Value
 `@INSTANCE_NAME`_IdleTimer:`@CTAB40` BLK   `@DSHID_bNumInterfaces`h    ; Idle Timers
 `@INSTANCE_NAME`_Protocol:`@CTAB40` BLK   `@DSHID_bNumInterfaces`h    ; Active Protocol

AREA Bootloader (ROM, REL)
;-----------------------------------------------------------------------------
;  FUNCTION NAME: `@INSTANCE_NAME`_bGetProtocol
;
;  DESCRIPTION:   Returns the selected protocol value to the application
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:    A: Interface number
;
;  RETURNS:      A: Protocol values
;
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;  THEORY of OPERATION or PROCEDURE:
;
;-----------------------------------------------------------------------------
.SECTION
 `@INSTANCE_NAME`_bGetProtocol:
_`@INSTANCE_NAME`_bGetProtocol:
    MOV    X, A`@CTAB40`; Argument is the index
    MOV    A, [X + `@INSTANCE_NAME`_Protocol] ; Return the protocol
    RET
.ENDSECTION
;-----------------------------------------------------------------------------
;  FUNCTION NAME: `@INSTANCE_NAME`_UpdateHIDTimer
;
;  DESCRIPTION:    Updates the HID report timer and reloads it if it expires
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:     A: Interface number
;
;  RETURNS:       A: USB_IDLE_TIMER_EXPIRED, if the timer is running and expired
;                    USB_IDLE_TIMER_RUNNING, if the timer is running
;                    USB_IDLE_TIMER_INDEFINITE, if the report should be made on change
;
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;  THEORY of OPERATION or PROCEDURE:
;
;-----------------------------------------------------------------------------
.SECTION
 `@INSTANCE_NAME`_UpdateHIDTimer:
_`@INSTANCE_NAME`_UpdateHIDTimer:
    MOV    X, A`@CTAB40`; Make the argument the index
; Flow here to check if the timer is "indefinite"
    CMP    [X + `@INSTANCE_NAME`_IdleReload], 0`@CTAB40`; Indefinite?
    JZ     .indefinite`@CTAB40`; Jump if Indefinite?
; Flow here to check the timers
    DEC    [X + `@INSTANCE_NAME`_IdleTimer]`@CTAB40`; Decrement the timer
    JC     .expired
; Flow here if the timer has not expired
    MOV    A, USB_IDLE_TIMER_RUNNING`@CTAB40`; Return value (not expired)
    RET    `@CTAB40`; Quick exit
; Jump here if the timer expired
.expired:
    MOV    A, [X + `@INSTANCE_NAME`_IdleReload]`@CTAB40`; Reload the timer
    MOV    [X + `@INSTANCE_NAME`_IdleTimer], A`@CTAB40`; 
    MOV    A, USB_IDLE_TIMER_EXPIRED `@CTAB40`; Return value (expired)
    RET    `@CTAB40`; Quick exit
; Jump here to make return "on change/indefinite"
.indefinite:
    MOV    A, USB_IDLE_TIMER_INDEFINITE`@CTAB40`; Return value (change/indefinite)
    RET    `@CTAB40`; Exit
.ENDSECTION
;-----------------------------------------------------------------------------
;  FUNCTION NAME: `@INSTANCE_NAME`_CB_d2h_std_ifc_06
;
;  DESCRIPTION:   Get Interface Descriptor
;
;****************************************************************
; STANDARD INTERFACE IN REQUEST: Get_Interface_Descriptor
;****************************************************************
;
; bmRequestType   : (IN | STANDARD | INTERFACE)    = 81h
; bRequest        : GET_DESCRIPTOR                 = 06h    
; wValue          : DESCRIPTOR TYPE | INDEX        = xxxxh  
; wIndex          : INTERFACE                      = --xxh
; wLength         : DESCRIPTOR_LENGTH              = --xxh  
; 
; The GET_INTERFACE_DESCRIPTOR request returns the specified 
; descriptor if the descriptor exists. 
;
; The upper byte of request_value contains the descriptor type and 
; the lower byte contains the descriptor index. request_index 
; contains either 0000h or the Language ID. request_length contains 
; the descriptor length. The actual descriptor information is 
; transferred in subsequent data packets. 
;
; USB defines only a DEVICE recipient but the HID spec added 
; support for the INTERFACE recipient.
;
; Get Descriptor from an HID interface returns either HID, 
; REPORT, or PHYSICAL descriptors.
;
;****************************************************************
IF (USB_CB_SRC_d2h_std_ifc_06 & USB_UM_SUPPLIED)
export  `@INSTANCE_NAME`_CB_d2h_std_ifc_06
`@INSTANCE_NAME`_CB_d2h_std_ifc_06:
    CALL    `@INSTANCE_NAME`_GetInterfaceLookupTable `@CTAB40`; Point the the interface lookup table
    PUSH    A`@CTAB40`; Save the MSB
    MOV     A, REG[`@INSTANCE_NAME`_EP0DATA+wValueHi]`@CTAB40`; Get descriptor type
    CMP     A, DESCR_TYPE_HID_CLASS`@CTAB40`; HID Class descriptor?
    JZ      .send_hid_class_descr
    CMP     A, DESCR_TYPE_HID_REPORT`@CTAB40`; HID Report descriptor?
    JZ      .send_hid_report_descr
; Jump or flow here if the request is not supported
.not_supported:
    POP     A`@CTAB40`; Restore the stack
    JMP     `@INSTANCE_NAME`_Not_Supported_Local_Hid
; Jump here to send the HID Report Descriptor
.send_hid_report_descr:
    POP     A`@CTAB40`; Restore the interface lookup table MSB
    SWAP    A, X`@CTAB40`; Add the offset
    ADD     A, 2  `@CTAB40`; Point to the right table entry
    JMP     .finish
; Jump here to send the HID Class Descriptor
.send_hid_class_descr:
    POP     A`@CTAB40`; Restore the interface lookup table MSB
    SWAP    A, X`@CTAB40`; Add the offset
    ADD     A, 4  `@CTAB40`; Point to the right table entry
; Jump or flow here with A:X Pointing to the 
.finish:
    SWAP    A, X`@CTAB40`; Back where they belong
    ADC     A, 0`@CTAB40`; Don't forget the carry
    MOV     [`@INSTANCE_NAME`_t2],`@INSTANCE_NAME`_t1 `@CTAB40`; Set the GETWORD destination 
    LCALL   `@INSTANCE_NAME`_GETWORD `@CTAB40`; Get the pointer to the transfer descriptor table
                                     `@CTAB40`; ITempW has the address
; Get the interface number
    MOV     A, REG[`@INSTANCE_NAME`_EP0DATA+wIndexLo]`@CTAB40`; Get the interface number
    MOV     [`@INSTANCE_NAME`_t2], A`@CTAB40`; Save it for the call to LOOKUP
    MOV     A, [`@INSTANCE_NAME`_t1] `@CTAB40`; Get the transfer descriptor ROM Address MSB
    MOV     X, [`@INSTANCE_NAME`_t1+1] `@CTAB40`; Get the transfer descriptor ROM Address LSB

    JMP     `@INSTANCE_NAME`_GetTableEntry_Local_Hid
ENDIF
;-----------------------------------------------------------------------------
;  FUNCTION NAME: `@INSTANCE_NAME`_CB_d2h_cls_ifc_01
;
;  DESCRIPTION:   Get Report
;
;****************************************************************
; HID CLASS INTERFACE IN REQUEST: Get_Report   
;****************************************************************
;
; bmRequestType  : (IN | CLASS | INTERFACE)       = A1h
; bRequest       : GET_REPORT                     = 01h    
; wValue         : REPORT TYPE | REPORT ID        = xxxxh  
; wIndex         : INTERFACE                      = --xxh
; wLength        : REPORT LENGTH                  = --xxh  
; 
; The GET_REPORT request allows the host to receive a report from 
; a specific interface via the control pipe. 
;
;****************************************************************
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:
;
;  RETURNS:
;
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;  THEORY of OPERATION or PROCEDURE:
;
;-----------------------------------------------------------------------------
IF (USB_CB_SRC_d2h_cls_ifc_01 & USB_UM_SUPPLIED)
export  `@INSTANCE_NAME`_CB_d2h_cls_ifc_01
`@INSTANCE_NAME`_CB_d2h_cls_ifc_01:

    CALL    Find_Report
    NULL_PTR_CHECK `@INSTANCE_NAME`_Not_Supported_Local_Hid
    
    JMP     `@INSTANCE_NAME`_GetTableEntry_Local_Hid

ENDIF
;-----------------------------------------------------------------------------
;  FUNCTION NAME: `@INSTANCE_NAME`_CB_d2h_cls_ifc_02
;
;  DESCRIPTION:   Get Idle
;
;****************************************************************
; HID CLASS INTERFACE IN REQUEST: Get_Idle
;****************************************************************
;
; bmRequestType  : (OUT | CLASS | INTERFACE)      = A1h
; bRequest       : GET_IDLE                       = 02h    
; wValue         : REPORT ID                      = 00xxh  
; wIndex         : INTERFACE                      = --xxh
; wLength        : Report Size                    = 0001h  
; 
; The GET_IDLE request reads the current idle rate for a given 
; input report on a specific interface. 
;
;****************************************************************
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:
;
;  RETURNS:
;
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;  THEORY of OPERATION or PROCEDURE:
;
;-----------------------------------------------------------------------------
IF (USB_CB_SRC_d2h_cls_ifc_02 & USB_UM_SUPPLIED)
.LITERAL
GetSetIdleTable:
    TD_START_TABLE  `@DSHID_bNumInterfaces`h `@CTAB40`; One entry for each interface
    TD_ENTRY        USB_DS_RAM, 1, `@INSTANCE_NAME`_IdleReload,   NULL_PTR `@CTAB40`; Reuse the transfer buffer
    TD_ENTRY        USB_DS_RAM, 1, `@INSTANCE_NAME`_IdleReload+1, NULL_PTR `@CTAB40`; Reuse the transfer buffer
.ENDLITERAL
export  `@INSTANCE_NAME`_CB_d2h_cls_ifc_02
`@INSTANCE_NAME`_CB_d2h_cls_ifc_02:
    MOV     A, REG[`@INSTANCE_NAME`_EP0DATA+wValueLo]`@CTAB40`; Get the report number
    CMP     A, 0 `@CTAB40`; We don't support report by report idle
    JNZ     `@INSTANCE_NAME`_Not_Supported_Local_Hid

    MOV     A, REG[`@INSTANCE_NAME`_EP0DATA+wIndexLo]`@CTAB40`; Get the interface number
    CMP     A, `@DSHID_bNumInterfaces`h `@CTAB40`; We don't support report by report idle
    JNC     `@INSTANCE_NAME`_Not_Supported_Local_Hid

    MOV     [`@INSTANCE_NAME`_t2], A `@CTAB40`; Use the UM temp var--Selector
    MOV     A,>GetSetIdleTable `@CTAB40`; Get the ROM Address MSB
    MOV     X,<GetSetIdleTable `@CTAB40`; Get the ROM Address LSB
    
    JMP     `@INSTANCE_NAME`_GetTableEntry_Local_Hid
ENDIF
;-----------------------------------------------------------------------------
;  FUNCTION NAME: `@INSTANCE_NAME`_CB_d2h_cls_ifc_03
;
;  DESCRIPTION:   Get Protocol
;
;****************************************************************
; HID CLASS INTERFACE IN REQUEST: Get_Protocol
;****************************************************************
;
; bmRequestType  : (OUT | CLASS | INTERFACE)      = A1h
; bRequest       : GET_PROTOCOL                   = 03h    
; wValue         : RESERVED                       = 0000h  
; wIndex         : INTERFACE                      = --xxh
; wLength        : SIZEOF_INTERFACE_PROTOCOL      = 0001h  
; 
; The GET_PROTOCOL request reads which protocol is currently 
; active.
;
;****************************************************************
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:
;
;  RETURNS:
;
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;  THEORY of OPERATION or PROCEDURE:
;
;-----------------------------------------------------------------------------
IF (USB_CB_SRC_d2h_cls_ifc_03 & USB_UM_SUPPLIED)
.LITERAL
GetProtocolTable:
    TD_START_TABLE  2 `@CTAB40`; One entry for BOOT/One entry for REPORT
    TD_ENTRY        USB_DS_ROM, 1, ROM_ZERO,   NULL_PTR `@CTAB40`; Simply use a a hard coded zero or one
    TD_ENTRY        USB_DS_ROM, 1, ROM_ONE,    NULL_PTR `@CTAB40`; 
ROM_ZERO:   DB  0
ROM_ONE:    DB  1
.ENDLITERAL
export  `@INSTANCE_NAME`_CB_d2h_cls_ifc_03
`@INSTANCE_NAME`_CB_d2h_cls_ifc_03:
    MOV     A, REG[`@INSTANCE_NAME`_EP0DATA+wIndexLo] `@CTAB40`; Get the interface number
    CMP     A, `@DSHID_bNumInterfaces`h `@CTAB40`; Range check
    JNC     `@INSTANCE_NAME`_Not_Supported_Local_Hid

    MOV     X, A `@CTAB40`; Get the protocol for the requested interface
    MOV     A, [X + `@INSTANCE_NAME`_Protocol] `@CTAB40`; 

    MOV     [`@INSTANCE_NAME`_t2], A `@CTAB40`; Use the UM temp var--Selector

    MOV     A,>GetProtocolTable `@CTAB40`; Get the ROM Address MSB
    MOV     X,<GetProtocolTable `@CTAB40`; Get the ROM Address LSB
    
    JMP     `@INSTANCE_NAME`_GetTableEntry_Local_Hid
ENDIF
;-----------------------------------------------------------------------------
;  FUNCTION NAME: `@INSTANCE_NAME`_CB_h2d_cls_ifc_09
;
;  DESCRIPTION:   Set Report
;
;****************************************************************
; HID CLASS INTERFACE OUT REQUEST: Set_Report
;****************************************************************
;
; bmRequestType   : (OUT | CLASS | INTERFACE)      = 21h
; bRequest        : SET_REPORT                     = 09h    
; wValue          : REPORT TYPE | REPORT ID        = xxxxh  
; wIndex          : INTERFACE                      = --xxh
; wLength         : REPORT LENGTH                  = --xxh  
; 
; The SET_REPORT request allows the host to send a report to the 
; device, possibly setting the state of input, output or feature 
; controls. 
;
;****************************************************************
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:
;
;  RETURNS:
;
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;  THEORY of OPERATION or PROCEDURE:
;
;-----------------------------------------------------------------------------
IF (USB_CB_SRC_h2d_cls_ifc_09 & USB_UM_SUPPLIED)
export  `@INSTANCE_NAME`_CB_h2d_cls_ifc_09
`@INSTANCE_NAME`_CB_h2d_cls_ifc_09:
    CALL    Find_Report
    NULL_PTR_CHECK `@INSTANCE_NAME`_Not_Supported_Local_Hid
    
    JMP     `@INSTANCE_NAME`_GetTableEntry_Local_Hid
ENDIF
;-----------------------------------------------------------------------------
;  FUNCTION NAME: `@INSTANCE_NAME`_CB_h2d_cls_ifc_10
;
;  DESCRIPTION:   Set Idle
;
;****************************************************************
; HID CLASS INTERFACE OUT REQUEST: Set_Idle
;****************************************************************
;
; bmRequestType   : (OUT | CLASS | INTERFACE)      = 21h
; bRequest        : SET_IDLE                       = 0Ah    
; wValue          : DURATION | REPORT ID           = xxxxh  
; wIndex          : INTERFACE                      = --xxh
; wLength         : ZERO                           = 0000h  
; 
; The SET_IDLE request silences a particular input report (or all 
; input reports) on a specific interface until a new event occurs 
; or the specified amount of time passes. 
;
;****************************************************************
; Note: This function does not support multiple reports per interface.
;****************************************************************
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:
;
;  RETURNS:
;
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;  THEORY of OPERATION or PROCEDURE:
;
;-----------------------------------------------------------------------------
IF (USB_CB_SRC_h2d_cls_ifc_10 & USB_UM_SUPPLIED)
export  `@INSTANCE_NAME`_CB_h2d_cls_ifc_10
`@INSTANCE_NAME`_CB_h2d_cls_ifc_10:
    MOV     A, REG[`@INSTANCE_NAME`_EP0DATA+wValueLo] `@CTAB40`; Get the report number
    CMP     A, 0 `@CTAB40`; We don't support report by report idle
    JNZ     `@INSTANCE_NAME`_Not_Supported_Local_Hid

    MOV     A, REG[`@INSTANCE_NAME`_EP0DATA+wIndexLo] `@CTAB40`; Get the interface number
    CMP     A, `@DSHID_bNumInterfaces`h `@CTAB40`; Range Check
    JNC     `@INSTANCE_NAME`_Not_Supported_Local_Hid

    MOV     X, A `@CTAB40`; Interface Number becomes an index

    MOV     A, REG[`@INSTANCE_NAME`_EP0DATA+wValueHi] `@CTAB40`; Get the duration

    MOV     [X+`@INSTANCE_NAME`_IdleReload], A `@CTAB40`; Save the reload immediately
    CMP     A, 0 `@CTAB40`; Is this request setting the duration to indefinite?
    JZ      .reload `@CTAB40`; If so, reload the timer 

    ; Otherwise, we need to determine if we reset the current expiry
    ; (HID Spec says to send the next report if we are within 4 ms (1 count)
    ; of sending the next report
    CMP     [X+`@INSTANCE_NAME`_IdleTimer], 1 `@CTAB40`; Within 4 ms?
    JZ      .done `@CTAB40`; Jump to let the timer expire "naturally" 

; Jump or Flow here to reload the timer
.reload:
    MOV     [x+`@INSTANCE_NAME`_IdleTimer], A `@CTAB40`; Reload the timer
            
.done:
    JMP     `@INSTANCE_NAME`_NoDataStageControlTransfer
ENDIF
;-----------------------------------------------------------------------------
;  FUNCTION NAME: `@INSTANCE_NAME`_CB_h2d_cls_ifc_11
;
;  DESCRIPTION:   Set Idle
;
;****************************************************************
; HID CLASS INTERFACE OUT REQUEST: Set_Protocol
;****************************************************************
;
; bmRequestType  : (OUT | CLASS | INTERFACE)      = 21h
; bRequest       : SET_PROTOCOL                   = 0Bh    
; wValue         : DURATION | REPORT ID           = xxxxh  
; wIndex         : PROTOCOL                       = --xxh
; wLength        : ZERO                           = 0000h  
; 
; The SET_PROTOCOL request switches between the boot protocol and 
; the report protocol (or vice versa). 
;
;****************************************************************
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:
;
;  RETURNS:
;
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;  THEORY of OPERATION or PROCEDURE:
;
;-----------------------------------------------------------------------------
IF (USB_CB_SRC_h2d_cls_ifc_11 & USB_UM_SUPPLIED)
export  `@INSTANCE_NAME`_CB_h2d_cls_ifc_11
`@INSTANCE_NAME`_CB_h2d_cls_ifc_11:
    MOV     A, REG[`@INSTANCE_NAME`_EP0DATA+wIndexLo] `@CTAB40`; Get the interface number
    CMP     A, `@DSHID_bNumInterfaces`h `@CTAB40`; Range check
    JNC     `@INSTANCE_NAME`_Not_Supported_Local_Hid

    MOV     X, A `@CTAB40`; Save the interface number

    MOV     A, REG[`@INSTANCE_NAME`_EP0DATA+wValueLo] `@CTAB40`; Get the protocol
    CMP     A, (1+1) `@CTAB40`; Must be zero or one
    JNC     `@INSTANCE_NAME`_Not_Supported_Local_Hid

    MOV     [X + `@INSTANCE_NAME`_Protocol], A `@CTAB40`; Save the new protocol

    LJMP     `@INSTANCE_NAME`_NoDataStageControlTransfer
ENDIF
;-----------------------------------------------------------------------------
;  FUNCTION NAME:   Find_Report
;
;  DESCRIPTION:     Scan the HID Report Tree and return a pointer to the 
;                   HID Report Transfer Descriptor (TD) or NULL
;                   This function is called in during the processing of
;                   GET_REPORT or SET_REPORT HID Class Requests.
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:       
;
;  RETURNS:
;
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;  THEORY of OPERATION or PROCEDURE:
;
;-----------------------------------------------------------------------------
Find_Report:
    CALL    `@INSTANCE_NAME`_GetInterfaceLookupTable `@CTAB40`; Point the the interface lookup table
    ; The first entry of the table point to the report table.
    MOV     [`@INSTANCE_NAME`_t2],`@INSTANCE_NAME`_t1 `@CTAB40`; Set the GETWORD destination 
    CALL    `@INSTANCE_NAME`_GETWORD `@CTAB40`; Get the pointer to the transfer descriptor table
                                     `@CTAB40`; ITempW has the address
    MOV     A, REG[`@INSTANCE_NAME`_EP0DATA+wIndexLo] `@CTAB40`; Get the interface number
    MOV     [`@INSTANCE_NAME`_t2], A `@CTAB40`; Use the UM temp var--Selector
    MOV     A, [`@INSTANCE_NAME`_t1] `@CTAB40`; Get the Table Address MSB
    MOV     X, [`@INSTANCE_NAME`_t1+1] `@CTAB40`; Get the Table Address LSB

    ASL     [`@INSTANCE_NAME`_t2] `@CTAB40`; Convert the index to offset

    SWAP    A, X
    ADD     A, [`@INSTANCE_NAME`_t2]
    SWAP    A, X
    ADC     A, 0 `@CTAB40`; A:X now points to the table entry we want

; Get the pointer to the Report Type Table
    GET_WORD

; Dereference to the requested Report Type
    PUSH    A  `@CTAB40`; Don't loose the pointer MSB
    MOV     A, REG[`@INSTANCE_NAME`_EP0DATA+wValueHi] `@CTAB40`; Get the Report Type
    DEC     A`@CTAB40`; Make it 0 based
    MOV     [`@INSTANCE_NAME`_t2], A `@CTAB40`; Use the UM temp var--Selector
    POP     A `@CTAB40`; Get the MSB back
    PUSH    A  `@CTAB40`; Don't loose the pointer MSB
    ROMX       `@CTAB40`; Get the table size
    CMP     A, [`@INSTANCE_NAME`_t2] `@CTAB40`; Range check
    JC      .not_supported_pop_1
        
    POP     A `@CTAB40`; Get the MSB back
    INC     X `@CTAB40`; Point to the next  entry
    ADC     A, 0 `@CTAB40`;

    LT_INDEX_TO_OFFSET `@INSTANCE_NAME`_t2 `@CTAB40`; Convert the index to offset

    SWAP    A, X
    ADD     A, [`@INSTANCE_NAME`_t2]
    SWAP    A, X
    ADC     A, 0 `@CTAB40`; A:X now points to the table entry we want
; Get the pointer to the requested Report Table
    GET_WORD `@CTAB40` ; A:X points to the 

    NULL_PTR_CHECK .not_supported `@CTAB40`; Null Table entries indicated not supported
; Dereference to the requested TRANSFER DESCRIPTOR
    PUSH    A  `@CTAB40`; Don't loose the pointer MSB
    MOV     A, REG[`@INSTANCE_NAME`_EP0DATA+wValueLo] `@CTAB40`; Get the Report ID
    MOV     [`@INSTANCE_NAME`_t2], A `@CTAB40`; Use the UM temp var--Selector
    POP     A `@CTAB40`; Get the MSB back
    PUSH    A  `@CTAB40`; Don't loose the pointer MSB
    ROMX    `@CTAB40`; Get the table size
    CMP     A, [`@INSTANCE_NAME`_t2] `@CTAB40`; Range check
    JC      .not_supported_pop_1
        
    POP     A `@CTAB40`; Get the MSB back

    RET     `@CTAB40`; Finished A:X point to the TD

.not_supported_pop_1:
    POP     A    `@CTAB40`; Restore the stack
.not_supported:
    MOV     A, 0 `@CTAB40`; Return a null pointer
    MOV     X, A `@CTAB40`; 
    RET

;-----------------------------------------------------------------------------
;  FUNCTION NAME: `@INSTANCE_NAME`_GetInterfaceLookupTable
;
;  DESCRIPTION:   Point to the interface lookup table
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:
;
;  RETURNS:
;
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;  THEORY of OPERATION or PROCEDURE:
;
;-----------------------------------------------------------------------------
export  `@INSTANCE_NAME`_GetInterfaceLookupTable:
`@INSTANCE_NAME`_GetInterfaceLookupTable:
    CALL    `@INSTANCE_NAME`_GET_CONFIG_TABLE_ENTRY`@CTAB40`; Get the CONFIG_LOOKUP entry
    SWAP    A, X`@CTAB40`; Second entry points to the HID_LOOKUP table
    ADD     A, 2`@CTAB40`; So add two
    SWAP    A, X`@CTAB40`; 
    ADC     A, 0`@CTAB40`; Don't forget the carry
    MOV     [`@INSTANCE_NAME`_t2],`@INSTANCE_NAME`_t1 `@CTAB40`; Set the GETWORD destination 
    CALL    `@INSTANCE_NAME`_GETWORD `@CTAB40`; Get the pointer to the HID_LOOKUP table
                                     `@CTAB40`; ITempW has the address
    MOV     A, [`@INSTANCE_NAME`_t1] `@CTAB40`; Get the table address MSB
    MOV     X, [`@INSTANCE_NAME`_t1+1] `@CTAB40`; Get the table address LSB
    RET
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;  USB 2nd Tier Dispactch Jump Tables for HID Class Requests (based on bRequest)
;-----------------------------------------------------------------------------
;  FUNCTION NAME: ;  USB 2nd Tier Dispactch Jump Table
;
;  DESCRIPTION:   The following tables dispatch to the Standard request handler
;                 functions.  (Assumes bmRequestType(5:6) is 0, Standard)
;
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
`@INSTANCE_NAME`_DT_h2d_cls_ifc:
;-----------------------------------------------------------------------------
    jmp     `@INSTANCE_NAME`_CB_h2d_cls_ifc_00
    jmp     `@INSTANCE_NAME`_CB_h2d_cls_ifc_01
    jmp     `@INSTANCE_NAME`_CB_h2d_cls_ifc_02
    jmp     `@INSTANCE_NAME`_CB_h2d_cls_ifc_03
    jmp     `@INSTANCE_NAME`_CB_h2d_cls_ifc_04
    jmp     `@INSTANCE_NAME`_CB_h2d_cls_ifc_05
    jmp     `@INSTANCE_NAME`_CB_h2d_cls_ifc_06
    jmp     `@INSTANCE_NAME`_CB_h2d_cls_ifc_07
    jmp     `@INSTANCE_NAME`_CB_h2d_cls_ifc_08
    jmp     `@INSTANCE_NAME`_CB_h2d_cls_ifc_09
    jmp     `@INSTANCE_NAME`_CB_h2d_cls_ifc_10
    jmp     `@INSTANCE_NAME`_CB_h2d_cls_ifc_11
    jmp     `@INSTANCE_NAME`_CB_h2d_cls_ifc_12

`@INSTANCE_NAME`_DT_h2d_cls_ifc_End:
`@INSTANCE_NAME`_DT_h2d_cls_ifc_Size: equ (`@INSTANCE_NAME`_DT_h2d_cls_ifc_End-`@INSTANCE_NAME`_DT_h2d_cls_ifc) / 2
`@INSTANCE_NAME`_DT_h2d_cls_ifc_Dispatch::
    CMP     [`@INSTANCE_NAME`_Configuration], 0 `@CTAB40`; Is the device configured?
    JNZ     .configured `@CTAB40`; Jump on configured
    JMP     `@INSTANCE_NAME`_Not_Supported_Local_Hid `@CTAB40`; Stall the request if not configured
; Jump here if the device is configured
.configured:
    MOV     A, REG[`@INSTANCE_NAME`_EP0DATA + bRequest]`@CTAB50`; Get the request number
    DISPATCHER `@INSTANCE_NAME`_DT_h2d_cls_ifc, `@INSTANCE_NAME`_DT_h2d_cls_ifc_Size, `@INSTANCE_NAME`_Not_Supported_Local_Hid 

;-----------------------------------------------------------------------------
`@INSTANCE_NAME`_DT_d2h_cls_ifc:
;-----------------------------------------------------------------------------

    jmp     `@INSTANCE_NAME`_CB_d2h_cls_ifc_00
    jmp     `@INSTANCE_NAME`_CB_d2h_cls_ifc_01
    jmp     `@INSTANCE_NAME`_CB_d2h_cls_ifc_02
    jmp     `@INSTANCE_NAME`_CB_d2h_cls_ifc_03

`@INSTANCE_NAME`_DT_d2h_cls_ifc_End:
`@INSTANCE_NAME`_DT_d2h_cls_ifc_Size: equ (`@INSTANCE_NAME`_DT_d2h_cls_ifc_End-`@INSTANCE_NAME`_DT_d2h_cls_ifc) / 2
`@INSTANCE_NAME`_DT_d2h_cls_ifc_Dispatch::
    CMP     [`@INSTANCE_NAME`_Configuration], 0 `@CTAB40`; Is the device configured?
    JNZ     .configured `@CTAB40`; Jump on configured
    JMP     `@INSTANCE_NAME`_Not_Supported_Local_Hid `@CTAB40`; Stall the request if not configured
; Jump here if the device is configured
.configured:
    MOV     A, REG[`@INSTANCE_NAME`_EP0DATA + bRequest]`@CTAB50`; Get the request number
    DISPATCHER `@INSTANCE_NAME`_DT_d2h_cls_ifc, `@INSTANCE_NAME`_DT_d2h_cls_ifc_Size, `@INSTANCE_NAME`_Not_Supported_Local_Hid 

IF (USB_CB_SRC_d2h_cls_ifc_00 & USB_NOT_SUPPORTED)
export  `@INSTANCE_NAME`_CB_d2h_cls_ifc_00
`@INSTANCE_NAME`_CB_d2h_cls_ifc_00:
ENDIF
IF (USB_CB_SRC_d2h_cls_ifc_01 & USB_NOT_SUPPORTED)
export  `@INSTANCE_NAME`_CB_d2h_cls_ifc_01
`@INSTANCE_NAME`_CB_d2h_cls_ifc_01:
ENDIF
IF (USB_CB_SRC_d2h_cls_ifc_02 & USB_NOT_SUPPORTED)
export  `@INSTANCE_NAME`_CB_d2h_cls_ifc_02
`@INSTANCE_NAME`_CB_d2h_cls_ifc_02:
ENDIF
IF (USB_CB_SRC_d2h_cls_ifc_03 & USB_NOT_SUPPORTED)
export  `@INSTANCE_NAME`_CB_d2h_cls_ifc_03
`@INSTANCE_NAME`_CB_d2h_cls_ifc_03:
ENDIF
IF (USB_CB_SRC_h2d_cls_ifc_00 & USB_NOT_SUPPORTED)
export  `@INSTANCE_NAME`_CB_h2d_cls_ifc_00
`@INSTANCE_NAME`_CB_h2d_cls_ifc_00:
ENDIF
IF (USB_CB_SRC_h2d_cls_ifc_01 & USB_NOT_SUPPORTED)
export  `@INSTANCE_NAME`_CB_h2d_cls_ifc_01
`@INSTANCE_NAME`_CB_h2d_cls_ifc_01:
ENDIF
IF (USB_CB_SRC_h2d_cls_ifc_02 & USB_NOT_SUPPORTED)
export  `@INSTANCE_NAME`_CB_h2d_cls_ifc_02
`@INSTANCE_NAME`_CB_h2d_cls_ifc_02:
ENDIF
IF (USB_CB_SRC_h2d_cls_ifc_03 & USB_NOT_SUPPORTED)
export  `@INSTANCE_NAME`_CB_h2d_cls_ifc_03
`@INSTANCE_NAME`_CB_h2d_cls_ifc_03:
ENDIF
IF (USB_CB_SRC_h2d_cls_ifc_04 & USB_NOT_SUPPORTED)
export  `@INSTANCE_NAME`_CB_h2d_cls_ifc_04
`@INSTANCE_NAME`_CB_h2d_cls_ifc_04:
ENDIF
IF (USB_CB_SRC_h2d_cls_ifc_05 & USB_NOT_SUPPORTED)
export  `@INSTANCE_NAME`_CB_h2d_cls_ifc_05
`@INSTANCE_NAME`_CB_h2d_cls_ifc_05:
ENDIF
IF (USB_CB_SRC_h2d_cls_ifc_06 & USB_NOT_SUPPORTED)
export  `@INSTANCE_NAME`_CB_h2d_cls_ifc_06
`@INSTANCE_NAME`_CB_h2d_cls_ifc_06:
ENDIF
IF (USB_CB_SRC_h2d_cls_ifc_07 & USB_NOT_SUPPORTED)
export  `@INSTANCE_NAME`_CB_h2d_cls_ifc_07
`@INSTANCE_NAME`_CB_h2d_cls_ifc_07:
ENDIF
IF (USB_CB_SRC_h2d_cls_ifc_08 & USB_NOT_SUPPORTED)
export  `@INSTANCE_NAME`_CB_h2d_cls_ifc_08
`@INSTANCE_NAME`_CB_h2d_cls_ifc_08:
ENDIF
IF (USB_CB_SRC_h2d_cls_ifc_09 & USB_NOT_SUPPORTED)
export  `@INSTANCE_NAME`_CB_h2d_cls_ifc_09
`@INSTANCE_NAME`_CB_h2d_cls_ifc_09:
ENDIF
IF (USB_CB_SRC_h2d_cls_ifc_10 & USB_NOT_SUPPORTED)
export  `@INSTANCE_NAME`_CB_h2d_cls_ifc_10
`@INSTANCE_NAME`_CB_h2d_cls_ifc_10:
ENDIF
IF (USB_CB_SRC_h2d_cls_ifc_11 & USB_NOT_SUPPORTED)
export  `@INSTANCE_NAME`_CB_h2d_cls_ifc_11
`@INSTANCE_NAME`_CB_h2d_cls_ifc_11:
ENDIF
IF (USB_CB_SRC_h2d_cls_ifc_12 & USB_NOT_SUPPORTED)
export  `@INSTANCE_NAME`_CB_h2d_cls_ifc_12
`@INSTANCE_NAME`_CB_h2d_cls_ifc_12:
ENDIF

`@INSTANCE_NAME`_Not_Supported_Local_Hid:
    LJMP     `@INSTANCE_NAME`_Not_Supported

`@INSTANCE_NAME`_GetTableEntry_Local_Hid:
    LJMP     `@INSTANCE_NAME`_GetTableEntry

;-----------------------------------------------
; Add custom application code for routines 
; redefined by USB_APP_SUPPLIED in USB_HID.INC
;-----------------------------------------------

   ;@PSoC_UserCode_BODY_1@ (Do not change this line.)
   ;---------------------------------------------------
   ; Insert your custom code below this banner
   ;---------------------------------------------------

   ;---------------------------------------------------
   ; Insert your custom code above this banner
   ;---------------------------------------------------
   ;@PSoC_UserCode_END@ (Do not change this line.)

; End of File `@INSTANCE_NAME`_cls_hid.asm
