;;*****************************************************************************
;;*****************************************************************************
;;  FILENAME: `@INSTANCE_NAME`.asm
;;   Version: 1.5, Updated on 2006/06/19 at 11:43:56
;;  `@PSOC_VERSION`
;;
;;  DESCRIPTION: USB Device User Module software implementation file
;;               for the enCoRe II family of devices
;;
;;  NOTE: User Module APIs conform to the fastcall convention for marshalling
;;        arguments and observe the associated "Registers are volatile" policy.
;;        This means it is the caller's responsibility to preserve any values
;;        in the X and A registers that are still needed after the API
;;        function returns. Even though these registers may be preserved now,
;;        there is no guarantee they will be preserved in future releases.
;;-----------------------------------------------------------------------------
;;  Copyright (c) Cypress Semiconductor 2004, 2005. All Rights Reserved.
;;*****************************************************************************
;;*****************************************************************************

include "m8c.inc"
include "`@INSTANCE_NAME`_macros.inc"
include "`@INSTANCE_NAME`.inc"
IF 0x`@DSHID_bNumStringDescrs` & 0x10000000
PSOC_ERROR Please run the USB Setup Wizard.  Device Editor, Right Click the USB User Module
; This message will only appear if the USB Setup Wizard has not be run and the descriptors
; and associated data structures have been created.
; After running the USB Setup Wizard, you must also select the Config/Generate Application
; menu item from PSoC Designer in order to generate USB User Module data structures and
; descriptors.
ENDIF
;-----------------------------------------------
;  Global Symbols
;-----------------------------------------------
EXPORT `@INSTANCE_NAME`_Start
EXPORT _`@INSTANCE_NAME`_Start
EXPORT `@INSTANCE_NAME`_Stop
EXPORT _`@INSTANCE_NAME`_Stop
EXPORT `@INSTANCE_NAME`_bCheckActivity
EXPORT _`@INSTANCE_NAME`_bCheckActivity
EXPORT `@INSTANCE_NAME`_bGetConfiguration
EXPORT _`@INSTANCE_NAME`_bGetConfiguration
EXPORT `@INSTANCE_NAME`_bGetEPState
EXPORT _`@INSTANCE_NAME`_bGetEPState
EXPORT `@INSTANCE_NAME`_bGetEPCount
EXPORT _`@INSTANCE_NAME`_bGetEPCount
EXPORT `@INSTANCE_NAME`_XLoadEP
EXPORT _`@INSTANCE_NAME`_XLoadEP
EXPORT `@INSTANCE_NAME`_EnableOutEP
EXPORT _`@INSTANCE_NAME`_EnableOutEP
EXPORT `@INSTANCE_NAME`_DisableOutEP
EXPORT _`@INSTANCE_NAME`_DisableOutEP
EXPORT `@INSTANCE_NAME`_EnableEP
EXPORT _`@INSTANCE_NAME`_EnableEP
EXPORT `@INSTANCE_NAME`_DisableEP
EXPORT _`@INSTANCE_NAME`_DisableEP
EXPORT `@INSTANCE_NAME`_Force
EXPORT _`@INSTANCE_NAME`_Force
EXPORT `@INSTANCE_NAME`_Suspend
EXPORT _`@INSTANCE_NAME`_Suspend
EXPORT `@INSTANCE_NAME`_Resume
EXPORT _`@INSTANCE_NAME`_Resume
EXPORT `@INSTANCE_NAME`_bRWUEnabled
EXPORT _`@INSTANCE_NAME`_bRWUEnabled

AREA usbRAM (RAM,REL)
;-----------------------------------------------
;  Variable Allocation
;-----------------------------------------------
;----------------------------------------------------------------------------
EXPORT `@INSTANCE_NAME`_APITemp
 `@INSTANCE_NAME`_APITemp: `@CTAB40`BLK   2 ; Two bytes of temporary
                          `@CTAB40`         ; storage shared by the API
                          `@CTAB40`         ; functions
EXPORT `@INSTANCE_NAME`_APIEPNumber, _`@INSTANCE_NAME`_APIEPNumber
_`@INSTANCE_NAME`_APIEPNumber:
 `@INSTANCE_NAME`_APIEPNumber: `@CTAB40`BLK   1 ; API storage for speed
EXPORT `@INSTANCE_NAME`_APICount, _`@INSTANCE_NAME`_APICount
_`@INSTANCE_NAME`_APICount:
 `@INSTANCE_NAME`_APICount: `@CTAB40`BLK   1 ; API storage for speed

EXPORT `@INSTANCE_NAME`_bActivity
 `@INSTANCE_NAME`_bActivity: `@CTAB40`BLK   1 ; Activity flag (Shared between the ISR and API)

EXPORT Port_1_Data_SHADE, _Port_1_Data_SHADE
_Port_1_Data_SHADE:
Port_1_Data_SHADE: `@CTAB40`BLK   1 ; Shadow register for Port 1

;-----------------------------------------------
;  Constant Data Allocation
;-----------------------------------------------
AREA Bootloader (ROM, REL)
EXPORT `@INSTANCE_NAME`_USB_EP_BIT_LOOKUP
.LITERAL
`@INSTANCE_NAME`_USB_EP_BIT_LOOKUP:  ;
    DB     01H                       ; EP0
    DB     02H                       ; EP1
    DB     04H                       ; EP2
.ENDLITERAL

AREA Bootloader (ROM, REL)

;-----------------------------------------------------------------------------
;  FUNCTION NAME: `@INSTANCE_NAME`_Start
;
;  DESCRIPTION:    Starts the USB User Module
;                    Sets the device selection
;                    Set the configuration to unconfigured
;                    Enables the SIE for Address 0
;                    Enables the USB pullup (D- for low speed, D+ for full speed)
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:    A is the desired device setting
;
;  RETURNS:
;
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;  THEORY of OPERATION or PROCEDURE:
;
;-----------------------------------------------------------------------------
.SECTION
 `@INSTANCE_NAME`_Start:
_`@INSTANCE_NAME`_Start:
    MOV    REG[P10CR], 0x00`@CTAB40`; Disable the states
    MOV    REG[P11CR], 0x00`@CTAB40`; 

    MOV     [`@INSTANCE_NAME`_bCurrentDevice], A`@CTAB40`; The app selects the desired device

    MOV     [`@INSTANCE_NAME`_TransferType], USB_TRANS_STATE_IDLE`@CTAB40`; Transaction Idle State
    MOV     [`@INSTANCE_NAME`_Configuration], 0`@CTAB40`; Unconfigured
    MOV     [`@INSTANCE_NAME`_DeviceStatus], 0`@CTAB40`; Clears device status

    MOV     [`@INSTANCE_NAME`_EPDataToggle], 0 `@CTAB40`; Clear all EP data toggles

; Flow here to enable the SIE
    MOV     REG[`@INSTANCE_NAME`_ADDR], USB_ADDR_ENABLE`@CTAB40`; Enable Address 0
    OR      REG[`@INSTANCE_NAME`_USBXCR], USB_PULLUP_ENABLE`@CTAB40`; Pullup D-
    MOV     REG[`@INSTANCE_NAME`_EP0MODE], USB_MODE_STALL_IN_OUT`@CTAB40`; ACK Setup/Stall IN/OUT
    NOP
    MOV     A, REG[`@INSTANCE_NAME`_EP0MODE]`@CTAB40`; Read the mode register as a debug marker

    M8C_EnableIntMask INT_MSK1, (INT_MSK1_USB_ACTIVITY | INT_MSK1_USB_BUS_RESET | INT_MSK1_USB_EP0)
    RET
.ENDSECTION
;-----------------------------------------------------------------------------
;  FUNCTION NAME: `@INSTANCE_NAME`_Stop
;
;  DESCRIPTION:
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:
;
;  RETURNS:
;
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;  THEORY of OPERATION or PROCEDURE:
;
;-----------------------------------------------------------------------------
.SECTION
 `@INSTANCE_NAME`_Stop:
_`@INSTANCE_NAME`_Stop:
    MOV     [`@INSTANCE_NAME`_bCurrentDevice], 0`@CTAB40`; The app selects the desired device

    MOV     [`@INSTANCE_NAME`_TransferType], USB_TRANS_STATE_IDLE`@CTAB40`; Transaction Idle State
    MOV     [`@INSTANCE_NAME`_Configuration], 0`@CTAB40`; Unconfigured
    MOV     [`@INSTANCE_NAME`_DeviceStatus], 0`@CTAB40`; Clear the  device status
    MOV     [`@INSTANCE_NAME`_bActivity], 0`@CTAB40`; Clear the activity flag
    MOV     REG[`@INSTANCE_NAME`_ADDR], 0`@CTAB40`; Clear the addfress and Address 0
    AND     REG[`@INSTANCE_NAME`_USBXCR], ~USB_PULLUP_ENABLE`@CTAB40`; Release D-
    M8C_DisableIntMask    INT_MSK1, (INT_MSK1_USB_ACTIVITY | INT_MSK1_USB_BUS_RESET | INT_MSK1_USB_EP0 | INT_MSK1_USB_EP1 | INT_MSK1_USB_EP2) ; Enable the interrupt

    RET
.ENDSECTION
;-----------------------------------------------------------------------------
;  FUNCTION NAME: `@INSTANCE_NAME`_bCheckActivity
;
;  DESCRIPTION:
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:
;
;  RETURNS:
;
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;  THEORY of OPERATION or PROCEDURE:
;
;   The activity interrupt sets a RAM flag indicating activity and disables the
;   interrupt.  Disabling the interrupt keeps the bus activity from creating too
;   many interrupts.  bCheckActivity checks and clears the flag, the enables
;   interrupts for the next interval.
;
;-----------------------------------------------------------------------------
.SECTION
 `@INSTANCE_NAME`_bCheckActivity:
_`@INSTANCE_NAME`_bCheckActivity:
    MOV    A, [`@INSTANCE_NAME`_bActivity]`@CTAB40`; Activity?
    CMP    A, 1`@CTAB40`; 
    JZ     .active`@CTAB40`; Jump on Activity
; Flow here on no activity
    RET
; Jump here if activity was detected
.active:
    MOV    [`@INSTANCE_NAME`_bActivity], 0`@CTAB40`; Clear the activity flag for next time
    M8C_EnableIntMask INT_MSK1, INT_MSK1_USB_ACTIVITY`@CTAB40`; Enable the activity interupt
    RET
.ENDSECTION
;-----------------------------------------------------------------------------
;  FUNCTION NAME: `@INSTANCE_NAME`_bGetConfiguration
;
;  DESCRIPTION:   Returns the current configuration number
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:    None
;
;  RETURNS:      A contains the current configuration number
;
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;  THEORY of OPERATION or PROCEDURE:
;
;-----------------------------------------------------------------------------
.SECTION
 `@INSTANCE_NAME`_bGetConfiguration:
_`@INSTANCE_NAME`_bGetConfiguration:
    MOV     A,[`@INSTANCE_NAME`_Configuration]
    RET
.ENDSECTION
;-----------------------------------------------------------------------------
;  FUNCTION NAME: `@INSTANCE_NAME`_bGetEPState
;
;  DESCRIPTION:   Returns the current endpoint state
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:   A: Endpoint Number
;
;  RETURNS:     A: NO_EVENT_ALLOWED
;                  EVENT_PENDING
;                  NO_EVENT_PENDING
;
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;  THEORY of OPERATION or PROCEDURE:
;
;-----------------------------------------------------------------------------
.SECTION
 `@INSTANCE_NAME`_bGetEPState:
_`@INSTANCE_NAME`_bGetEPState:
    CMP     A, (USB_MAX_EP_NUMBER + 1)`@CTAB40`; Range check
    JNC     .invalid_ep`@CTAB40`; Bail out
; Flow here to enable an endpoint        
    MOV     X, A`@CTAB40`; Endpoint number is the index
    MOV     A, [X+`@INSTANCE_NAME`_EndpointAPIStatus]; Get the state
    JMP     .exit`@CTAB40`; Go to the common exit
; Jump here for an invalid endpoint
.invalid_ep:
    MOV     A, 0`@CTAB40`; Return 0 for an invalid ep
; Jump or flow here for a common exit
.exit:
    RET`@CTAB40`; All done
.ENDSECTION
;-----------------------------------------------------------------------------
;  FUNCTION NAME: `@INSTANCE_NAME`_bRWUEnabled
;
;  DESCRIPTION:   Returns 1 if Remote Wake Up is enabled, otherwise 0
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:   None
;
;  RETURNS:     A: 1--Remote Wake Up Enabled
;                  0--Remote Wake Up Disabled
;
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;  THEORY of OPERATION or PROCEDURE:
;
;-----------------------------------------------------------------------------
.SECTION
 `@INSTANCE_NAME`_bRWUEnabled:
_`@INSTANCE_NAME`_bRWUEnabled:
    TST     [`@INSTANCE_NAME`_DeviceStatus], USB_DEVICE_STATUS_REMOTE_WAKEUP
    JNZ     .enabled`@CTAB40`; Jump if enabled
; Flow here if RWU is disabled        
    MOV     A, 0`@CTAB40`; Return disabled
    JMP     .exit`@CTAB40`; Go to the common exit
; Jump when RWU is enabled
.enabled:
    MOV     A, 1`@CTAB40`; Return enabled
; Jump or flow here for a common exit
.exit:
    RET`@CTAB40`; All done
.ENDSECTION
;-----------------------------------------------------------------------------
;  FUNCTION NAME: `@INSTANCE_NAME`_bGetEPCount
;
;  DESCRIPTION:
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:
;
;  RETURNS:
;
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;  THEORY of OPERATION or PROCEDURE:
;
;-----------------------------------------------------------------------------
.SECTION
 `@INSTANCE_NAME`_bGetEPCount:
_`@INSTANCE_NAME`_bGetEPCount:
    CMP     A, (USB_MAX_EP_NUMBER + 1)`@CTAB40`; Range check
    JNC     .invalid_ep`@CTAB40`; Bail out
; Flow here to get the endpoint count
    MOV     X, A`@CTAB40`; Endpoint number is the index
    MOV     A, REG[X+EP0CNT]`@CTAB40`; Here is the count
    AND     A, 0x1F`@CTAB40`; Mask off the count
    SUB     A, 2`@CTAB40`; Ours includes the two byte checksum
    JMP     .exit`@CTAB40`; Go to the common exit
; Jump here for an invalid endpoint
.invalid_ep:
    MOV     A, 0`@CTAB40`; Return 0 for an invalid ep
; Jump or flow here for a common exit
.exit:
    RET
.ENDSECTION
;-----------------------------------------------------------------------------
;  FUNCTION NAME: `@INSTANCE_NAME`_LoadEP
;
;  DESCRIPTION:    This function loads the specified endpoint buffer
;                  with the number of bytes previously set in the count
;                  register.
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:  A:X Pointer to the ram buffer containing the data to transfer
;              `@INSTANCE_NAME`_APIEPNumber loaded with the endpoint number
;              `@INSTANCE_NAME`_APICount loaded with the number of bytes to load
;
;  RETURNS:    NONE
;
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;  THEORY of OPERATION or PROCEDURE:
;
;-----------------------------------------------------------------------------
.SECTION
 `@INSTANCE_NAME`_XLoadEP:
_`@INSTANCE_NAME`_XLoadEP:
; extern void `@INSTANCE_NAME`_LoadEP(BYTE, BYTE*);
    CMP     [`@INSTANCE_NAME`_APIEPNumber], (USB_MAX_EP_NUMBER + 1)`@CTAB40`; Range check
    JNC     .exit`@CTAB40`; Bail out
; Flow here to get the endpoint count
    MOV     [`@INSTANCE_NAME`_APITemp], X`@CTAB40`; Use this temp as the MVI pointer

    MOV     A, [`@INSTANCE_NAME`_APIEPNumber]`@CTAB40`; Get the endpoint number
    INDEX   EPREGPTR`@CTAB40`; Get the address of the endpoint data register array
    MOV     X, A`@CTAB40`; We are going to use index access to the register array
    
    MOV     A, [`@INSTANCE_NAME`_APICount]`@CTAB40`; Get the count
    MOV     [`@INSTANCE_NAME`_APITemp+1], A`@CTAB40`; Use this temp as the count
; Copy loop
.loop:
    DEC     [`@INSTANCE_NAME`_APITemp+1]`@CTAB40`; Are we done?
    JC      .done`@CTAB40`; Jump if we are
    MVI     A, [`@INSTANCE_NAME`_APITemp]`@CTAB40`; Get the data, inc the pointer
    MOV     REG[X + 0], A `@CTAB40`; Store the data
    INC     X`@CTAB40`; Index the next data register
    JMP     .loop`@CTAB40`; Copy the next byte or finish

; Jump here when the copy is finished
.done:
    MOV     X, [`@INSTANCE_NAME`_APIEPNumber]`@CTAB40`; Get the endpoint number
    MOV     A, X
    INDEX   `@INSTANCE_NAME`_USB_EP_BIT_LOOKUP
    AND     A, [`@INSTANCE_NAME`_EPDataToggle]
    JZ      .addcount
    MOV     A, USB_CNT_TOGGLE

.addcount:     
    OR      A, [`@INSTANCE_NAME`_APICount]`@CTAB40`; Get the count
    MOV     REG[X + EP0CNT], A`@CTAB40`; Update the count register

    MOV     A, REG[0xF7]  ; save GIE state
    MOV     [`@INSTANCE_NAME`_APITemp], A  ; save GIE state
    M8C_DisableGInt

    ; Check that we aren't halted
    CMP     [X + `@INSTANCE_NAME`_EndpointAPIStatus], NO_EVENT_ALLOWED `@CTAB40`; Get the state
    JZ      .no_exempt_exit

    MOV     [X + `@INSTANCE_NAME`_EndpointAPIStatus], NO_EVENT_PENDING`@CTAB40`; Set the state

.write_again:
    MOV     REG[X + EP0MODE], USB_MODE_ACK_IN`@CTAB40`; Enable the endpoint
    MOV     A, REG[X + EP0MODE]
    CMP     A, USB_MODE_ACK_IN
    JNZ     .write_again

.no_exempt_exit:
    TST     [`@INSTANCE_NAME`_APITemp], 0x01
    JZ      .exit
    M8C_EnableGInt

; Jump or flow here for a common exit
.exit:
    RET
.LITERAL
EPREGPTR:    DB    EP0DATA, EP1DATA, EP2DATA
.ENDLITERAL
.ENDSECTION
;-----------------------------------------------------------------------------
;  FUNCTION NAME: `@INSTANCE_NAME`_EnableEP
;
;  DESCRIPTION:    This function enables an OUT endpoint.  It should not be
;                  called for an IN endpoint.
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:      A contains the endpoint number
;
;  RETURNS:        None
;
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;  THEORY of OPERATION or PROCEDURE:
;
;-----------------------------------------------------------------------------
.SECTION
 `@INSTANCE_NAME`_EnableOutEP:
_`@INSTANCE_NAME`_EnableOutEP:
 `@INSTANCE_NAME`_EnableEP:
_`@INSTANCE_NAME`_EnableEP:
    CMP     A, 0`@CTAB40`; Can't enable EP0
    JZ      .exit`@CTAB40`; Bail out
    CMP     A, (USB_MAX_EP_NUMBER + 1)`@CTAB40`; Range check
    JNC     .exit`@CTAB40`; Bail out

; Flow here to enable an endpoint        
    MOV     X, A`@CTAB40`; Endpoint number is the index

    MOV     A, REG[0xF7]  ; save GIE state
    MOV     [`@INSTANCE_NAME`_APITemp], A  ; save GIE state
    M8C_DisableGInt

    ; Check that we aren't halted
    CMP     [X + `@INSTANCE_NAME`_EndpointAPIStatus], NO_EVENT_ALLOWED `@CTAB40`
    JZ      .no_exempt_exit

    MOV     [X+`@INSTANCE_NAME`_EndpointAPIStatus], NO_EVENT_PENDING`@CTAB40`; For the API
    MOV     A, REG[X+EP0MODE]`@CTAB40`; Unlock the mode register
    MOV     REG[X+EP0MODE], USB_MODE_ACK_OUT`@CTAB40`; Enable the endpoint

.no_exempt_exit:
    TST     [`@INSTANCE_NAME`_APITemp], 0x01
    JZ      .exit
    M8C_EnableGInt

; Jump or flow here for a common exit
.exit:
    RET`@CTAB40`; All done
.ENDSECTION
;-----------------------------------------------------------------------------
;  FUNCTION NAME: `@INSTANCE_NAME`_DisableEP
;
;  DESCRIPTION:    This function disables an OUT endpoint.  It should not be
;                  called for an IN endpoint.
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:
;
;  RETURNS:
;
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;  THEORY of OPERATION or PROCEDURE:
;
;-----------------------------------------------------------------------------
.SECTION
 `@INSTANCE_NAME`_DisableOutEP:
_`@INSTANCE_NAME`_DisableOutEP:
 `@INSTANCE_NAME`_DisableEP:
_`@INSTANCE_NAME`_DisableEP:
    CMP     A, 0`@CTAB40`; Can't disable EP0
    JZ      .exit`@CTAB40`; Bail out
    CMP     A, (USB_MAX_EP_NUMBER + 1)`@CTAB40`; Range check
    JNC     .exit`@CTAB40`; Bail out
; Flow here to disable an endpoint        
    MOV     X, A`@CTAB40`; Endpoint number is the index
    MOV     A, REG[X+EP0MODE]`@CTAB40`; Unlock the mode register
    MOV     REG[X+EP0MODE], USB_MODE_NAK_OUT`@CTAB40`; Disable the endpoint
; Jump or flow here for a common exit
.exit:
    RET`@CTAB40`; All done
.ENDSECTION
;-----------------------------------------------------------------------------
;  FUNCTION NAME: `@INSTANCE_NAME`_Force
;
;  DESCRIPTION:    Force the J/K/SE0 State of D+/D-
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:     A: USB_FORCE_J
;                    USB_FORCE_K
;                    USB_FORCE_SE0
;                    USB_FORCE_NONE
;
;  RETURNS:       Nothing
;
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;  THEORY of OPERATION or PROCEDURE:
;
;-----------------------------------------------------------------------------
.SECTION
 `@INSTANCE_NAME`_Force:
_`@INSTANCE_NAME`_Force:
    CMP    A, USB_FORCE_NONE`@CTAB40`; Are we done forcing D+/D-?
    JZ     .none`@CTAB40`; Jump if we are done
; Flow here to start checking 
    CMP    A, USB_FORCE_J`@CTAB40`; Force J?
    JNZ    .check_k`@CTAB40`; Jump if not J
; Flow here to force J
    OR     [Port_1_Data_SHADE], 0x02`@CTAB40`; D- = 1
    AND    [Port_1_Data_SHADE], ~(0x01)`@CTAB40`; D+ = 0
    JMP    .force`@CTAB40`; Go set the force register
; Jump here to check Force K
.check_k:
    CMP    A, USB_FORCE_K`@CTAB40`; Force K?
    JNZ    .check_se0`@CTAB40`; Jump if not K
; Flow here to force K
    OR     [Port_1_Data_SHADE], 0x01`@CTAB40`; D+ = 1
    AND    [Port_1_Data_SHADE], ~(0x02)`@CTAB40`; D- = 0
    JMP    .force`@CTAB40`; Go set the force register
; Jump here to check Force SE0
.check_se0:
    CMP    A, USB_FORCE_SE0`@CTAB40`; Force SE0?
    JZ     .invalid`@CTAB40`; Jump if not SE0
; Flow here to force SE0
    AND    [Port_1_Data_SHADE], ~(0x03)`@CTAB40`; D- = 0,  D+ = 0
; Jump or flow here to enable forcing (Port bits are set in the shadow register)
.force:
    MOV    A, [Port_1_Data_SHADE]`@CTAB40`; Get the shadow
    MOV    REG[P1DATA], A`@CTAB40`; Update the port
    OR     REG[USBXCR], USB_FORCE_STATE`@CTAB40`; Enable FORCING D+/D-
    RET    `@CTAB40`; Exit
; Jump here to clear forcing
.none:
    AND    REG[USBXCR], ~(USB_FORCE_STATE)`@CTAB40`; Disable FORCING D+/D-
; Jump or flow here to exit on end forcing or an invalid parameter
.invalid:
    RET    `@CTAB40`; Exit
.ENDSECTION
;-----------------------------------------------------------------------------
;  FUNCTION NAME: `@INSTANCE_NAME`_Suspend
;
;  DESCRIPTION:    Puts the USB Transceiver into power-down mode, while
;                  maintaining the USB address assigned by the USB host. 
;                  To restore the USB Transceiver to normal operation, the
;                  `@INSTANCE_NAME`_Resume function should be called.
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:     None
;
;  RETURNS:       Nothing
;
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;  THEORY of OPERATION or PROCEDURE:
;
;-----------------------------------------------------------------------------
.SECTION
 `@INSTANCE_NAME`_Suspend:
_`@INSTANCE_NAME`_Suspend:
    AND     REG[`@INSTANCE_NAME`_ADDR], ~(USB_ADDR_ENABLE)`@CTAB40`; Disable transceiver
    RET    `@CTAB40`; Exit
.ENDSECTION
;-----------------------------------------------------------------------------
;  FUNCTION NAME: `@INSTANCE_NAME`_Resume
;
;  DESCRIPTION:    Puts the USB Transceiver into normal operation, following
;                  a call to USB_Suspend. It retains the USB address that had
;                  been assigned by the USB host.
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:     None
;
;  RETURNS:       Nothing
;
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;  THEORY of OPERATION or PROCEDURE:
;
;-----------------------------------------------------------------------------
.SECTION
 `@INSTANCE_NAME`_Resume:
_`@INSTANCE_NAME`_Resume:
    OR     REG[`@INSTANCE_NAME`_ADDR], (USB_ADDR_ENABLE)`@CTAB40`; Enable transceiver
    RET    `@CTAB40`; Exit
.ENDSECTION


;-----------------------------------------------
; Add custom application code for routines 
;-----------------------------------------------

   ;@PSoC_UserCode_BODY_1@ (Do not change this line.)
   ;---------------------------------------------------
   ; Insert your custom code below this banner
   ;---------------------------------------------------

   ;**************************************************************************
   ;--------------------------------------------------------------------------
   ;  FUNCTION NAME: bootCustomUserCode
   ;
   ;  DESCRIPTION:
   ;    Here the user can add code to perform system critical tasks in
   ;    case of failed downloads by the bootloader
   ;
   ;  ARGUMENTS: NONE
   ;
   ;  SIDE EFFECTS: NONE
   ;--------------------------------------------------------------------------
   
   bootCustomUserCode::
   _bootCustomUserCode::
   
       ; Enter user critical code here
       
       ret
   ;**************************************************************************

   ;---------------------------------------------------
   ; Insert your custom code above this banner
   ;---------------------------------------------------
   ;@PSoC_UserCode_END@ (Do not change this line.)

; End of File `@INSTANCE_NAME`.asm
