;;*****************************************************************************
;;*****************************************************************************
;;  FILENAME: `@INSTANCE_NAME`INT.asm
;;   Version: 1.5, Updated on 2006/06/19 at 11:43:56
;;  `@PSOC_VERSION`
;;
;;  DESCRIPTION: USB Device User Module software implementation file
;;               for the enCoRe II family of devices
;;
;;  NOTE: User Module APIs conform to the fastcall convention for marshalling
;;        arguments and observe the associated "Registers are volatile" policy.
;;        This means it is the caller's responsibility to preserve any values
;;        in the X and A registers that are still needed after the API
;;        function returns. Even though these registers may be preserved now,
;;        there is no guarantee they will be preserved in future releases.
;;-----------------------------------------------------------------------------
;;  Copyright (c) Cypress Semiconductor 2004, 2005. All Rights Reserved.
;;*****************************************************************************
;;*****************************************************************************

include "m8c.inc"
include "`@INSTANCE_NAME`_macros.inc"
include "`@INSTANCE_NAME`.inc"

;-----------------------------------------------
;  Global Symbols
;-----------------------------------------------

AREA usbRAM (RAM,REL)

;-----------------------------------------------
;  Constant Definitions
;-----------------------------------------------
;-----------------------------------------------
; Variable Allocation
;-----------------------------------------------

AREA Bootloader (ROM, REL)
;-----------------------------------------------------------------------------
;  FUNCTION NAME: `@INSTANCE_NAME`_EP1_ISR
;
;  DESCRIPTION:    Handle the Endpoint 1 event by updating the data toggle
;                  and setting the endpoint state to EVENT_PENDING.  The SIE
;                  automatically set the mode to NAK both IN and out transfers
;-----------------------------------------------------------------------------
export  `@INSTANCE_NAME`_EP1_ISR
export _`@INSTANCE_NAME`_EP1_ISR
 `@INSTANCE_NAME`_EP1_ISR:
_`@INSTANCE_NAME`_EP1_ISR:
   ;@PSoC_UserCode_BODY_EP1@ (Do not change this line.)
   ;---------------------------------------------------
   ; Insert your custom code below this banner
   ;---------------------------------------------------
   ;   NOTE: interrupt service routines must preserve
   ;   the values of the A and X CPU registers.
STD_EP1:    EQU     1   ; Set this equate to 0 to remove the standard
                        ; endpoint handling code
   ;---------------------------------------------------
   ; Insert your custom code above this banner
   ;---------------------------------------------------
   ;@PSoC_UserCode_END@ (Do not change this line.)

IF  STD_EP1
    PUSH A
    XOR     [`@INSTANCE_NAME`_EPDataToggle], 2 ; Update EP1 data toggle
    MOV     A, REG[EP1MODE]`@CTAB40`; Get the mode
    MOV     [`@INSTANCE_NAME`_EndpointAPIStatus+1], EVENT_PENDING`@CTAB40`; For the API
    POP A
    RETI
ENDIF
;-----------------------------------------------------------------------------
;  FUNCTION NAME: `@INSTANCE_NAME`_EP2_ISR
;
;  DESCRIPTION:    Handle the Endpoint 2 event by updating the data toggle
;                  and setting the endpoint state to EVENT_PENDING.  The SIE
;                  automatically set the mode to NAK both IN and out transfers
;-----------------------------------------------------------------------------
export  `@INSTANCE_NAME`_EP2_ISR
export _`@INSTANCE_NAME`_EP2_ISR
 `@INSTANCE_NAME`_EP2_ISR:
_`@INSTANCE_NAME`_EP2_ISR:
   ;@PSoC_UserCode_BODY_EP2@ (Do not change this line.)
   ;---------------------------------------------------
   ; Insert your custom code below this banner
   ;---------------------------------------------------
   ;   NOTE: interrupt service routines must preserve
   ;   the values of the A and X CPU registers.
STD_EP2:    EQU     1   ; Set this equate to 0 to remove the standard
                        ; endpoint handling code
   ;---------------------------------------------------
   ; Insert your custom code above this banner
   ;---------------------------------------------------
   ;@PSoC_UserCode_END@ (Do not change this line.)

IF  STD_EP2
    PUSH A
    XOR     [`@INSTANCE_NAME`_EPDataToggle], 4 ; Update EP2 data toggle
    MOV     A, REG[EP2MODE]`@CTAB40`; Get the mode
    MOV     [`@INSTANCE_NAME`_EndpointAPIStatus + 2], EVENT_PENDING`@CTAB40`; For the API
    POP A
    RETI
ENDIF
;-----------------------------------------------------------------------------
;  FUNCTION NAME: `@INSTANCE_NAME`_RESET_ISR
;
;  DESCRIPTION:    Handle the USB Bus Reset Interrupt
;-----------------------------------------------------------------------------
export  `@INSTANCE_NAME`_RESET_ISR
export _`@INSTANCE_NAME`_RESET_ISR
 `@INSTANCE_NAME`_RESET_ISR:
_`@INSTANCE_NAME`_RESET_ISR:

   ;@PSoC_UserCode_BODY_USB_RESET@ (Do not change this line.)
   ;---------------------------------------------------
   ; Insert your custom code below this banner
   ;---------------------------------------------------
   ;   NOTE: interrupt service routines must preserve
   ;   the values of the A and X CPU registers.

STD_USB_RESET:    EQU     1 ; Set this equate to 0 to remove the standard
                            ; USB reset handling code below

   ;---------------------------------------------------
   ; Insert your custom code above this banner
   ;---------------------------------------------------
   ;@PSoC_UserCode_END@ (Do not change this line.)

IF  STD_USB_RESET
    PUSH A
    PUSH X
    MOV     A, [`@INSTANCE_NAME`_bCurrentDevice]     ; Select the current device
    LCALL   _`@INSTANCE_NAME`_Start     ; Restart USB
    POP X
    POP A
ENDIF

    RETI
;-----------------------------------------------------------------------------
;  FUNCTION NAME: `@INSTANCE_NAME`_ACT_ISR
;
;  DESCRIPTION:    Handle the activity ISR
;
;  THEORY of OPERATION or PROCEDURE:
;
;   The activity interrupt sets a RAM flag indicating activity and disables the
;   interrupt.  Disabling the interrupt keeps the bus activity from creating too
;   many interrupts.  bCheckActivity checks and clears the flag, the enables
;   interrupts for the next interval.
;
;-----------------------------------------------------------------------------
export  `@INSTANCE_NAME`_ACT_ISR
export _`@INSTANCE_NAME`_ACT_ISR
 `@INSTANCE_NAME`_ACT_ISR:
_`@INSTANCE_NAME`_ACT_ISR:
    MOV    [`@INSTANCE_NAME`_bActivity], 1`@CTAB40`; Set the activity flag
    M8C_DisableIntMask INT_MSK1, INT_MSK1_USB_ACTIVITY
    RETI
; End of File `@INSTANCE_NAME`_std.asm
