;;*****************************************************************************
;;*****************************************************************************
;;  FILENAME: `@INSTANCE_NAME`_drv.asm
;;   Version: 1.5, Updated on 2006/06/19 at 11:43:56
;;  `@PSOC_VERSION`
;;
;;  DESCRIPTION: USB Device User Module control endpoint driver
;;               for the enCoRe II family of devices
;;
;;  NOTE: User Module APIs conform to the fastcall convention for marshalling
;;        arguments and observe the associated "Registers are volatile" policy.
;;        This means it is the caller's responsibility to preserve any values
;;        in the X and A registers that are still needed after the API
;;        function returns. Even though these registers may be preserved now,
;;        there is no guarantee they will be preserved in future releases.
;;-----------------------------------------------------------------------------
;;  Copyright (c) Cypress Semiconductor 2004. All Rights Reserved.
;;*****************************************************************************
;;*****************************************************************************

include "m8c.inc"
include "`@INSTANCE_NAME`_macros.inc"
include "`@INSTANCE_NAME`.inc"

;-----------------------------------------------
;  Global Symbols
;-----------------------------------------------
export  _`@INSTANCE_NAME`_EP0_ISR
export  `@INSTANCE_NAME`_InitControlRead
export  `@INSTANCE_NAME`_InitControlWrite
export  `@INSTANCE_NAME`_InitNoDataStageControlTransfer
export  `@INSTANCE_NAME`_NoDataStageControlTransfer

;-----------------------------------------------
;  Macro Definitions
;-----------------------------------------------

;-----------------------------------------------
;  Constant Definitions
;-----------------------------------------------

;-----------------------------------------------
; Variable Allocation
;-----------------------------------------------
AREA usbRAM (RAM,REL)
;----------------------------------------------------------------------------
; Current Device
;----------------------------------------------------------------------------
EXPORT `@INSTANCE_NAME`_bCurrentDevice
`@INSTANCE_NAME`_bCurrentDevice:`@CTAB40` BLK   1    ;  Current Device
;----------------------------------------------------------------------------
; Current Configuration
;----------------------------------------------------------------------------
EXPORT `@INSTANCE_NAME`_Configuration, _`@INSTANCE_NAME`_Configuration
_`@INSTANCE_NAME`_Configuration:
 `@INSTANCE_NAME`_Configuration:`@CTAB40` BLK   1    ;  Current Configuration
;----------------------------------------------------------------------------
; Current Device Status
;----------------------------------------------------------------------------
EXPORT `@INSTANCE_NAME`_DeviceStatus
 `@INSTANCE_NAME`_DeviceStatus:`@CTAB40` BLK   1    ;  Current Device Status
;----------------------------------------------------------------------------
; Interface Setting
;----------------------------------------------------------------------------
EXPORT `@INSTANCE_NAME`_InterfaceSetting
 `@INSTANCE_NAME`_InterfaceSetting:`@CTAB40` BLK   `@DSHID_bNumInterfaces`h    ; Interface Setting
;----------------------------------------------------------------------------
; Endpoint Status--USB Status
;----------------------------------------------------------------------------
EXPORT `@INSTANCE_NAME`_EndpointStatus
 `@INSTANCE_NAME`_EndpointStatus:`@CTAB40` BLK   USB_NUM_ENDPOINTS    ; Endpoint Status
;----------------------------------------------------------------------------
; Last Packet Size
;----------------------------------------------------------------------------
EXPORT `@INSTANCE_NAME`_LastSize
 `@INSTANCE_NAME`_LastSize:`@CTAB40` BLK   1    ;  Last Packet Size
;----------------------------------------------------------------------------
; Control Transfer State Machine
; State values for Control Write
; State values for Control Read
;----------------------------------------------------------------------------
EXPORT `@INSTANCE_NAME`_TransferType
 `@INSTANCE_NAME`_TransferType:`@CTAB40` BLK   1    ; Control Transfer State Machine
;----------------------------------------------------------------------------
; Control Transfer Intermediate Buffer--Shared among the requests
;----------------------------------------------------------------------------
EXPORT `@INSTANCE_NAME`_TransferBuffer
 `@INSTANCE_NAME`_TransferBuffer:`@CTAB40` BLK   8
;----------------------------------------------------------------------------
; Transfer Descriptor Data for Control Transfer
;  --The following data have the same format as the first 5 bytes of the TD_ENTRY
;----------------------------------------------------------------------------
; Control Transfer Data Source
;   USB_DS_ROM
;   USB_DS_RAM
;   USB_DS_RAM_AS_NEEDED
;----------------------------------------------------------------------------
EXPORT `@INSTANCE_NAME`_CurrentTD, _`@INSTANCE_NAME`_CurrentTD
 `@INSTANCE_NAME`_CurrentTD:
_`@INSTANCE_NAME`_CurrentTD:
EXPORT `@INSTANCE_NAME`_DataSource, _`@INSTANCE_NAME`_DataSource
 `@INSTANCE_NAME`_DataSource:
_`@INSTANCE_NAME`_DataSource: `@CTAB40`BLK   1
;----------------------------------------------------------------------------
; Control Transfer Data Size
;----------------------------------------------------------------------------
EXPORT `@INSTANCE_NAME`_TransferSize, _`@INSTANCE_NAME`_TransferSize
 `@INSTANCE_NAME`_TransferSize:
 _`@INSTANCE_NAME`_TransferSize: `@CTAB40`BLK   2
;----------------------------------------------------------------------------
; Control Transfer Data Pointer
;   Source for Control Read
;   Destination for Control Write
;----------------------------------------------------------------------------
EXPORT `@INSTANCE_NAME`_DataPtr, _`@INSTANCE_NAME`_DataPtr
 `@INSTANCE_NAME`_DataPtr:
 _`@INSTANCE_NAME`_DataPtr: `@CTAB40`BLK   2
;----------------------------------------------------------------------------
; Transfer Completion Notification
;----------------------------------------------------------------------------
EXPORT `@INSTANCE_NAME`_StatusBlockPtr, _`@INSTANCE_NAME`_StatusBlockPtr
 `@INSTANCE_NAME`_StatusBlockPtr:
 _`@INSTANCE_NAME`_StatusBlockPtr: `@CTAB40`BLK   2
;----------------------------------------------------------------------------
; Control Transfer _TransferByteCount (Actually transfered)
;----------------------------------------------------------------------------
 `@INSTANCE_NAME`_TransferByteCount:            `@CTAB40` BLK   2
;----------------------------------------------------------------------------
; Control Endpoint Data toggle
EXPORT `@INSTANCE_NAME`_EPDataToggle
 `@INSTANCE_NAME`_EPDataToggle:
 `@INSTANCE_NAME`_EP0DataToggle:  `@CTAB40` BLK   1
;----------------------------------------------------------------------------
; Control Endpoint Data Pending Flag
EXPORT `@INSTANCE_NAME`_fDataPending
 `@INSTANCE_NAME`_fDataPending:  `@CTAB40` BLK   1
;----------------------------------------------------------------------------
; Control Endpoint Data Pending Flag
;EXPORT `@INSTANCE_NAME`_PendingData
; `@INSTANCE_NAME`_PendingData:  `@CTAB40` BLK   1
;----------------------------------------------------------------------------
; Temporary Data registers
EXPORT `@INSTANCE_NAME`_t2, `@INSTANCE_NAME`_t1, `@INSTANCE_NAME`_t0
 `@INSTANCE_NAME`_t2:`@CTAB40` BLK   1    ; Temporary shared by the UM
 `@INSTANCE_NAME`_t1:`@CTAB40` BLK   1    ; Temporary shared by the UM
 `@INSTANCE_NAME`_t0:`@CTAB40` BLK   1    ; Temporary shared by the UM
;----------------------------------------------------------------------------
; Endpoint Transfer--API Status
;----------------------------------------------------------------------------
EXPORT `@INSTANCE_NAME`_EndpointAPIStatus, _`@INSTANCE_NAME`_EndpointAPIStatus
 `@INSTANCE_NAME`_EndpointAPIStatus:
_`@INSTANCE_NAME`_EndpointAPIStatus:`@CTAB40` BLK   USB_NUM_ENDPOINTS    ; Endpoint Status

AREA Bootloader (ROM, REL)
;-----------------------------------------------------------------------------
;  FUNCTION NAME: `@INSTANCE_NAME`_EP0_ISR

;
;  DESCRIPTION:   The EPO ISR serves the control endpoint interrupts and
;                 dispaches all SETUP, IN, and OUT transfers to the proper
;                 dispatch routines for all supported USB requests.
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:   n/a
;
;  RETURNS:     n/a
;
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;  THEORY of OPERATION or PROCEDURE:
;
;-----------------------------------------------------------------------------
 `@INSTANCE_NAME`_EP0_ISR:
_`@INSTANCE_NAME`_EP0_ISR:
    PUSH    A`@CTAB40`; Save the context
    PUSH    X`@CTAB40`;
    ; Dispatch to setup/in/out handlers
EXPORT  `@INSTANCE_NAME`_mode_read
`@INSTANCE_NAME`_mode_read:
   MOV    A, REG[`@INSTANCE_NAME`_EP0MODE]`@CTAB40` ; Get the mode reg

; MSB is the SETUP bit, followed by IN, then OUT
    asl    a `@CTAB40`; Shift to the carry and jump if SETUP bit set
    jc      `@INSTANCE_NAME`_EP0_Setup

    asl    a `@CTAB40`; Shift to the carry and jump if IN bit set
    jc      `@INSTANCE_NAME`_EP0_IN

    asl    a `@CTAB40`; Shift to the carry and jump if OUT bit set
    jc     `@INSTANCE_NAME`_EP0_OUT
    JMP    `@INSTANCE_NAME`_Not_Supported_Local_Drv
; ISR Exit Point to update the mode register
;   mode and count have been pushed onto the stack
EXPORT `@INSTANCE_NAME`_EP0_UPD_MODE_EXIT
`@INSTANCE_NAME`_EP0_UPD_MODE_EXIT:

    MOV    REG[`@INSTANCE_NAME`_EP0CNT], A`@CTAB40`; Update the count
    MOV    A, X`@CTAB40`; Get the new mode
    MOV    REG[`@INSTANCE_NAME`_EP0MODE], A`@CTAB40`; Update the node

`@INSTANCE_NAME`_EP0_ISR_EXIT:

    POP X`@CTAB40`;
    POP A`@CTAB40`; Restore Context
    RETI
;-----------------------------------------------------------------------------
;  FUNCTION NAME: `@INSTANCE_NAME`_EP0_Setup
;
;  DESCRIPTION:   Dispatch a USB SETUP
;
;-----------------------------------------------------------------------------
 `@INSTANCE_NAME`_EP0_Setup:
_`@INSTANCE_NAME`_EP0_Setup:
; Check the byte count and validity.  All SETUP are 8 bytes and 0 toggle
    PUSH    A`@CTAB40`; Save the mode register
    MOV     A, USB_XFER_PREMATURE`@CTAB40`; Return a Premature Completion?
    CALL    `@INSTANCE_NAME`_UpdateStatusBlock
    POP     A`@CTAB40`; Restore the mode register
    MOV    A, REG[`@INSTANCE_NAME`_EP0CNT] `@CTAB45` ; Get the count reg
    CMP    A, (USB_CNT_VALID | 0x0A)
    JZ      .dispatch

    JMP    `@INSTANCE_NAME`_Not_Supported_Local_Drv
;-----------------------------------------------------------------------------
; Jump here to dispatch the request
; The SETUP request is encoded in [bmRequestType]. Among the 8 bits in [bmRequestType], only bits
; 7,6,5,1,0 determine what the request is. Bits [2:4] are default to zero. The below code
; re-organizes [bmRequestType] to the following format:
; ( Zero, Zero, Bit7, Bit6, Bit5, Bit1, Bit0, Zero ), and depending on the value of this
; "re-organization", the firmware will jump to an appropriate table to handle the request.
;-----------------------------------------------------------------------------
.dispatch:
    MOV     A, REG[`@INSTANCE_NAME`_EP0DATA+bmRequestType] `@CTAB45` ; Get bmRequestType
    AND     A, E3h         `@CTAB45` ; clear bits 4-3-2, these unused for our purposes
    PUSH    A              `@CTAB45` ; store value on the stack
    ASR     A              `@CTAB45` ; move bits 7-6-5 into 4-3-2's place
    ASR     A              `@CTAB45` ; "asr" instruction shift all bits one place to the right.
    ASR     A              `@CTAB45` ; Bit7 remains the same.
    MOV     [`@INSTANCE_NAME`_t2], A  `@CTAB45` ; store shifted value
    POP     A              `@CTAB45` ; get original value
    OR      A, [`@INSTANCE_NAME`_t2]  `@CTAB45` ; or the two to get the 5-bit field
    AND     A, 1Fh         `@CTAB45` ; clear bits 7-6-5 (asr wraps bit7)
    `@CTAB45` ; Bit0 is loaded with a Zero. This results in multiplying
    `@CTAB45` ; the accumulator by 2, and the reason to mutiply it by 2
    `@CTAB45` ; is that each "jmp" instruction in the tables is two bytes long.

    LJMP `@INSTANCE_NAME`_bmRequestType_Dispatch
;-----------------------------------------------------------------------------
;  FUNCTION NAME: `@INSTANCE_NAME`_EP0_IN
;
;  DESCRIPTION:   Dispatch a USB IN
;
;-----------------------------------------------------------------------------
 `@INSTANCE_NAME`_EP0_IN:
_`@INSTANCE_NAME`_EP0_IN:

    MOV     A, [`@INSTANCE_NAME`_TransferType]
`@INSTANCE_NAME`_ControlInDispatch:
    JACC    `@INSTANCE_NAME`_ControlInDispatchTable
`@INSTANCE_NAME`_ControlInDispatchTable:
    JMP     `@INSTANCE_NAME`_Not_Supported_Local_Drv  ; USB_TRANS_STATE_IDLE
    JMP     `@INSTANCE_NAME`_ControlReadDataStage     ; USB_TRANS_STATE_CONTROL_READ
    JMP     `@INSTANCE_NAME`_ControlWriteStatusStage  ; USB_TRANS_STATE_CONTROL_WRITE
    JMP     `@INSTANCE_NAME`_NoDataControlStatusStage ; USB_TRANS_STATE_NO_DATA_CONTROL
;-----------------------------------------------------------------------------
;  FUNCTION NAME: `@INSTANCE_NAME`_EP0_OUT
;
;  DESCRIPTION:   Dispatch a USB OUT
;
;-----------------------------------------------------------------------------
 `@INSTANCE_NAME`_EP0_OUT:
_`@INSTANCE_NAME`_EP0_OUT:
    MOV     A, [`@INSTANCE_NAME`_TransferType]
    JACC    `@INSTANCE_NAME`_ControlOutDispatchTable
`@INSTANCE_NAME`_ControlOutDispatchTable:
    JMP     `@INSTANCE_NAME`_Not_Supported_Local_Drv  ; USB_TRANS_STATE_IDLE
    JMP     `@INSTANCE_NAME`_ControlReadStatusStage   ; USB_TRANS_STATE_CONTROL_READ
    JMP     `@INSTANCE_NAME`_ControlWriteDataStage    ; USB_TRANS_STATE_CONTROL_WRITE
    JMP     `@INSTANCE_NAME`_NoDataControlError       ; USB_TRANS_STATE_NO_DATA_CONTROL
;-----------------------------------------------------------------------------
;  FUNCTION NAME: `@INSTANCE_NAME`_InitControlRead
;
;  DESCRIPTION:   This routine initializes a control read.  It must be JUMPed to,
;                 not called.  It assumes a tranfer descriptor has been loaded
;                 into the driver `@INSTANCE_NAME`_CurrentTD data structure.
;
;-----------------------------------------------------------------------------
 `@INSTANCE_NAME`_InitControlRead:
    MOV     [`@INSTANCE_NAME`_LastSize], A`@CTAB40`; Save the packet size?
    CALL    `@INSTANCE_NAME`_InitializeStatusBlock
    MOV     [`@INSTANCE_NAME`_TransferType], USB_TRANS_STATE_CONTROL_READ

    ; Check the transfer size against the request size
    MOV    A, REG[`@INSTANCE_NAME`_EP0DATA+wLengthHi]`@CTAB40`; MSB of wLength
    CMP    A, [`@INSTANCE_NAME`_TransferSize]
    JNZ    .L1

    MOV    A, REG[`@INSTANCE_NAME`_EP0DATA+wLengthLo]`@CTAB40`; LSB of wLength
    CMP    A, [`@INSTANCE_NAME`_TransferSize+1]
    JZ     .L9
.L1:
    JNC    .L9
;
    MOV    [`@INSTANCE_NAME`_TransferSize+1], A`@CTAB40`;
    MOV    A, REG[`@INSTANCE_NAME`_EP0DATA+wLengthHi]`@CTAB40`;
    MOV    [`@INSTANCE_NAME`_TransferSize], A`@CTAB40`;
.L9:
    MOV    [`@INSTANCE_NAME`_TransferByteCount], 0`@CTAB40`;
    MOV    [`@INSTANCE_NAME`_TransferByteCount+1], 0`@CTAB40`;

    OR     [`@INSTANCE_NAME`_EP0DataToggle], 1 ; setup EP0 data toggle
    JMP    `@INSTANCE_NAME`_LoadEndpoint`@CTAB40`;
;-----------------------------------------------------------------------------
;  FUNCTION NAME: `@INSTANCE_NAME`_ControlReadDataStage
;
;  DESCRIPTION:   This routine processes the data stage of a control read.  It
;                 must be JUMPed to, not called.  It assumes a tranfer descriptor
;                 has been loaded into the driver `@INSTANCE_NAME`_CurrentTD
;                 data structure.
;
;-----------------------------------------------------------------------------
 `@INSTANCE_NAME`_ControlReadDataStage:
    JMP     `@INSTANCE_NAME`_LoadEndpoint

;-----------------------------------------------------------------------------
;  FUNCTION NAME: `@INSTANCE_NAME`_ControlReadStatusStage
;
;  DESCRIPTION:   This routine processes the status stage of a control read.  It
;                 must be JUMPed to, not called.  It handles short or 0 packet
;                 It assumes a tranfer descriptor has been loaded into the
;                 driver `@INSTANCE_NAME`_CurrentTD data structure.
;
;-----------------------------------------------------------------------------
 `@INSTANCE_NAME`_ControlReadStatusStage:
    MOV    A, [`@INSTANCE_NAME`_LastSize]`@CTAB40`; Get the number of bytes from the last transfer
    ADD    [`@INSTANCE_NAME`_TransferByteCount + 1], A`@CTAB40`; Update the transfer byte count
    ADC    [`@INSTANCE_NAME`_TransferByteCount], 0`@CTAB40`;
    MOV    A, USB_XFER_STATUS_ACK`@CTAB40`; Return a Status ACK Completion
    CALL   `@INSTANCE_NAME`_UpdateStatusBlock
    MOV    [`@INSTANCE_NAME`_TransferType], USB_TRANS_STATE_IDLE
    MOV    A, 0`@CTAB40`; Count Register
    MOV    X, USB_MODE_STALL_IN_OUT
    JMP    `@INSTANCE_NAME`_EP0_UPD_MODE_EXIT
;-----------------------------------------------------------------------------
;  FUNCTION NAME: `@INSTANCE_NAME`_NoDataStageControlTransfer
;                 `@INSTANCE_NAME`_InitNoDataStageControlTransfer
;
;  DESCRIPTION:   This routine processes the status stage of a no data control
;                 write.  It must be JUMPed to, not called.
;
;-----------------------------------------------------------------------------
 `@INSTANCE_NAME`_NoDataStageControlTransfer:
 `@INSTANCE_NAME`_InitNoDataStageControlTransfer:
    CALL    `@INSTANCE_NAME`_InitializeStatusBlock

    MOV    [`@INSTANCE_NAME`_TransferType], USB_TRANS_STATE_NO_DATA_CONTROL

    MOV    A, 0`@CTAB40`; Count Register
    MOV    X, USB_MODE_STATUS_IN_ONLY
    JMP    `@INSTANCE_NAME`_EP0_UPD_MODE_EXIT
;-----------------------------------------------------------------------------
;  FUNCTION NAME: `@INSTANCE_NAME`_InitControlWrite
;
;  DESCRIPTION:   This routine initializes control write.  It must be JUMPed
;                 to, not called.  It assumes a tranfer descriptor has been loaded
;                 into the driver `@INSTANCE_NAME`_CurrentTD data structure.
;
;-----------------------------------------------------------------------------
 `@INSTANCE_NAME`_InitControlWrite:
    MOV     A, [`@INSTANCE_NAME`_DataSource] `@CTAB40`; Need to make sure the destination is not ROM
    CMP     A, USB_DS_ROM
    JZ      `@INSTANCE_NAME`_Not_Supported_Local_Drv

    CALL    `@INSTANCE_NAME`_InitializeStatusBlock

    MOV    [`@INSTANCE_NAME`_TransferType], USB_TRANS_STATE_CONTROL_WRITE

    OR     [`@INSTANCE_NAME`_EP0DataToggle], 1 ; setup EP0 data toggle

    MOV    A, 0`@CTAB40`; Count Register
    MOV    X, USB_MODE_ACK_OUT_STATUS_IN
    JMP    `@INSTANCE_NAME`_EP0_UPD_MODE_EXIT
;-----------------------------------------------------------------------------
;  FUNCTION NAME: `@INSTANCE_NAME`_ControlWriteDataStage
;
;  DESCRIPTION:   This routine processes the data stage of a control
;                 write.  It must be JUMPed to, not called.  It assumes a
;                 tranfer descriptor has been loaded into the driver
;                 `@INSTANCE_NAME`_CurrentTD data structure.
;
;-----------------------------------------------------------------------------
 `@INSTANCE_NAME`_ControlWriteDataStage:
    XOR    [`@INSTANCE_NAME`_EP0DataToggle], 1 `@CTAB40`; Update data toggle

    MOV    A,REG[`@INSTANCE_NAME`_EP0CNT]`@CTAB40`; Get the count

    AND     A, 0x0F
    SUB     A, 2`@CTAB40`; Count include the two byte checksum

    MOV     [`@INSTANCE_NAME`_t2], A `@CTAB40`; Assume we have room to receive the whole packet

    MOV     A, 0
    CMP     A, [`@INSTANCE_NAME`_TransferSize] `@CTAB40`; If the MSB has anything just use the count
    JNZ     .L1

    MOV     A, [`@INSTANCE_NAME`_t2]
    CMP     A, [`@INSTANCE_NAME`_TransferSize+1] `@CTAB40`;
    JZ      .L6
.L1:
    JC      .L6
    MOV     [`@INSTANCE_NAME`_t2], [`@INSTANCE_NAME`_TransferSize+1]

.L6:
    MOV     A, [`@INSTANCE_NAME`_t2]
    SUB     [`@INSTANCE_NAME`_TransferSize+1],A  `@CTAB40`; Update the bytes remaining
    SBB     [`@INSTANCE_NAME`_TransferSize], 0 `@CTAB40`;

    ADD     [`@INSTANCE_NAME`_TransferByteCount + 1], A`@CTAB40`; Update the transfer byte count
    ADC     [`@INSTANCE_NAME`_TransferByteCount], 0`@CTAB40`;

    MOV     X,0 `@CTAB40`; Start the index at 0

.RAM_COPY:
    MOV     A, REG[X+`@INSTANCE_NAME`_EP0DATA] `@CTAB40`; Get the data
    MVI     [`@INSTANCE_NAME`_DataPtr+1], A `@CTAB40`; Store the data, bump the destination

    INC     X   `@CTAB40`; Bump the destination offset
    MOV     A,X `@CTAB40`; Are we done?
    CMP     A, [`@INSTANCE_NAME`_t2]
    JC      .RAM_COPY `@CTAB40`; Not done

    MOV    A, 0`@CTAB40`; Count Register
    MOV    X, USB_MODE_ACK_OUT_STATUS_IN
    JMP    `@INSTANCE_NAME`_EP0_UPD_MODE_EXIT

; Jump here on data toggle error
.error:
    MOV    [`@INSTANCE_NAME`_TransferType], USB_TRANS_STATE_IDLE `@CTAB40`; This simply aborts the transfer
    MOV    A, 0`@CTAB40`; Count Register
    MOV    X, USB_MODE_STALL_IN_OUT
    JMP    `@INSTANCE_NAME`_EP0_UPD_MODE_EXIT
;-----------------------------------------------------------------------------
;  FUNCTION NAME: `@INSTANCE_NAME`_ControlWriteStatusStage
;
;  DESCRIPTION:   This routine processes the status stage of a control
;                 write.  It must be JUMPed to, not called.  It assumes a
;                 tranfer descriptor has been loaded into the driver
;                 `@INSTANCE_NAME`_CurrentTD data structure.
;
;-----------------------------------------------------------------------------
 `@INSTANCE_NAME`_ControlWriteStatusStage:
    MOV    A, USB_XFER_STATUS_ACK`@CTAB40`; Return a Status ACK Completion
    CALL   `@INSTANCE_NAME`_UpdateStatusBlock
    MOV    [`@INSTANCE_NAME`_TransferType], USB_TRANS_STATE_IDLE `@CTAB40`; The packet is done
    MOV    A, 0`@CTAB40`; Count Register
    MOV    X, USB_MODE_STALL_IN_OUT
    JMP    `@INSTANCE_NAME`_EP0_UPD_MODE_EXIT
;-----------------------------------------------------------------------------
;  FUNCTION NAME: `@INSTANCE_NAME`_NoDataControlStatusStage
;
;  DESCRIPTION:   This routine processes the status stage of a control
;                 write.  It must be JUMPed to, not called.  It assumes a
;                 tranfer descriptor has been loaded into the driver
;                 `@INSTANCE_NAME`_CurrentTD data structure.
;
;                 USB Device Addressing happens here because we can't change
;                 the SIE Address before the Status IN is received.
;
;-----------------------------------------------------------------------------
 `@INSTANCE_NAME`_NoDataControlStatusStage:
    MOV     A, USB_XFER_STATUS_ACK`@CTAB40`; Return a Status ACK Completion
    CALL    `@INSTANCE_NAME`_UpdateStatusBlock
    ; Dispatch to the proper handler
    CMP     [`@INSTANCE_NAME`_fDataPending], USB_ADDRESS_CHANGE_PENDING
    JNZ     .L1

    ; USB ADDRESS CHANGE
    MOV     A, [`@INSTANCE_NAME`_TransferBuffer] `@CTAB40`; Get the pending data

    OR      A, USB_ADDR_ENABLE `@CTAB40`; Set the enable bit
    MOV     REG[`@INSTANCE_NAME`_ADDR], A `@CTAB40`; Update the SIE address
    JMP     .EXIT
.L1:

.EXIT:
    MOV     [`@INSTANCE_NAME`_fDataPending], 0 `@CTAB40`; Clear data pending

    MOV    A, 0`@CTAB40`; Count Register
    MOV    X, USB_MODE_STATUS_IN_ONLY`@CTAB40`; Wait for the next SETUP
    JMP    `@INSTANCE_NAME`_EP0_UPD_MODE_EXIT
;-----------------------------------------------------------------------------
;  FUNCTION NAME: `@INSTANCE_NAME`_NoDataControlError
;
;  DESCRIPTION:   This routine handles the condition when we expected a
;                 status IN, but receive an OUT
;
;-----------------------------------------------------------------------------
 `@INSTANCE_NAME`_NoDataControlError:
    MOV    A, USB_XFER_ERROR`@CTAB40`; Return Transaction Error
    CALL   `@INSTANCE_NAME`_UpdateStatusBlock
    MOV    A, 0`@CTAB40`; Count Register
    MOV    X, USB_MODE_STALL_IN_OUT `@CTAB40`; Set the mode register
    JMP    `@INSTANCE_NAME`_EP0_UPD_MODE_EXIT
;-----------------------------------------------------------------------------
;  FUNCTION NAME: `@INSTANCE_NAME`_LoadEndpoint
;
;  DESCRIPTION:  Load the endpoint buffer with the next data for the control
;                transfer.  It assumes a tranfer descriptor has been loaded
;                into the driver `@INSTANCE_NAME`_CurrentTD data structure.
;
;-----------------------------------------------------------------------------
`@INSTANCE_NAME`_LoadEndpoint:
    MOV     A, [`@INSTANCE_NAME`_LastSize]`@CTAB40`; Get the number of bytes from the last transfer
    ADD     [`@INSTANCE_NAME`_TransferByteCount + 1], A`@CTAB40`; Update the transfer byte count
    ADC     [`@INSTANCE_NAME`_TransferByteCount], 0`@CTAB40`;
    MOV     A,  [`@INSTANCE_NAME`_TransferSize]`@CTAB40`; Check to see if we have any
    OR      A,  [`@INSTANCE_NAME`_TransferSize+1]`@CTAB40`;  more data to send
    JNZ     .cont `@CTAB40`; Jump if we have to send more data

; Flow here if there is no more data to send
    CMP     [`@INSTANCE_NAME`_LastSize], 8`@CTAB40`; Was it a full packet?
    JZ      .START_TRANSFER`@CTAB40`; Jump if it was full (need to send a zero length)

; Flow here if we are entering the status stage
    MOV    A, 0`@CTAB40`; Count Register
    MOV    [`@INSTANCE_NAME`_LastSize], A`@CTAB40`; Clear the byte count
    MOV    X, USB_MODE_STATUS_OUT_ONLY`@CTAB40`; Only ACK the Status Out
    JMP    `@INSTANCE_NAME`_EP0_UPD_MODE_EXIT

; Jump here to determine how many bytes should we transfer
.cont:
    CMP     [`@INSTANCE_NAME`_TransferSize], 0 `@CTAB40`; Check the MSB
    JNZ     .L1

    CMP     [`@INSTANCE_NAME`_TransferSize+1], 8 `@CTAB40`; Check the LSB
    JNC     .L1

    MOV     A,[`@INSTANCE_NAME`_TransferSize+1]  `@CTAB40`; Transfer all the remaing data
    JMP     .L3

.L1:
    MOV     A, 8     `@CTAB40`; Just transfer the next 8 bytes

.L3:

    SUB     [`@INSTANCE_NAME`_TransferSize+1],A  `@CTAB40`; Update the bytes remaining
    SBB     [`@INSTANCE_NAME`_TransferSize],0

    MOV     [`@INSTANCE_NAME`_t2],A       `@CTAB40`; Save the count
    MOV     X,0

    CMP     [`@INSTANCE_NAME`_DataSource],USB_DS_ROM `@CTAB40`; RAM or ROM copy?
    JNZ     .RAM_COPY

; Copy data from a ROM source
.ROM_COPY:
    PUSH    X `@CTAB40`; Save the destination offset
    MOV     A,[`@INSTANCE_NAME`_DataPtr]    `@CTAB40`; Get the transfer source MSB
    MOV     X,[`@INSTANCE_NAME`_DataPtr+1]  `@CTAB40`; Set the transfer source LSB
    INC     [`@INSTANCE_NAME`_DataPtr+1]    `@CTAB40`; Increment the data pointer
    ADC     [`@INSTANCE_NAME`_DataPtr], 0   `@CTAB40`;   MSB if necessary

    ROMX    `@CTAB40`; Get the data byte

    POP     X   `@CTAB40`; Get the destination offset
    MOV     REG[X + `@INSTANCE_NAME`_EP0DATA], A`@CTAB40`; Load the data
    INC     X   `@CTAB40`; Bump the destination offset
    MOV     A,X `@CTAB40`; Are we done?
    CMP     A, [`@INSTANCE_NAME`_t2]
    JC      .ROM_COPY `@CTAB40`; Not done
    JMP     .START_TRANSFER `@CTAB40`; Otherwise go start the transfer

; Copy data from a RAM source
.RAM_COPY:
    MVI     A, [`@INSTANCE_NAME`_DataPtr+1] `@CTAB40`; Get the data, bump the source

    MOV     REG[X +`@INSTANCE_NAME`_EP0DATA], A`@CTAB40`; Load the data
    INC     X   `@CTAB40`; Bump the destination offset
    MOV     A,X `@CTAB40`; Are we done?
    CMP     A, [`@INSTANCE_NAME`_t2]
    JC      .RAM_COPY `@CTAB40`; Not done

;; Set up the IN transfer count/mode/etc
;    A contains the byte count
.START_TRANSFER:
    MOV     [`@INSTANCE_NAME`_LastSize], A`@CTAB40`; Save the packet size

    MOV     A, 1
    AND     A, [`@INSTANCE_NAME`_EP0DataToggle]
    JZ      .BYPASS_T1
    MOV     A, USB_CNT_TOGGLE `@CTAB40`; Or T1 in the data toggle

.BYPASS_T1:
    OR      A, [`@INSTANCE_NAME`_LastSize]
    XOR     [`@INSTANCE_NAME`_EP0DataToggle], 1   `@CTAB40`; Update the data toggle for next time

    MOV    X, USB_MODE_ACK_IN_STATUS_OUT `@CTAB40`; Set the mode register
    JMP    `@INSTANCE_NAME`_EP0_UPD_MODE_EXIT
;-----------------------------------------------------------------------------
;  FUNCTION NAME: `@INSTANCE_NAME`_GetTableEntry
;
;  DESCRIPTION:   Copies a TD to the `@INSTANCE_NAME`_CurrentTD data structure.
;
;-----------------------------------------------------------------------------
EXPORT `@INSTANCE_NAME`_GetTableEntry
`@INSTANCE_NAME`_GetTableEntry:

    INC     X `@CTAB40`; Point to the first table entry
    ADC     A, 0 `@CTAB40`;

    TD_INDEX_TO_OFFSET `@INSTANCE_NAME`_t2 ; Convert the index

    SWAP    A, X
    ADD     A, [`@INSTANCE_NAME`_t2]
    SWAP    A, X
    ADC     A, 0 `@CTAB40`; A:X now points to the descriptor table entry we want

; Flow here to load the Transfer Descriptor (TD_ENTRY)
    MOV     [`@INSTANCE_NAME`_t2], `@INSTANCE_NAME`_CurrentTD `@CTAB40`; Use Temp as MVI pointer
    CALL     `@INSTANCE_NAME`_GETBYTE`@CTAB40`; Get the descriptor data source
    CALL     `@INSTANCE_NAME`_GETWORD`@CTAB40`; Get the descriptor size
    CALL     `@INSTANCE_NAME`_GETWORD`@CTAB40`; Get the descriptor address
    CALL     `@INSTANCE_NAME`_GETWORD`@CTAB40`; Get the Status Pointer
; Dispatch to InitControlRead or InitControlWrite based on d2h/h2d in the request
    MOV    A, REG[`@INSTANCE_NAME`_EP0DATA+bmRequestType]`@CTAB45`; Get bmRequestType
    AND     A,0x80 `@CTAB45`; Control Read or Write
    JZ      .control_write

    JMP     `@INSTANCE_NAME`_InitControlRead

.control_write:
    JMP     `@INSTANCE_NAME`_InitControlWrite

;-----------------------------------------------------------------------------
;  FUNCTION NAME: `@INSTANCE_NAME`_LOOKUP
;
;  DESCRIPTION:    Returns the address of an entry in a lookup table (LT_ENTRY)
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:    A:X Point to the lookup table
;                `@INSTANCE_NAME`_t2 contain the table index
;
;  RETURNS:      Address of the LT_ENTRY in A:X
;
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;  THEORY of OPERATION or PROCEDURE:
;
;-----------------------------------------------------------------------------
EXPORT `@INSTANCE_NAME`_LOOKUP
`@INSTANCE_NAME`_LOOKUP:
    INC     X `@CTAB40`; Point to the first table entry
    ADC     A, 0 `@CTAB40`;

    LT_INDEX_TO_OFFSET `@INSTANCE_NAME`_t2`@CTAB40`; Convert the index
    SWAP    A, X
    ADD     A, [`@INSTANCE_NAME`_t2]`@CTAB40`;
    SWAP    A, X
    ADC     A, 0
    RET
;-----------------------------------------------------------------------------
;  FUNCTION NAME: `@INSTANCE_NAME`_GETWORD/`@INSTANCE_NAME`_GETBYTE
;
;  DESCRIPTION:    Get a word value from ROM
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:    A:X is the ROM Address
;                `@INSTANCE_NAME`_t2 is the destination address
;
;  RETURNS:      `@INSTANCE_NAME`_t1
;
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;                USES `@INSTANCE_NAME`_t2
;                A:X points to the subsequent locaction ROM location
;  THEORY of OPERATION or PROCEDURE:
;
;-----------------------------------------------------------------------------
EXPORT `@INSTANCE_NAME`_GETWORD
`@INSTANCE_NAME`_GETWORD:

    PUSH    A  `@CTAB40`; Don't loose the pointer MSB
    ROMX       `@CTAB40`; Data source flag
    MVI     [`@INSTANCE_NAME`_t2], A   `@CTAB40`; Save the data source
    POP     A `@CTAB40`; Get the MSB back
    INC     X `@CTAB40`; Point to the next  entry
    ADC     A, 0 `@CTAB40`;

EXPORT `@INSTANCE_NAME`_GETBYTE
`@INSTANCE_NAME`_GETBYTE:

    PUSH    A  `@CTAB40`; Don't loose the pointer MSB
    ROMX       `@CTAB40`; Data source flag
    MVI     [`@INSTANCE_NAME`_t2], A   `@CTAB40`; Save the data source
    POP     A `@CTAB40`; Get the MSB back
    INC     X `@CTAB40`; Point to the next  entry
    ADC     A, 0 `@CTAB40`;
    RET
;-----------------------------------------------------------------------------
;  FUNCTION NAME: `@INSTANCE_NAME`_GET_TABLE_PTR
;
;  DESCRIPTION:    Get the address of one of three tables via a table 
;                  fixed at address 100h.
;                  Not intended for use by C fucntions
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:      A is the index of 0 - 2
;                  0: _DEVICE_LOOKUP
;                  1: _DEVICE_DESCR_TABLE
;                  2: _StringTable
;
;  RETURNS:        A:X points to the table
;
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;  THEORY of OPERATION or PROCEDURE:
;
;-----------------------------------------------------------------------------
EXPORT `@INSTANCE_NAME`_GET_TABLE_PTR
`@INSTANCE_NAME`_GET_TABLE_PTR:
    ASL     A `@CTAB40`; Adjust for word pointers
    MOV     X, A `@CTAB40`; Table is located at 100h
    MOV     A, >USB_VECTOR_TABLE
    ROMX `@CTAB40`; Get LSB and save to stack
    PUSH    A
    INC     X `@CTAB40`; Next byte
    MOV     A, >USB_VECTOR_TABLE
    ROMX `@CTAB40`; Get MSB and save to stack
    POP     X
    SWAP    A, X `@CTAB40`; Adjust MSB/LSB byte positions
    RET
;-----------------------------------------------------------------------------
;  FUNCTION NAME: `@INSTANCE_NAME`_GET_DEVICE_TABLE_ENTRY
;
;  DESCRIPTION:    Get the address of the current DEVICE_TABLE entry
;                  Not intended for use by C fucntions
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:
;
;  RETURNS:        A:X points the the current DEVICE_TABLE entry
;                  Carry flag is set if the current device index is out of range
;
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;  THEORY of OPERATION or PROCEDURE:
;
;-----------------------------------------------------------------------------
EXPORT `@INSTANCE_NAME`_GET_DEVICE_TABLE_ENTRY
`@INSTANCE_NAME`_GET_DEVICE_TABLE_ENTRY:
    MOV     [`@INSTANCE_NAME`_t2], [`@INSTANCE_NAME`_bCurrentDevice] `@CTAB40`; Use the UM temp var--Selector
    MOV     A, 0 `@CTAB40`; Get pointer to `@INSTANCE_NAME`_DEVICE_LOOKUP
    CALL    `@INSTANCE_NAME`_GET_TABLE_PTR
    PUSH    A
    ROMX      `@CTAB40`; First entry is the table size (only a byte)
    CMP     A, [`@INSTANCE_NAME`_t2] `@CTAB40`; Range check
    POP     A
    JC      .exit
; Flow here if the index is valid
    CALL    `@INSTANCE_NAME`_LOOKUP `@CTAB40`; Look up the configuration
; Jump or flow here on exit
.exit:
    RET
;-----------------------------------------------------------------------------
;  FUNCTION NAME: `@INSTANCE_NAME`_GET_CONFIG_TABLE_ENTRY
;
;  DESCRIPTION:    Get the address of the current DEVICE_TABLE entry
;                  Not intended for use by C fucntions
;                  Does not do range checking on
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:
;
;  RETURNS:        A:X points the the current CONFIG_TABLE entry
;                  Carry flag is set if the current device index is out of range
;
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;  THEORY of OPERATION or PROCEDURE:
;
;-----------------------------------------------------------------------------
EXPORT `@INSTANCE_NAME`_GET_CONFIG_TABLE_ENTRY
`@INSTANCE_NAME`_GET_CONFIG_TABLE_ENTRY:
    CALL    `@INSTANCE_NAME`_GET_DEVICE_TABLE_ENTRY `@CTAB40`; Get the selected device
    MOV     [`@INSTANCE_NAME`_t2],`@INSTANCE_NAME`_t1 `@CTAB40`; Set the GETWORD destination
    CALL    `@INSTANCE_NAME`_GETWORD `@CTAB40`; Get the pointer to the CONFIG_LOOKUP table
                                     `@CTAB40`; ITempW has the address
    MOV     A, REG[`@INSTANCE_NAME`_EP0DATA+wValueLo] `@CTAB40`; Get the configuration number
    MOV     [`@INSTANCE_NAME`_t2],A `@CTAB40`; Save it
    MOV     A, [`@INSTANCE_NAME`_t1] `@CTAB40`; Get the CONFIG_LOOKUP ROM Address MSB
    MOV     X, [`@INSTANCE_NAME`_t1+1] `@CTAB40`; Get the CONFIG_LOOKUP ROM Address LSB

; A:X Points to the CONFIG_LOOKUP, so get the current entry
    MOV     [`@INSTANCE_NAME`_t2], [`@INSTANCE_NAME`_Configuration]`@CTAB40`; Get the configuration number
    DEC     [`@INSTANCE_NAME`_t2]`@CTAB40`; We don't populate the 0th entry
    CALL    `@INSTANCE_NAME`_LOOKUP `@CTAB40`; Look up the configuration
    RET
;-----------------------------------------------------------------------------
;  FUNCTION NAME: `@INSTANCE_NAME`_UpdateStatusBlock
;
;  DESCRIPTION:    Update the Completion Status Block for a Request.  The
;                  block is updated with the completion code from the
;                  argument (A) and the _TransferByteCount.
;
;                  The StatusBlock Pointer (_StatusBlockPtr) is set to NULL (0)
;                  to make sure no other updates are made to the StatusBlock by
;                  the USB User Module.
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:      A contains the Completion Status Code
;
;  RETURNS:        None
;
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;  THEORY of OPERATION or PROCEDURE:
;
;-----------------------------------------------------------------------------
`@INSTANCE_NAME`_UpdateStatusBlock:
    MOV     X, [`@INSTANCE_NAME`_StatusBlockPtr + 1]`@CTAB40`;
    SWAP    A, X `@CTAB40`; Don't loose the completion code
    CMP     A, 0 `@CTAB40`; NULL?
    JZ      .done`@CTAB40`; No update on NULL
; Flow here to update the VSR Completion Status Block
    SWAP    A, X `@CTAB40`; Completion code A, Pointer in X
    MOV     [X + 0], A`@CTAB40`; Update the completion Code
    MOV     A, [`@INSTANCE_NAME`_TransferByteCount]`@CTAB40`; Actual Byte Count MSB
    MOV     [X + 1], A
    MOV     A, [`@INSTANCE_NAME`_TransferByteCount + 1]`@CTAB40`; Actual Byte Count LSB
    MOV     [X + 2], A
    MOV     [`@INSTANCE_NAME`_StatusBlockPtr + 1], 0`@CTAB40`; Clear the Block Pointer
.done:
    RET`@CTAB40`; All done
;-----------------------------------------------------------------------------
;  FUNCTION NAME: `@INSTANCE_NAME`_InitializeStatusBlock
;
;  DESCRIPTION:    Initialize the Completion Status Block for a Request.
;                  The completion code is set to USB_XFER_IDLE.
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:      None
;
;  RETURNS:        None
;
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;  THEORY of OPERATION or PROCEDURE:
;
;-----------------------------------------------------------------------------
`@INSTANCE_NAME`_InitializeStatusBlock:
    MOV     A, [`@INSTANCE_NAME`_StatusBlockPtr + 1]`@CTAB40`;
    CMP     A, 0 `@CTAB40`; NULL?
    JZ      .done`@CTAB40`; No update on NULL
; Flow here to initialize the Completion Status Block
    SWAP    A, X `@CTAB40`; Pointer in X
    MOV     [X + 0], USB_XFER_IDLE`@CTAB40`; Initialize the completion code (0)
    MOV     [`@INSTANCE_NAME`_TransferByteCount], 0`@CTAB40`; Clear the byte count
    MOV     [`@INSTANCE_NAME`_TransferByteCount + 1], 0`@CTAB40`;
.done:
    RET`@CTAB40`; All done
;-----------------------------------------------------------------------------
;  FUNCTION NAME: ;  USB 1st Tier Dispactch Jump Table (based on bmRequestType)
;
;  DESCRIPTION:
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:
;
;  RETURNS:
;
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;  THEORY of OPERATION or PROCEDURE:
;
;-----------------------------------------------------------------------------
MACRO BMREQUEST_DISPATCH
IF (USB_CB_@0_@1_@2 & 1)
    jmp     `@INSTANCE_NAME`_DT_@0_@1_@2_Dispatch
ELSE
    jmp     `@INSTANCE_NAME`_Not_Supported_Local_Drv
ENDIF
ENDM
`@INSTANCE_NAME`_DT_bmRequestType::
    BMREQUEST_DISPATCH    h2d,std,dev
    BMREQUEST_DISPATCH    h2d,std,ifc
    BMREQUEST_DISPATCH    h2d,std,ep
    BMREQUEST_DISPATCH    h2d,std,oth
    BMREQUEST_DISPATCH    h2d,cls,dev
    BMREQUEST_DISPATCH    h2d,cls,ifc
    BMREQUEST_DISPATCH    h2d,cls,ep
    BMREQUEST_DISPATCH    h2d,cls,oth
    BMREQUEST_DISPATCH    h2d,vnd,dev
    BMREQUEST_DISPATCH    h2d,vnd,ifc
    BMREQUEST_DISPATCH    h2d,vnd,ep
    BMREQUEST_DISPATCH    h2d,vnd,oth
    BMREQUEST_DISPATCH    h2d,rsv,dev
    BMREQUEST_DISPATCH    h2d,rsv,ifc
    BMREQUEST_DISPATCH    h2d,rsv,ep
    BMREQUEST_DISPATCH    h2d,rsv,oth
    BMREQUEST_DISPATCH    d2h,std,dev
    BMREQUEST_DISPATCH    d2h,std,ifc
    BMREQUEST_DISPATCH    d2h,std,ep
    BMREQUEST_DISPATCH    d2h,std,oth
    BMREQUEST_DISPATCH    d2h,cls,dev
    BMREQUEST_DISPATCH    d2h,cls,ifc
    BMREQUEST_DISPATCH    d2h,cls,ep
    BMREQUEST_DISPATCH    d2h,cls,oth
    BMREQUEST_DISPATCH    d2h,vnd,dev
    BMREQUEST_DISPATCH    d2h,vnd,ifc
    BMREQUEST_DISPATCH    d2h,vnd,ep
    BMREQUEST_DISPATCH    d2h,vnd,oth
    BMREQUEST_DISPATCH    d2h,rsv,dev
    BMREQUEST_DISPATCH    d2h,rsv,ifc
    BMREQUEST_DISPATCH    d2h,rsv,ep
    BMREQUEST_DISPATCH    d2h,rsv,oth
`@INSTANCE_NAME`_DT_End:
`@INSTANCE_NAME`_DT_Size: equ (`@INSTANCE_NAME`_DT_End-`@INSTANCE_NAME`_DT_bmRequestType) / 2
`@INSTANCE_NAME`_bmRequestType_Dispatch::
    DISPATCHER `@INSTANCE_NAME`_DT_bmRequestType, `@INSTANCE_NAME`_DT_Size, `@INSTANCE_NAME`_Not_Supported_Local_Drv

`@INSTANCE_NAME`_Not_Supported_Local_Drv:
        LJMP     `@INSTANCE_NAME`_Not_Supported


;-----------------------------------------------
; Add custom application code for routines
;-----------------------------------------------

   ;@PSoC_UserCode_BODY_1@ (Do not change this line.)
   ;---------------------------------------------------
   ; Insert your custom code below this banner
   ;---------------------------------------------------

   ;---------------------------------------------------
   ; Insert your custom code above this banner
   ;---------------------------------------------------
   ;@PSoC_UserCode_END@ (Do not change this line.)

; End of File `@INSTANCE_NAME`_drv.asm
