;;*****************************************************************************
;;*****************************************************************************
;;  FILENAME: `@INSTANCE_NAME`_std.asm
;;   Version: 1.5, Updated on 2006/06/19 at 11:43:56
;;  `@PSOC_VERSION`
;;
;;  DESCRIPTION: USB Device User Module software implementation file
;;               for the enCoRe II family of devices
;;
;;  NOTE: User Module APIs conform to the fastcall convention for marshalling
;;        arguments and observe the associated "Registers are volatile" policy.
;;        This means it is the caller's responsibility to preserve any values
;;        in the X and A registers that are still needed after the API
;;        function returns. Even though these registers may be preserved now,
;;        there is no guarantee they will be preserved in future releases.
;;-----------------------------------------------------------------------------
;;  Copyright (c) Cypress Semiconductor 2004. All Rights Reserved.
;;*****************************************************************************
;;*****************************************************************************

include "m8c.inc"
include "`@INSTANCE_NAME`_macros.inc"
include "`@INSTANCE_NAME`.inc"

;-----------------------------------------------
;  Global Symbols
;-----------------------------------------------

AREA usbRAM (RAM,REL)

;-----------------------------------------------
;  Constant Definitions
;-----------------------------------------------
;-----------------------------------------------
; Variable Allocation
;-----------------------------------------------

AREA Bootloader (ROM, REL)

;-----------------------------------------------------------------------------
;  FUNCTION NAME: `@INSTANCE_NAME`_Not_Supported
;
;  DESCRIPTION:   The `@INSTANCE_NAME`_Not_Supported routine is invoked for any
;         USB request that is not supported.  It is invoked with a
;         JMP/LJMP and does not return.  It exits through the common
;                 EP0 exit.
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:     None
;
;  RETURNS:       Does not return
;
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;  THEORY of OPERATION or PROCEDURE:
;
;-----------------------------------------------------------------------------

; d2h_std_dev
IF (USB_CB_SRC_d2h_std_dev_00 & USB_NOT_SUPPORTED)
export  `@INSTANCE_NAME`_CB_d2h_std_dev_00
`@INSTANCE_NAME`_CB_d2h_std_dev_00:
ENDIF
IF (USB_CB_SRC_d2h_std_dev_01 & USB_NOT_SUPPORTED)
export  `@INSTANCE_NAME`_CB_d2h_std_dev_01
`@INSTANCE_NAME`_CB_d2h_std_dev_01:
ENDIF
IF (USB_CB_SRC_d2h_std_dev_02 & USB_NOT_SUPPORTED)
export  `@INSTANCE_NAME`_CB_d2h_std_dev_02
`@INSTANCE_NAME`_CB_d2h_std_dev_02:
ENDIF
IF (USB_CB_SRC_d2h_std_dev_03 & USB_NOT_SUPPORTED)
export  `@INSTANCE_NAME`_CB_d2h_std_dev_03
`@INSTANCE_NAME`_CB_d2h_std_dev_03:
ENDIF
IF (USB_CB_SRC_d2h_std_dev_04 & USB_NOT_SUPPORTED)
export  `@INSTANCE_NAME`_CB_d2h_std_dev_04
`@INSTANCE_NAME`_CB_d2h_std_dev_04:
ENDIF
IF (USB_CB_SRC_d2h_std_dev_05 & USB_NOT_SUPPORTED)
export  `@INSTANCE_NAME`_CB_d2h_std_dev_05
`@INSTANCE_NAME`_CB_d2h_std_dev_05:
ENDIF
IF (USB_CB_SRC_d2h_std_dev_06 & USB_NOT_SUPPORTED)
export  `@INSTANCE_NAME`_CB_d2h_std_dev_06
`@INSTANCE_NAME`_CB_d2h_std_dev_06:
ENDIF
IF (USB_CB_SRC_d2h_std_dev_07 & USB_NOT_SUPPORTED)
export  `@INSTANCE_NAME`_CB_d2h_std_dev_07
`@INSTANCE_NAME`_CB_d2h_std_dev_07:
ENDIF
IF (USB_CB_SRC_d2h_std_dev_08 & USB_NOT_SUPPORTED)
export  `@INSTANCE_NAME`_CB_d2h_std_dev_08
`@INSTANCE_NAME`_CB_d2h_std_dev_08:
ENDIF

; h2d_std_dev
IF (USB_CB_SRC_h2d_std_dev_00 & USB_NOT_SUPPORTED)
export  `@INSTANCE_NAME`_CB_h2d_std_dev_00
`@INSTANCE_NAME`_CB_h2d_std_dev_00:
ENDIF
IF (USB_CB_SRC_h2d_std_dev_01 & USB_NOT_SUPPORTED)
export  `@INSTANCE_NAME`_CB_h2d_std_dev_01
`@INSTANCE_NAME`_CB_h2d_std_dev_01:
ENDIF
IF (USB_CB_SRC_h2d_std_dev_02 & USB_NOT_SUPPORTED)
export  `@INSTANCE_NAME`_CB_h2d_std_dev_02
`@INSTANCE_NAME`_CB_h2d_std_dev_02:
ENDIF
IF (USB_CB_SRC_h2d_std_dev_03 & USB_NOT_SUPPORTED)
export  `@INSTANCE_NAME`_CB_h2d_std_dev_03
`@INSTANCE_NAME`_CB_h2d_std_dev_03:
ENDIF
IF (USB_CB_SRC_h2d_std_dev_04 & USB_NOT_SUPPORTED)
export  `@INSTANCE_NAME`_CB_h2d_std_dev_04
`@INSTANCE_NAME`_CB_h2d_std_dev_04:
ENDIF
IF (USB_CB_SRC_h2d_std_dev_05 & USB_NOT_SUPPORTED)
export  `@INSTANCE_NAME`_CB_h2d_std_dev_05
`@INSTANCE_NAME`_CB_h2d_std_dev_05:
ENDIF
IF (USB_CB_SRC_h2d_std_dev_06 & USB_NOT_SUPPORTED)
export  `@INSTANCE_NAME`_CB_h2d_std_dev_06
`@INSTANCE_NAME`_CB_h2d_std_dev_06:
ENDIF
IF (USB_CB_SRC_h2d_std_dev_07 & USB_NOT_SUPPORTED)
export  `@INSTANCE_NAME`_CB_h2d_std_dev_07
`@INSTANCE_NAME`_CB_h2d_std_dev_07:
ENDIF
IF (USB_CB_SRC_h2d_std_dev_08 & USB_NOT_SUPPORTED)
export  `@INSTANCE_NAME`_CB_h2d_std_dev_08
`@INSTANCE_NAME`_CB_h2d_std_dev_08:
ENDIF
IF (USB_CB_SRC_h2d_std_dev_09 & USB_NOT_SUPPORTED)
export  `@INSTANCE_NAME`_CB_h2d_std_dev_09
`@INSTANCE_NAME`_CB_h2d_std_dev_09:
ENDIF

; d2h_std_ifc
IF (USB_CB_SRC_d2h_std_ifc_00 & USB_NOT_SUPPORTED)
export  `@INSTANCE_NAME`_CB_d2h_std_ifc_00
`@INSTANCE_NAME`_CB_d2h_std_ifc_00:
ENDIF
IF (USB_CB_SRC_d2h_std_ifc_01 & USB_NOT_SUPPORTED)
export  `@INSTANCE_NAME`_CB_d2h_std_ifc_01
`@INSTANCE_NAME`_CB_d2h_std_ifc_01:
ENDIF
IF (USB_CB_SRC_d2h_std_ifc_02 & USB_NOT_SUPPORTED)
export  `@INSTANCE_NAME`_CB_d2h_std_ifc_02
`@INSTANCE_NAME`_CB_d2h_std_ifc_02:
ENDIF
IF (USB_CB_SRC_d2h_std_ifc_03 & USB_NOT_SUPPORTED)
export  `@INSTANCE_NAME`_CB_d2h_std_ifc_03
`@INSTANCE_NAME`_CB_d2h_std_ifc_03:
ENDIF
IF (USB_CB_SRC_d2h_std_ifc_04 & USB_NOT_SUPPORTED)
export  `@INSTANCE_NAME`_CB_d2h_std_ifc_04
`@INSTANCE_NAME`_CB_d2h_std_ifc_04:
ENDIF
IF (USB_CB_SRC_d2h_std_ifc_05 & USB_NOT_SUPPORTED)
export  `@INSTANCE_NAME`_CB_d2h_std_ifc_05
`@INSTANCE_NAME`_CB_d2h_std_ifc_05:
ENDIF
IF (USB_CB_SRC_d2h_std_ifc_06 & USB_NOT_SUPPORTED)
export  `@INSTANCE_NAME`_CB_d2h_std_ifc_06
`@INSTANCE_NAME`_CB_d2h_std_ifc_06:
ENDIF
IF (USB_CB_SRC_d2h_std_ifc_07 & USB_NOT_SUPPORTED)
export  `@INSTANCE_NAME`_CB_d2h_std_ifc_07
`@INSTANCE_NAME`_CB_d2h_std_ifc_07:
ENDIF
IF (USB_CB_SRC_d2h_std_ifc_08 & USB_NOT_SUPPORTED)
export  `@INSTANCE_NAME`_CB_d2h_std_ifc_08
`@INSTANCE_NAME`_CB_d2h_std_ifc_08:
ENDIF
IF (USB_CB_SRC_d2h_std_ifc_09 & USB_NOT_SUPPORTED)
export  `@INSTANCE_NAME`_CB_d2h_std_ifc_09
`@INSTANCE_NAME`_CB_d2h_std_ifc_09:
ENDIF
IF (USB_CB_SRC_d2h_std_ifc_10 & USB_NOT_SUPPORTED)
export  `@INSTANCE_NAME`_CB_d2h_std_ifc_10
`@INSTANCE_NAME`_CB_d2h_std_ifc_10:
ENDIF

; d2h_std_ifc
IF (USB_CB_SRC_h2d_std_ifc_00 & USB_NOT_SUPPORTED)
export  `@INSTANCE_NAME`_CB_h2d_std_ifc_00
`@INSTANCE_NAME`_CB_h2d_std_ifc_00:
ENDIF

; d2h_std_ep
IF (USB_CB_SRC_d2h_std_ep_00 & USB_NOT_SUPPORTED)
export  `@INSTANCE_NAME`_CB_d2h_std_ep_00
`@INSTANCE_NAME`_CB_d2h_std_ep_00:
ENDIF

; h2d_std_ep
IF (USB_CB_SRC_h2d_std_ep_00 & USB_NOT_SUPPORTED)
export  `@INSTANCE_NAME`_CB_h2d_std_ep_00
`@INSTANCE_NAME`_CB_h2d_std_ep_00:
ENDIF
IF (USB_CB_SRC_h2d_std_ep_01 & USB_NOT_SUPPORTED)
export  `@INSTANCE_NAME`_CB_h2d_std_ep_01
`@INSTANCE_NAME`_CB_h2d_std_ep_01:
ENDIF
IF (USB_CB_SRC_h2d_std_ep_02 & USB_NOT_SUPPORTED)
export  `@INSTANCE_NAME`_CB_h2d_std_ep_02
`@INSTANCE_NAME`_CB_h2d_std_ep_02:
ENDIF
IF (USB_CB_SRC_h2d_std_ep_03 & USB_NOT_SUPPORTED)
export  `@INSTANCE_NAME`_CB_h2d_std_ep_03
`@INSTANCE_NAME`_CB_h2d_std_ep_03:
ENDIF

export  `@INSTANCE_NAME`_Not_Supported
export _`@INSTANCE_NAME`_Not_Supported
`@INSTANCE_NAME`_Not_Supported:
_`@INSTANCE_NAME`_Not_Supported:
   MOV    A, 0`@CTAB40`; Count 0
   MOV    X, USB_MODE_STALL_IN_OUT`@CTAB40`; Stall the request
   LJMP   `@INSTANCE_NAME`_EP0_UPD_MODE_EXIT
;-----------------------------------------------------------------------------
;  FUNCTION NAME: `@INSTANCE_NAME`_CB_d2h_std_dev_00
;
;  DESCRIPTION:   Get Device Status
;
;****************************************************************
; STANDARD DEVICE IN REQUEST: Get_Device_Status
;****************************************************************
;
; bmRequestType  : (IN | STANDARD | DEVICE)       = 80h     
; bRequest       : GET_STATUS                     = 00h    
; wValue         : RESERVED                       = 0000h  
; wIndex         : RESERVED                       = 0000h
; wLength        : SIZEOF_ENDPOINT_STATUS         = 0002h  
; 
; The GET_DEVICE_STATUS request returns the current device status.
;
;****************************************************************
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:
;
;  RETURNS:
;
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;  THEORY of OPERATION or PROCEDURE:
;
;-----------------------------------------------------------------------------
IF (USB_CB_SRC_d2h_std_dev_00 & USB_UM_SUPPLIED)
.LITERAL
GetStatusTransferDescrTable:
    TD_START_TABLE 1`@CTAB40`; One entry
    TD_ENTRY    USB_DS_RAM, 2, `@INSTANCE_NAME`_TransferBuffer, NULL_PTR `@CTAB40`; Intermediate Buffer
.ENDLITERAL
export  `@INSTANCE_NAME`_CB_d2h_std_dev_00
`@INSTANCE_NAME`_CB_d2h_std_dev_00:
    MOV     [`@INSTANCE_NAME`_t2], 0 `@CTAB40`; Use the UM temp var--Selector

    MOV     [`@INSTANCE_NAME`_TransferBuffer+1], 0 `@CTAB40`; Use the UM Transfer Buffer
    MOV     [`@INSTANCE_NAME`_TransferBuffer], [`@INSTANCE_NAME`_DeviceStatus]

    MOV     A,>GetStatusTransferDescrTable `@CTAB40`; Get the ROM Address MSB
    MOV     X,<GetStatusTransferDescrTable `@CTAB40`; Get the ROM Address LSB
    JMP     `@INSTANCE_NAME`_GetTableEntry_Local_Std
ENDIF
;-----------------------------------------------------------------------------
; FUNCTION NAME: `@INSTANCE_NAME`_CB_d2h_std_dev_06
;
; DESCRIPTION:   Get Device Descriptor
;
;****************************************************************
; STANDARD DEVICE IN REQUEST: Get_Device_Descriptor
;****************************************************************
;
; bmRequestType  : (IN | STANDARD | DEVICE)       = 80h
; bRequest       : GET_DESCRIPTOR                 = 06h    
; wValue         : DESCRIPTOR TYPE | INDEX        = xxxxh  
; wIndex         : ZERO or LANG_ID                = xxxxh
; wLength        : SIZEOF_DESCRIPTOR              = --xxh  
; 
; The GET_DEVICE_DESCRIPTOR returns the specified descriptor if 
; the descriptor exists. 
; 
; The upper byte of wValue contains the descriptor type and 
; the lower byte contains the descriptor index. wIndex 
; contains either 0000h or the Language ID. wLength contains 
; the descriptor length. The actual descriptor information is 
; transferred in subsequent data packets. 
;
;****************************************************************
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:
;
;  RETURNS:
;
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;  THEORY of OPERATION or PROCEDURE:
;
;-----------------------------------------------------------------------------
IF (USB_CB_SRC_d2h_std_dev_06 & USB_UM_SUPPLIED)
export  `@INSTANCE_NAME`_CB_d2h_std_dev_06
`@INSTANCE_NAME`_CB_d2h_std_dev_06:
;-----------------------------------------------------------------------------
; Dispatch to the proper handler
;-----------------------------------------------------------------------------
    MOV     A, REG[`@INSTANCE_NAME`_EP0DATA+wValueHi]`@CTAB40`; Get the descrptor type
    ASL     A `@CTAB40`; Make it into a offset
    CMP     A, GET_DESCR_DISPATCH_SIZE        `@CTAB40`; Validity check
    JNC     `@INSTANCE_NAME`_Not_Supported

    JACC    GET_DESCR_DISPATCH

GET_DESCR_DISPATCH:
    JMP     `@INSTANCE_NAME`_Not_Supported `@CTAB40`; Invalid
    JMP     `@INSTANCE_NAME`_SendDeviceDescr `@CTAB40`; Device Descriptor
    JMP     `@INSTANCE_NAME`_SendConfigDescr `@CTAB40`; Configuration Descriptor
IF  `@INSTANCE_NAME`_bNumStringDescrs`@CTAB40`; Number of String Descriptor defined with the Wizard
    JMP     `@INSTANCE_NAME`_SendStringDescr `@CTAB40`; String Descriptor
ELSE
    JMP     `@INSTANCE_NAME`_Not_Supported `@CTAB40`; Not supported if we don't have
                                           `@CTAB40`; any String Descriptor
ENDIF
GET_DESCR_DISPATCH_END:
GET_DESCR_DISPATCH_SIZE: EQU (GET_DESCR_DISPATCH_END - GET_DESCR_DISPATCH)     
;-----------------------------------------------------------------------------
; Configuration Descriptor Handler
;-----------------------------------------------------------------------------
`@INSTANCE_NAME`_SendConfigDescr:
    CMP     [`@INSTANCE_NAME`_bCurrentDevice], 0xFF
    JNZ     .cfg_not_bl
    MOV     [`@INSTANCE_NAME`_t2], 0 `@CTAB40`; Use the UM temp var--Selector
    MOV     A,>USB_BL_D0_CONFIG_DESCR_TABLE `@CTAB40`; Get the ROM Address MSB
    MOV     X,<USB_BL_D0_CONFIG_DESCR_TABLE `@CTAB40`; Get the ROM Address LSB
    JMP     `@INSTANCE_NAME`_GetTableEntry_Local_Std

.cfg_not_bl:
    CALL    `@INSTANCE_NAME`_GET_DEVICE_TABLE_ENTRY
    SWAP    A, X
    ADD     A, 2`@CTAB40`; We want the pointer to the descriptor table (second entry)
    SWAP    A, X
    ADC     A, 0`@CTAB40`; Don't forget the carry
    MOV    [`@INSTANCE_NAME`_t2], `@INSTANCE_NAME`_t1`@CTAB40`; Set up the destination  
    CALL    `@INSTANCE_NAME`_GETWORD`@CTAB40`; Get the pointer
    
    MOV     A, [`@INSTANCE_NAME`_t1]`@CTAB40`; Pointer MSB
    MOV     X, [`@INSTANCE_NAME`_t1+1]`@CTAB40`; Pointer LSB
    
    PUSH    A`@CTAB40`; Save the MSB
    
    MOV     A, REG[`@INSTANCE_NAME`_EP0DATA+wValueLo] `@CTAB40`; Get the descrptor index
    MOV     [`@INSTANCE_NAME`_t2], A `@CTAB40`; Use the UM temp var--Selector
    POP     A`@CTAB40`; Need the MSB for the range check
    PUSH    A`@CTAB40`; Save the MSB for after the range check
    ROMX`@CTAB40`; First entry is the table size (only a byte)
    CMP     A, [`@INSTANCE_NAME`_t2] `@CTAB40`; Range check
    JNC     .range_ok
    
    POP     A`@CTAB40`; Fix the stack
    JMP    `@INSTANCE_NAME`_Not_Supported

.range_ok:
    POP     A`@CTAB40`; Get the MSB back
    JMP     `@INSTANCE_NAME`_GetTableEntry_Local_Std
;-----------------------------------------------------------------------------
; Device Descriptor Handler
;-----------------------------------------------------------------------------
`@INSTANCE_NAME`_SendDeviceDescr:
    CMP     [`@INSTANCE_NAME`_bCurrentDevice], 0xFF
    JNZ     .dev_not_bl
    MOV     [`@INSTANCE_NAME`_t2], 0 `@CTAB40`; Use the UM temp var--Selector
    MOV     A,>USB_BL_DEVICE_DESCR_TABLE `@CTAB40`; Get the ROM Address MSB
    MOV     X,<USB_BL_DEVICE_DESCR_TABLE `@CTAB40`; Get the ROM Address LSB
    JMP     `@INSTANCE_NAME`_GetTableEntry_Local_Std

.dev_not_bl:
    MOV     [`@INSTANCE_NAME`_t2], [`@INSTANCE_NAME`_bCurrentDevice] `@CTAB40`; Use the UM temp var--Selector
    MOV     A, 1 `@CTAB40`; Get pointer to `@INSTANCE_NAME`_DEVICE_DESCR_TABLE
    CALL    `@INSTANCE_NAME`_GET_TABLE_PTR
    PUSH    A
    ROMX      `@CTAB40`; First entry is the table size (only a byte)
    CMP     A, [`@INSTANCE_NAME`_t2] `@CTAB40`; Range check
    POP     A
    JC      `@INSTANCE_NAME`_Not_Supported
    JMP     `@INSTANCE_NAME`_GetTableEntry_Local_Std
;-----------------------------------------------------------------------------
; String Descriptor Handler
;-----------------------------------------------------------------------------
IF  `@INSTANCE_NAME`_bNumStringDescrs    ; Not needed if we don't have any String Descriptors
`@INSTANCE_NAME`_SendStringDescr:
    MOV     A, REG[`@INSTANCE_NAME`_EP0DATA+wValueLo] `@CTAB40`; Get the descrptor index
    MOV     [`@INSTANCE_NAME`_t2], A `@CTAB40`; Use the UM temp var--Selector

    MOV     A,>USB_BL_StringTable `@CTAB40`; Get the ROM Address MSB
    MOV     X,<USB_BL_StringTable `@CTAB40`; Get the ROM Address LSB
    CMP     [`@INSTANCE_NAME`_bCurrentDevice], 0xFF
    JZ      .str_get_count

    MOV     A, 2 `@CTAB40`; Get pointer to `@INSTANCE_NAME`_StringTable
    CALL    `@INSTANCE_NAME`_GET_TABLE_PTR
.str_get_count:
    PUSH    A
    ROMX      `@CTAB40`; First entry is the table size (only a byte)
    CMP     A, [`@INSTANCE_NAME`_t2] `@CTAB40`; Range check
    POP     A
    JC      `@INSTANCE_NAME`_Not_Supported
    JMP     `@INSTANCE_NAME`_GetTableEntry_Local_Std
ENDIF
ENDIF
;-----------------------------------------------------------------------------
;  FUNCTION NAME: `@INSTANCE_NAME`_CB_d2h_std_dev_08
;
;  DESCRIPTION:   Get Device Configuration
;
;****************************************************************
; STANDARD DEVICE IN REQUEST: Get_Device_Configuration
;****************************************************************
;
; bmRequestType  : (IN | STANDARD | DEVICE)       = 80h
; bRequest       : GET_CONFIGURATION              = 08h    
; wValue         : RESERVED                       = 0000h  
; wIndex         : RESERVED                       = 0000h
; wLength        : SIZEOF_DEVICE_CONFIGURATION    = 0001h  
; 
; The GET_DEVICE_CONFIGURATION request returns the currently 
; selected device configuration number. 
;
; request_value and request_index contain 0000h. request_length 
; contains 0001h and the one-byte configuration number is returned 
; in a separate data transfer.
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:
;
;  RETURNS:
;
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;  THEORY of OPERATION or PROCEDURE:
;
;-----------------------------------------------------------------------------
IF (USB_CB_SRC_d2h_std_dev_08 & USB_UM_SUPPLIED)
.LITERAL
GetConfigTransferDescrTable:
    TD_START_TABLE  1`@CTAB40`; One entry
    TD_ENTRY    USB_DS_RAM, 1, `@INSTANCE_NAME`_Configuration, NULL_PTR `@CTAB40`; Current configuration
.ENDLITERAL
export  `@INSTANCE_NAME`_CB_d2h_std_dev_08
`@INSTANCE_NAME`_CB_d2h_std_dev_08:
    MOV     [`@INSTANCE_NAME`_t2], 0 `@CTAB40`; Use the UM temp var--Selector
    MOV     A,>GetConfigTransferDescrTable `@CTAB40`; Get the ROM Address MSB
    MOV     X,<GetConfigTransferDescrTable `@CTAB40`; Get the ROM Address LSB
    JMP     `@INSTANCE_NAME`_GetTableEntry_Local_Std
ENDIF
;-----------------------------------------------------------------------------
;  FUNCTION NAME: `@INSTANCE_NAME`_CB_h2d_std_dev_01
;
;  DESCRIPTION:   Clear Device Feature
;
;****************************************************************
; STANDARD DEVICE OUT REQUEST: Clear_Device_Feature
;****************************************************************
;
; bmRequestType  : (OUT | STANDARD | DEVICE)      = 00h
; bRequest       : CLEAR_FEATURE                  = 01h    
; wValue         : FEATURE_SELECTOR               = --xxh  
; wIndex         : RESERVED                       = 0000h
; wLength        : RESERVED                       = 0000h  
; 
; The CLEAR_DEVICE_FEATURE request disables a particular feature 
; for a device. The only feature supported for a device is the 
; REMOTE_WAKEUP feature.
; 
;****************************************************************
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:
;
;  RETURNS:
;
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;  THEORY of OPERATION or PROCEDURE:
;
;-----------------------------------------------------------------------------
IF (USB_CB_SRC_h2d_std_dev_01 & USB_UM_SUPPLIED)
export  `@INSTANCE_NAME`_CB_h2d_std_dev_01
`@INSTANCE_NAME`_CB_h2d_std_dev_01:
    MOV     A, REG[`@INSTANCE_NAME`_EP0DATA+wValueLo] `@CTAB40`; Get the Feature Selector
          `@CTAB40`; Check against valid features
          `@CTAB40`;  for device receipient
    CMP     A, USB_DEVICE_REMOTE_WAKEUP `@CTAB40`; Only remote wakeup is defined for clear
    JNZ     `@INSTANCE_NAME`_Not_Supported `@CTAB40`;      
    AND     [`@INSTANCE_NAME`_DeviceStatus], ~USB_DEVICE_STATUS_REMOTE_WAKEUP
    JMP     `@INSTANCE_NAME`_NoDataStageControlTransfer_Local_Std
ENDIF
;-----------------------------------------------------------------------------
;  FUNCTION NAME: `@INSTANCE_NAME`_CB_h2d_std_dev_03
;
;  DESCRIPTION:   Set Device Featue
;
;****************************************************************
; STANDARD DEVICE OUT REQUEST: Set_Device_Feature
;****************************************************************
;
; bmRequestType  : (OUT | STANDARD | DEVICE)      = 00h
; bRequest       : SET_FEATURE                    = 03h    
; wValue         : FEATURE_SELECTOR               = --xxh  
; wIndex         : RESERVED                       = 0000h
; wLength        : RESERVED                       = 0000h  
; 
; The SET_DEVICE_FEATURE request enables a particular feature
; on a device. The only feature supported for a device is the 
; REMOTE_WAKEUP feature.
;
;****************************************************************
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:
;
;  RETURNS:
;
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;  THEORY of OPERATION or PROCEDURE:
;
;-----------------------------------------------------------------------------
IF (USB_CB_SRC_h2d_std_dev_03 & USB_UM_SUPPLIED)
export  `@INSTANCE_NAME`_CB_h2d_std_dev_03
`@INSTANCE_NAME`_CB_h2d_std_dev_03:
    MOV     A, REG[`@INSTANCE_NAME`_EP0DATA+wValueLo] `@CTAB40`; Get the Feature Selector
            `@CTAB40`; Check against valid features
            `@CTAB40`;  for device receipient
    CMP     A, USB_DEVICE_REMOTE_WAKEUP `@CTAB40`; Remote wakeup?
    JZ      .remote_wakeup

    CMP     A, USB_TEST_MODE `@CTAB40`; Test Mode 
    JZ      .test_mode
; Flow here for any other selector is invalid for device receipient
    JMP     `@INSTANCE_NAME`_Not_Supported
; Jump here to enable remote wake up
.remote_wakeup:
    OR      [`@INSTANCE_NAME`_DeviceStatus], USB_DEVICE_STATUS_REMOTE_WAKEUP
    JMP     .finish
; Jump here to enable test mode
.test_mode:
    JMP     `@INSTANCE_NAME`_Not_Supported
.finish:
    JMP     `@INSTANCE_NAME`_NoDataStageControlTransfer_Local_Std
ENDIF
;-----------------------------------------------------------------------------
;  FUNCTION NAME: `@INSTANCE_NAME`_CB_h2d_std_dev_05
;
;  DESCRIPTION:   Set Device Address
;
;****************************************************************
; STANDARD DEVICE OUT REQUEST: Set_Device_Address
;****************************************************************
;
; bmRequestType  : (OUT | STANDARD | DEVICE)      = 00h
; bRequest       : SET_ADDRESS                    = 05h    
; wValue         : DEVICE_ADDRESS                 = 00xxh  
; wIndex         : RESERVED                       = 0000h
; wLength        : RESERVED                       = 0000h  
; 
; The SET_DEVICE_ADDRESS request sets the USB device address
; for all future USB accesses. 
;
;****************************************************************
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:
;
;  RETURNS:
;
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;  THEORY of OPERATION or PROCEDURE:
;
;-----------------------------------------------------------------------------

IF (USB_CB_SRC_h2d_std_dev_05 & USB_UM_SUPPLIED)
export  `@INSTANCE_NAME`_CB_h2d_std_dev_05
`@INSTANCE_NAME`_CB_h2d_std_dev_05:
`@CTAB40 ; SET ADDRESS
    MOV     [`@INSTANCE_NAME`_fDataPending], USB_ADDRESS_CHANGE_PENDING `@CTAB40 ; Flag a pending change
    MOV     A, REG[`@INSTANCE_NAME`_EP0DATA+wValueLo] `@CTAB40 ; Get the address
    MOV     [`@INSTANCE_NAME`_TransferBuffer],A `@CTAB40 ; Save the address until 
                                             `@CTAB40 ;  the status stage (IN)
    JMP     `@INSTANCE_NAME`_NoDataStageControlTransfer_Local_Std
ENDIF
;-----------------------------------------------------------------------------
;  FUNCTION NAME: `@INSTANCE_NAME`_CB_h2d_std_dev_09
;
;  DESCRIPTION:   Set Configuration
;
;****************************************************************
; STANDARD DEVICE OUT REQUEST: Set_Device_Configuration
;****************************************************************
;
; bmRequestType  : (OUT | STANDARD | DEVICE)      = 00h
; bRequest       : SET_CONFIGURATION              = 09h    
; wValue         : CONFIGURATION_VALUE            = --xxh  
; wIndex         : RESERVED                       = 0000h
; wLength        : RESERVED                       = 0000h  
; 
; The SET_DEVICE_CONFIGURATION request selects a device 
; configuration to be activated as the current configuration. 
;
;****************************************************************
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:
;
;  RETURNS:
;
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;  THEORY of OPERATION or PROCEDURE:
;
;-----------------------------------------------------------------------------

IF (USB_CB_SRC_h2d_std_dev_09 & USB_UM_SUPPLIED)
export  `@INSTANCE_NAME`_CB_h2d_std_dev_09
`@INSTANCE_NAME`_CB_h2d_std_dev_09:
    CALL    `@INSTANCE_NAME`_GET_DEVICE_TABLE_ENTRY `@CTAB40`; Get the selected device
    MOV     [`@INSTANCE_NAME`_t2],`@INSTANCE_NAME`_t1 `@CTAB40`; Set the GETWORD destination 
    CALL    `@INSTANCE_NAME`_GETWORD `@CTAB40`; Get the pointer to the CONFIG_LOOKUP table
                                     `@CTAB40`; ITempW has the address
    MOV     A, REG[`@INSTANCE_NAME`_EP0DATA+wValueLo] `@CTAB40`; Get the configuration number
    MOV     [`@INSTANCE_NAME`_t2],A `@CTAB40`; Save it 
    MOV     A, [`@INSTANCE_NAME`_t1] `@CTAB40`; Get the CONFIG_LOOKUP ROM Address MSB
    MOV     X, [`@INSTANCE_NAME`_t1+1] `@CTAB40`; Get the CONFIG_LOOKUP ROM Address LSB
    ROMX      `@CTAB40`; First entry is the table size (only a byte)
    CMP     A, [`@INSTANCE_NAME`_t2] `@CTAB40`; Range check
    JC      `@INSTANCE_NAME`_Not_Supported

; Refactored from the two loops below
    MOV     [`@INSTANCE_NAME`_Configuration],[`@INSTANCE_NAME`_t2] `@CTAB40`; Save the config number

    CMP     [`@INSTANCE_NAME`_t2], 0 `@CTAB40`; Unconfigure?
    JZ      .unconfigure

; Flow here to configure the endpoints
    MOV     A, [`@INSTANCE_NAME`_t1] `@CTAB40`; Get the CONFIG_LOOKUP ROM Address MSB
    MOV     X, [`@INSTANCE_NAME`_t1+1] `@CTAB40`; Get the CONFIG_LOOKUP ROM Address LSB
    INC     X `@CTAB40`; Point to the first table entry
    ADC     A, 0 `@CTAB40`;
    MOV    [`@INSTANCE_NAME`_t2], `@INSTANCE_NAME`_t1`@CTAB40`; Set up the destination  
    CALL    `@INSTANCE_NAME`_GETWORD `@CTAB40`; Get the pointer to the CONFIG_LOOKUP table
                                     `@CTAB40`; ITempW has the address
    MOV     X, 0`@CTAB40`; Start the index at 0, but we INC first
.configure_next:
    INC     X`@CTAB40`; Do the next one
    PUSH    X`@CTAB40`; Save the endpoint number
    MOV     A, [`@INSTANCE_NAME`_t1] `@CTAB40`; Get the CONFIG_LOOKUP ROM Address MSB
    MOV     X, [`@INSTANCE_NAME`_t1+1] `@CTAB40`; Get the CONFIG_LOOKUP ROM Address LSB
    ROMX
    INC     [`@INSTANCE_NAME`_t1+1] `@CTAB40`; Point to the next 
    ADC     [`@INSTANCE_NAME`_t1], 0`@CTAB40`;
    POP     X
    CALL    ConfigureEP`@CTAB40`; X contains the EP number
                       `@CTAB40`; A contains the EP Direction
    MOV     A, X`@CTAB40`; 
    CMP     A, USB_MAX_EP_NUMBER`@CTAB40`; Configure each of the endpoints
    JNZ     .configure_next`@CTAB40`; Do another one?
; Flow here when we are done
    JMP     .done

; Jump here to unconfigure the endpoints
.unconfigure:
    MOV     X, USB_MAX_EP_NUMBER`@CTAB40`; Configure each of the endpoints
.unconfigure_next:
    MOV     [X+`@INSTANCE_NAME`_EndpointAPIStatus], NO_EVENT_ALLOWED`@CTAB40`; For the API
    MOV     REG[X+EP0MODE], USB_MODE_DISABLE`@CTAB40`; Disable the endpoint
    DEC     X`@CTAB40`; One more down
    JNZ     .unconfigure_next`@CTAB40`; Don't unconfigure EP0
.done:
    JMP     `@INSTANCE_NAME`_NoDataStageControlTransfer_Local_Std
ENDIF
;-----------------------------------------------------------------------------
;  FUNCTION NAME: `@INSTANCE_NAME`_CB_d2h_std_ifc_00
;
;  DESCRIPTION:   Get Interface Status
;
;****************************************************************
; STANDARD INTERFACE IN REQUEST: Get_Interface_Status
;****************************************************************
;
; bmRequestType  : (IN | STANDARD | INTERFACE)    = 81h    
; bRequest       : GET_STATUS                     = 00h    
; wValue         : RESERVED                       = 0000h  
; wIndex         : INTERFACE                      = --xxh
; wLength        : SIZEOF_INTERFACE_STATUS        = 0002h  
; 
; The GET_INTERFACE_STATUS request returns status for the 
; specified interface.
;
;****************************************************************
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:
;
;  RETURNS:
;
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;  THEORY of OPERATION or PROCEDURE:
;
;-----------------------------------------------------------------------------
IF (USB_CB_SRC_d2h_std_ifc_00 & USB_UM_SUPPLIED)
.LITERAL
GetInterfaceStatusTransferDescrTable:
    TD_START_TABLE  1 `@CTAB40`; One entry
    TD_ENTRY        USB_DS_RAM, 2, `@INSTANCE_NAME`_TransferBuffer, NULL_PTR `@CTAB40`; Reuse the transfer buffer
.ENDLITERAL
export  `@INSTANCE_NAME`_CB_d2h_std_ifc_00
`@INSTANCE_NAME`_CB_d2h_std_ifc_00:

    MOV     [`@INSTANCE_NAME`_TransferBuffer], 0    `@CTAB40`; Zero the transfer buffer
    MOV     [`@INSTANCE_NAME`_TransferBuffer+1`], 0 `@CTAB40`; 

    MOV     [`@INSTANCE_NAME`_t2], 0 `@CTAB40`; Use the UM temp var--Selector
    MOV     A,>GetInterfaceStatusTransferDescrTable `@CTAB40`; Get the ROM Address MSB
    MOV     X,<GetInterfaceStatusTransferDescrTable `@CTAB40`; Get the ROM Address LSB

    JMP     `@INSTANCE_NAME`_GetTableEntry_Local_Std
ENDIF
;-----------------------------------------------------------------------------
;  FUNCTION NAME: `@INSTANCE_NAME`_CB_d2h_std_ifc_10
;
;  DESCRIPTION:   Get Interface
;
;****************************************************************
; STANDARD INTERFACE IN REQUEST: Get_Interface
;****************************************************************
;
; bmRequestType  : (IN | STANDARD | INTERFACE)    = 81h
; bRequest       : GET_INTERFACE                  = 0Ah    
; wValue         : RESERVED                       = 0000h  
; wIndex         : INTERFACE                      = xxxxh
; wLength        : SIZEOF_GET_INTERFACE           = 0001h  
; 
; The GET_INTERFACE request returns the selected alternate 
; setting for the specified interface. 
;
;****************************************************************
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:
;
;  RETURNS:
;
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;  THEORY of OPERATION or PROCEDURE:
;
;-----------------------------------------------------------------------------

IF (USB_CB_SRC_d2h_std_ifc_10 & USB_UM_SUPPLIED)
.LITERAL
GetInterfaceTransferDescrTable:
    TD_START_TABLE  1 `@CTAB40`; One entry
    TD_ENTRY        USB_DS_RAM, 1, `@INSTANCE_NAME`_TransferBuffer, NULL_PTR `@CTAB40`; Reuse the transfer buffer
.ENDLITERAL
export  `@INSTANCE_NAME`_CB_d2h_std_ifc_10
`@INSTANCE_NAME`_CB_d2h_std_ifc_10:
    MOV     A, REG[`@INSTANCE_NAME`_EP0DATA+wIndexLo] `@CTAB40`; Get the interface number
    CMP     A, `@DSHID_bNumInterfaces`h `@CTAB40`; Valid interface number? (UM Parameter: NumInterfaces)
    JNC     `@INSTANCE_NAME`_Not_Supported

    MOV     X, A `@CTAB40`; The interface number is the index into alternates settings table

    MOV     A, [X + `@INSTANCE_NAME`_InterfaceSetting] `@CTAB40`; Save the current interface setting
    MOV     [`@INSTANCE_NAME`_TransferBuffer], A    `@CTAB40`; into the transfer buffer

    MOV     [`@INSTANCE_NAME`_t2], 0 `@CTAB40`; Use the UM temp var--Selector
    MOV     A,>GetInterfaceTransferDescrTable `@CTAB40`; Get the ROM Address MSB
    MOV     X,<GetInterfaceTransferDescrTable `@CTAB40`; Get the ROM Address LSB

    JMP     `@INSTANCE_NAME`_GetTableEntry_Local_Std
ENDIF
;-----------------------------------------------------------------------------
;  FUNCTION NAME: `@INSTANCE_NAME`_CB_d2h_std_ep_00
;
;  DESCRIPTION:   Get Endpoint Status
;
;****************************************************************
; STANDARD ENDPOINT IN REQUEST: Get_Endpoint_Status
;****************************************************************
;
; bmRequestType  : (IN | STANDARD | ENDPOINT)     = 82h    
; bRequest       : GET_STATUS                     = 00h    
; wValue         : RESERVED                       = 0000h  
; wIndex         : ENDPOINT                       = 00xxh
; wLength        : SIZEOF_ENDPOINT_STATUS         = 0002h  
; 
; The GET_ENDPOINT_STATUS request returns status for the specified 
; endpoint.
;
;****************************************************************
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:
;
;  RETURNS:
;
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;  THEORY of OPERATION or PROCEDURE:
;
;-----------------------------------------------------------------------------
IF (USB_CB_SRC_d2h_std_ep_00 & USB_UM_SUPPLIED)
export  `@INSTANCE_NAME`_CB_d2h_std_ep_00
`@INSTANCE_NAME`_CB_d2h_std_ep_00:
    MOV     A, REG[`@INSTANCE_NAME`_EP0DATA+wIndexLo] `@CTAB40`; Get the endpoint number
    AND     A, ~USB_DIR_IN`@CTAB40`; Strip off the direction bit
    CMP     A, USB_NUM_ENDPOINTS `@CTAB40`; Range check
    JNC     `@INSTANCE_NAME`_Not_Supported

    MOV     X, A `@CTAB40`; The endpoint number is the index

    MOV     [`@INSTANCE_NAME`_t2], 0 `@CTAB40`; Use the UM temp var--Selector

    MOV     [`@INSTANCE_NAME`_TransferBuffer + 1], 0 `@CTAB40`; Use the UM Transfer Buffer
    MOV     A, [X + `@INSTANCE_NAME`_EndpointStatus] `@CTAB40`; Get the status
    MOV     [`@INSTANCE_NAME`_TransferBuffer], A `@CTAB40`; Save it in the report

    MOV     A,>GetStatusTransferDescrTable `@CTAB40`; Get the ROM Address MSB
    MOV     X,<GetStatusTransferDescrTable `@CTAB40`; Get the ROM Address LSB

    JMP     `@INSTANCE_NAME`_GetTableEntry_Local_Std
ENDIF

;-----------------------------------------------------------------------------
;  FUNCTION NAME: `@INSTANCE_NAME`_CB_h2d_std_ep_01
;
;  DESCRIPTION:   Clear Endpoint Feature
;
;****************************************************************
; STANDARD ENDPOINT OUT REQUEST: Clear_Endpoint_Feature
;****************************************************************
;
; bmRequestType  : (OUT | STANDARD | ENDPOINT)    = 02h
; bRequest       : CLEAR_FEATURE                  = 01h    
; wValue         : FEATURE_SELECTOR               = --xxh  
; wIndex         : ENDPOINT                       = 00xxh
; wLength        : RESERVED                       = 0000h  
; 
; The CLEAR_ENDPOINT_FEATURE request disables a particular 
; feature for an endpoint. 
;
; The only feature supported for an endpoint is the EP_HALT 
; feature.
;
;****************************************************************
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:
;
;  RETURNS:
;
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;  THEORY of OPERATION or PROCEDURE:
;
;-----------------------------------------------------------------------------
IF (USB_CB_SRC_h2d_std_ep_01 & USB_UM_SUPPLIED)
export  `@INSTANCE_NAME`_CB_h2d_std_ep_01
`@INSTANCE_NAME`_CB_h2d_std_ep_01:
    MOV     A, REG[`@INSTANCE_NAME`_EP0DATA+wValueLo] `@CTAB40`; Get the feature selector
    CMP     A, USB_ENDPOINT_HALT `@CTAB40`; Halt is the only selector defined for endpoints
    JNZ     `@INSTANCE_NAME`_Not_Supported

    MOV     A, REG[`@INSTANCE_NAME`_EP0DATA+wIndexLo] `@CTAB40`; Get the Endpoint number
    AND     A, ~USB_DIR_IN`@CTAB40`; Strip off the direction bit
    CMP     A, 0 `@CTAB40`; Since we can't halt the Control Endpoint
    JZ      .done

    CMP     A, USB_NUM_ENDPOINTS `@CTAB40`; Range check
    JNC     `@INSTANCE_NAME`_Not_Supported

    MOV     X, A `@CTAB40`; Endpoint number is the index
    AND     [X+`@INSTANCE_NAME`_EndpointStatus], ~USB_ENDPOINT_STATUS_HALT `@CTAB40`; Clear the endpoint halt

    ; Clear data toggle for endpoint
    MOV     A, X
    INDEX   `@INSTANCE_NAME`_USB_EP_BIT_LOOKUP
    XOR     A, FFh
    AND     [`@INSTANCE_NAME`_EPDataToggle], A ; Clear the data toggle
	
    TST     REG[`@INSTANCE_NAME`_EP0DATA+wIndexLo], USB_DIR_IN `@CTAB40`; IN or OUT endpoint?
    JNZ     .in

    ; Mark endpoint as empty so it will be reloaded
	MOV	    [X+`@INSTANCE_NAME`_EndpointAPIStatus], NO_EVENT_PENDING
    MOV     REG[X + `@INSTANCE_NAME`_EP0MODE], USB_MODE_ACK_OUT `@CTAB40`; ACK the endpoint
    JMP     .done    
.in:
    ; Mark endpoint as empty so it will be reloaded
	MOV	    [X+`@INSTANCE_NAME`_EndpointAPIStatus], EVENT_PENDING
    MOV     REG[X + `@INSTANCE_NAME`_EP0MODE], USB_MODE_NAK_IN `@CTAB40`; NAK the endpoint
.done:        
    JMP     `@INSTANCE_NAME`_NoDataStageControlTransfer_Local_Std
ENDIF
;-----------------------------------------------------------------------------
;  FUNCTION NAME: `@INSTANCE_NAME`_CB_h2d_std_ep_03
;
;  DESCRIPTION:   Set Endpoint Feature
;
;****************************************************************
; STANDARD ENDPOINT OUT REQUEST: Set_Endpoint_Feature
;****************************************************************
;
; bmRequestType  : (OUT | STANDARD | ENDPOINT)    = 02h
; bRequest       : SET_FEATURE                    = 03h    
; wValue         : FEATURE_SELECTOR               = --xxh  
; wIndex         : ENDPOINT                       = 00xxh
; wLength        : RESERVED                       = 0000h  
; 
; The SET_ENDPOINT_FEATURE request enables a particular feature
; for a specific endpoint. The only feature supported for an 
; endpoint is the EP_HALT feature.
; 
;****************************************************************
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:
;
;  RETURNS:
;
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;  THEORY of OPERATION or PROCEDURE:
;
;-----------------------------------------------------------------------------
IF (USB_CB_SRC_h2d_std_ep_03 & USB_UM_SUPPLIED)
export  `@INSTANCE_NAME`_CB_h2d_std_ep_03
`@INSTANCE_NAME`_CB_h2d_std_ep_03:
    MOV     A, REG[`@INSTANCE_NAME`_EP0DATA+wValueLo] `@CTAB40`; Get the feature selector
    CMP     A, USB_ENDPOINT_HALT `@CTAB40`; Halt is the only selector defined for endpoints
    JNZ     `@INSTANCE_NAME`_Not_Supported

    MOV     A, REG[`@INSTANCE_NAME`_EP0DATA+wIndexLo] `@CTAB40`; Get the Endpoint number
    AND     A, ~USB_DIR_IN`@CTAB40`; Strip off the direction bit
    CMP     A, 0 `@CTAB40`; Never halt the Control Endpoint
    JZ      .done

    CMP     A, USB_NUM_ENDPOINTS `@CTAB40`; Range check
    JNC     `@INSTANCE_NAME`_Not_Supported

    MOV     X, A `@CTAB40`; Endpoint number is the index

    OR      [X+`@INSTANCE_NAME`_EndpointStatus], USB_ENDPOINT_STATUS_HALT `@CTAB40`; Halt the endpoint

	MOV	    [X+`@INSTANCE_NAME`_EndpointAPIStatus], NO_EVENT_ALLOWED

    TST     REG[`@INSTANCE_NAME`_EP0DATA+wIndexLo], USB_DIR_IN `@CTAB40`; IN or OUT endpoint?
    JNZ     .in

    MOV     REG[X + `@INSTANCE_NAME`_EP0MODE], USB_MODE_STALL_DATA_EP | USB_MODE_ACK_OUT `@CTAB40`; Stall the endpoint
    JMP     .done    
.in:
    MOV     REG[X + `@INSTANCE_NAME`_EP0MODE], USB_MODE_STALL_DATA_EP | USB_MODE_ACK_IN `@CTAB40`; Stall the endpoint
.done:        
    JMP     `@INSTANCE_NAME`_NoDataStageControlTransfer_Local_Std
ENDIF
;-----------------------------------------------------------------------------
;  FUNCTION NAME: ConfigureEP
;
;  DESCRIPTION:   Configure an endpoint
;
;  ARGUMENTS:    A contains the endpoint direction
;                X contains the endpoint number
;
;  RETURNS:
;
;  SIDE EFFECTS:  The A REGISTER IS VOLATILE.  X REGISTER IS MAINTAINED!
;
;  THEORY of OPERATION or PROCEDURE:
;
;-----------------------------------------------------------------------------
ConfigureEP:
    CMP     A, USB_DIR_UNUSED`@CTAB40`; Is this endpoint unused?
    JNZ     .enable`@CTAB40`; Only enable it if it is used
    RET`@CTAB40`; Quick exit if this endpoint is unused

; Jump here to enable an endpoint
.enable:
    PUSH    A`@CTAB40`; Save the endpoint direction
    MOV     A, X`@CTAB40`; We are using a JACC to dispatch to enable the interrupt
    ASL     A`@CTAB40`;  
    JACC    .EP_INT_ENABLE`@CTAB40`;  
.EP_INT_ENABLE:
    JMP     .EP0IntEnable`@CTAB40`; Enable EP0
    JMP     .EP1IntEnable`@CTAB40`; Enable EP1
    JMP     .EP2IntEnable`@CTAB40`; Enable EP2
; Jump here to enable EP0 Interrupts
.EP0IntEnable:
    M8C_EnableIntMask INT_MSK1, INT_MSK1_USB_EP0
    JMP     .cont
.EP1IntEnable:
    M8C_EnableIntMask INT_MSK1, INT_MSK1_USB_EP1
    JMP     .cont
.EP2IntEnable:
    M8C_EnableIntMask INT_MSK1, INT_MSK1_USB_EP2
; Jump or flow here to continue configuring the endpoint    
.cont:

    MOV     A, X
    INDEX   `@INSTANCE_NAME`_USB_EP_BIT_LOOKUP
    XOR     A, FFh
    AND     [`@INSTANCE_NAME`_EPDataToggle], A ; Clear the data toggle

    AND     [X+`@INSTANCE_NAME`_EndpointStatus], ~USB_ENDPOINT_STATUS_HALT `@CTAB40`; Clear any endpoint halts

    POP     A`@CTAB40`; Get the endpoint direction back
    AND     A, USB_DIR_IN`@CTAB40`; Is it an IN endpoint?
    JNZ     .in`@CTAB40`; Jump on IN
; Flow here for an OUT Endpoint
    MOV     REG[X+EP0MODE], USB_MODE_ACK_OUT`@CTAB40`; enable the endpoint
    MOV     [X+`@INSTANCE_NAME`_EndpointAPIStatus], NO_EVENT_PENDING`@CTAB40`; For the API
    RET
; Jump here for an IN Endpoint
.in:
    MOV     REG[X+EP0MODE], USB_MODE_NAK_IN`@CTAB40`; NAK the endpoint
    MOV     [X+`@INSTANCE_NAME`_EndpointAPIStatus], EVENT_PENDING`@CTAB40`; For the API
    RET
;-----------------------------------------------------------------------------
;  USB 2nd Tier Dispactch Jump Tables for Standard Requests (based on bRequest)
;-----------------------------------------------------------------------------
;  FUNCTION NAME: ;  USB 2nd Tier Dispactch Jump Table
;
;  DESCRIPTION:   The following tables dispatch to the Standard request handler
;                 functions.  (Assumes bmRequestType(5:6) is 0, Standard)
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:
;
;  RETURNS:
;
;  SIDE EFFECTS: REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;  THEORY of OPERATION or PROCEDURE:
;
;-----------------------------------------------------------------------------
`@INSTANCE_NAME`_DT_d2h_std_dev:
;-----------------------------------------------------------------------------
    jmp     `@INSTANCE_NAME`_CB_d2h_std_dev_00
    jmp     `@INSTANCE_NAME`_CB_d2h_std_dev_01
    jmp     `@INSTANCE_NAME`_CB_d2h_std_dev_02
    jmp     `@INSTANCE_NAME`_CB_d2h_std_dev_03
    jmp     `@INSTANCE_NAME`_CB_d2h_std_dev_04
    jmp     `@INSTANCE_NAME`_CB_d2h_std_dev_05
    jmp     `@INSTANCE_NAME`_CB_d2h_std_dev_06
    jmp     `@INSTANCE_NAME`_CB_d2h_std_dev_07
    jmp     `@INSTANCE_NAME`_CB_d2h_std_dev_08

`@INSTANCE_NAME`_DT_d2h_std_dev_End:
`@INSTANCE_NAME`_DT_d2h_std_dev_Size: equ (`@INSTANCE_NAME`_DT_d2h_std_dev_End-`@INSTANCE_NAME`_DT_d2h_std_dev) / 2
`@INSTANCE_NAME`_DT_d2h_std_dev_Dispatch::
    MOV    A, REG[`@INSTANCE_NAME`_EP0DATA + bRequest]
    DISPATCHER `@INSTANCE_NAME`_DT_d2h_std_dev, `@INSTANCE_NAME`_DT_d2h_std_dev_Size, `@INSTANCE_NAME`_Not_Supported 
;-----------------------------------------------------------------------------
`@INSTANCE_NAME`_DT_h2d_std_dev:
;-----------------------------------------------------------------------------
    jmp     `@INSTANCE_NAME`_CB_h2d_std_dev_00
    jmp     `@INSTANCE_NAME`_CB_h2d_std_dev_01
    jmp     `@INSTANCE_NAME`_CB_h2d_std_dev_02
    jmp     `@INSTANCE_NAME`_CB_h2d_std_dev_03
    jmp     `@INSTANCE_NAME`_CB_h2d_std_dev_04
    jmp     `@INSTANCE_NAME`_CB_h2d_std_dev_05
    jmp     `@INSTANCE_NAME`_CB_h2d_std_dev_06
    jmp     `@INSTANCE_NAME`_CB_h2d_std_dev_07
    jmp     `@INSTANCE_NAME`_CB_h2d_std_dev_08
    jmp     `@INSTANCE_NAME`_CB_h2d_std_dev_09

`@INSTANCE_NAME`_DT_h2d_std_dev_End:
`@INSTANCE_NAME`_DT_h2d_std_dev_Size: equ (`@INSTANCE_NAME`_DT_h2d_std_dev_End-`@INSTANCE_NAME`_DT_h2d_std_dev) / 2
`@INSTANCE_NAME`_DT_h2d_std_dev_Dispatch::

    MOV     A, REG[`@INSTANCE_NAME`_EP0DATA + bRequest]
    DISPATCHER `@INSTANCE_NAME`_DT_h2d_std_dev, `@INSTANCE_NAME`_DT_h2d_std_dev_Size, `@INSTANCE_NAME`_Not_Supported 

;-----------------------------------------------------------------------------
`@INSTANCE_NAME`_DT_d2h_std_ifc:
;-----------------------------------------------------------------------------
    jmp     `@INSTANCE_NAME`_CB_d2h_std_ifc_00
    jmp     `@INSTANCE_NAME`_CB_d2h_std_ifc_01
    jmp     `@INSTANCE_NAME`_CB_d2h_std_ifc_02
    jmp     `@INSTANCE_NAME`_CB_d2h_std_ifc_03
    jmp     `@INSTANCE_NAME`_CB_d2h_std_ifc_04
    jmp     `@INSTANCE_NAME`_CB_d2h_std_ifc_05
    jmp     `@INSTANCE_NAME`_CB_d2h_std_ifc_06
    jmp     `@INSTANCE_NAME`_CB_d2h_std_ifc_07
    jmp     `@INSTANCE_NAME`_CB_d2h_std_ifc_08
    jmp     `@INSTANCE_NAME`_CB_d2h_std_ifc_09
    jmp     `@INSTANCE_NAME`_CB_d2h_std_ifc_10
`@INSTANCE_NAME`_DT_d2h_std_ifc_End:
`@INSTANCE_NAME`_DT_d2h_std_ifc_Size: equ (`@INSTANCE_NAME`_DT_d2h_std_ifc_End-`@INSTANCE_NAME`_DT_d2h_std_ifc) / 2
`@INSTANCE_NAME`_DT_d2h_std_ifc_Dispatch::
    CMP     [`@INSTANCE_NAME`_Configuration], 0 `@CTAB40`; Is the device configured?
    JNZ     .configured `@CTAB40`; Jump on configured
    JMP    _`@INSTANCE_NAME`_Not_Supported `@CTAB40`; Stall the request if not configured
; Jump here if the device is configured
.configured:
    MOV     A, REG[`@INSTANCE_NAME`_EP0DATA + bRequest]
    DISPATCHER `@INSTANCE_NAME`_DT_d2h_std_ifc, `@INSTANCE_NAME`_DT_d2h_std_ifc_Size, `@INSTANCE_NAME`_Not_Supported 

;-----------------------------------------------------------------------------
`@INSTANCE_NAME`_DT_h2d_std_ifc:
;-----------------------------------------------------------------------------
    jmp     `@INSTANCE_NAME`_CB_h2d_std_ifc_00

`@INSTANCE_NAME`_DT_h2d_std_ifc_End:
`@INSTANCE_NAME`_DT_h2d_std_ifc_Size: equ (`@INSTANCE_NAME`_DT_h2d_std_ifc_End-`@INSTANCE_NAME`_DT_h2d_std_ifc) / 2
`@INSTANCE_NAME`_DT_h2d_std_ifc_Dispatch::
    CMP     [`@INSTANCE_NAME`_Configuration], 0 `@CTAB40`; Is the device configured?
    JNZ     .configured `@CTAB40`; Jump on configured
    JMP    _`@INSTANCE_NAME`_Not_Supported `@CTAB40`; Stall the request if not configured
; Jump here if the device is configured
.configured:
    MOV     A, REG[`@INSTANCE_NAME`_EP0DATA + bRequest]
    DISPATCHER `@INSTANCE_NAME`_DT_h2d_std_ifc, `@INSTANCE_NAME`_DT_h2d_std_ifc_Size, `@INSTANCE_NAME`_Not_Supported 

;-----------------------------------------------------------------------------
`@INSTANCE_NAME`_DT_d2h_std_ep:
;-----------------------------------------------------------------------------
    jmp     `@INSTANCE_NAME`_CB_d2h_std_ep_00

`@INSTANCE_NAME`_DT_d2h_std_ep_End:
`@INSTANCE_NAME`_DT_d2h_std_ep_Size: equ (`@INSTANCE_NAME`_DT_d2h_std_ep_End-`@INSTANCE_NAME`_DT_d2h_std_ep) / 2
`@INSTANCE_NAME`_DT_d2h_std_ep_Dispatch::
    CMP     [`@INSTANCE_NAME`_Configuration], 0 `@CTAB40`; Is the device configured?
    JNZ     .configured `@CTAB40`; Jump on configured

    MOV     A, REG[`@INSTANCE_NAME`_EP0DATA + wIndexHi]`@CTAB40`; Is the request for EP0?
    MOV     [`@INSTANCE_NAME`_t2], A `@CTAB40`; Use the UM temp var--Selector
    MOV     A, REG[`@INSTANCE_NAME`_EP0DATA + wIndexLo]`@CTAB40`;
    OR      [`@INSTANCE_NAME`_t2], A `@CTAB40`; Use the UM temp var--Selector
    JZ      .ep0_request

    JMP    _`@INSTANCE_NAME`_Not_Supported `@CTAB40`; Stall the request if not configured
; Jump here if the device is configured or EP0 request
.configured:
.ep0_request:
    MOV     A, REG[`@INSTANCE_NAME`_EP0DATA + bRequest]
    DISPATCHER `@INSTANCE_NAME`_DT_d2h_std_ep, `@INSTANCE_NAME`_DT_d2h_std_ep_Size, `@INSTANCE_NAME`_Not_Supported 

;-----------------------------------------------------------------------------
`@INSTANCE_NAME`_DT_h2d_std_ep:
;-----------------------------------------------------------------------------
    jmp     `@INSTANCE_NAME`_CB_h2d_std_ep_00
    jmp     `@INSTANCE_NAME`_CB_h2d_std_ep_01
    jmp     `@INSTANCE_NAME`_CB_h2d_std_ep_02
    jmp     `@INSTANCE_NAME`_CB_h2d_std_ep_03

`@INSTANCE_NAME`_DT_h2d_std_ep_End:
`@INSTANCE_NAME`_DT_h2d_std_ep_Size: equ (`@INSTANCE_NAME`_DT_h2d_std_ep_End-`@INSTANCE_NAME`_DT_h2d_std_ep) / 2
`@INSTANCE_NAME`_DT_h2d_std_ep_Dispatch::
    CMP     [`@INSTANCE_NAME`_Configuration], 0 `@CTAB40`; Is the device configured?
    JNZ     .configured `@CTAB40`; Jump on configured

    MOV     A, REG[`@INSTANCE_NAME`_EP0DATA + wIndexHi]`@CTAB40`; Is the request for EP0?
    MOV     [`@INSTANCE_NAME`_t2], A `@CTAB40`; Use the UM temp var--Selector
    MOV     A, REG[`@INSTANCE_NAME`_EP0DATA + wIndexLo]`@CTAB40`;
    OR      [`@INSTANCE_NAME`_t2], A `@CTAB40`; Use the UM temp var--Selector
    JZ      .ep0_request

    JMP    _`@INSTANCE_NAME`_Not_Supported `@CTAB40`; Stall the request if not configured
; Jump here if the device is configured or EP0 request
.configured:
.ep0_request:
    MOV     A, REG[`@INSTANCE_NAME`_EP0DATA + bRequest]
    DISPATCHER `@INSTANCE_NAME`_DT_h2d_std_ep, `@INSTANCE_NAME`_DT_h2d_std_ep_Size, `@INSTANCE_NAME`_Not_Supported 

`@INSTANCE_NAME`_GetTableEntry_Local_Std:
    LJMP    `@INSTANCE_NAME`_GetTableEntry

`@INSTANCE_NAME`_NoDataStageControlTransfer_Local_Std:
    LJMP    `@INSTANCE_NAME`_NoDataStageControlTransfer

;-----------------------------------------------
; Add custom application code for routines 
; redefined by USB_APP_SUPPLIED in USB_HID.INC
;-----------------------------------------------

   ;@PSoC_UserCode_BODY_1@ (Do not change this line.)
   ;---------------------------------------------------
   ; Insert your custom code below this banner
   ;---------------------------------------------------

   ;---------------------------------------------------
   ; Insert your custom code above this banner
   ;---------------------------------------------------
   ;@PSoC_UserCode_END@ (Do not change this line.)

; End of File `@INSTANCE_NAME`_std.asm
