/********************************************************************
 * Global
 *
 ********************************************************************/

/********************************************************************
 * Method: USBRoot.prototype.loadUI
 *
 ********************************************************************/
USBRoot.prototype.loadUI = function (node)
{
	var child = new TreeListControlNode(true, null, '');
	var ot = '<a href="javascript:insertDevice(' + child.ID + ');">Add Device</a>';
	child.setText('USB User Module Descriptor Root',this.instance_name,ot);
	node.add(child);
	for (var i = 0; i < this.devices.length; ++i)
	{
		this.devices[i].loadUI(child);
	}
}
/********************************************************************
 * Method: USBRoot.prototype.unloadUI
 *
 ********************************************************************/
USBRoot.prototype.unloadUI = function (node)
{
	var child = node.firstChild;
	this.instance_name = child.columnText[1];
	child = child.firstChild;
	
	while (child != null)
	{
		this.addDevice();
		this.devices[this.devices.length - 1].unloadUI(child);
		child = child.nextSibling;
	}
}
/********************************************************************
 * Function: insertDevice
 *
 ********************************************************************/
function insertDevice(node_id)
{
// TODO: Need to have a unique ID for the device name
	var device = new USBDevice(0);
	device.loadUI(g_tree.allNodes[node_id]);
}
/********************************************************************
 * Function: USBDevice.prototype.loadUI
 *
 ********************************************************************/
USBDevice.prototype.loadUI = function (node)
{
//debugBox("USBDevice::device:"+this.name);
	var it = new String();
	var ot = new String();

	var device_node = new TreeListControlNode(true, null, '');

	ot =  '<a href="javascript:removeDevice(' + device_node.ID + ');">Remove</a>' + ' | ' +
	      '<a href="javascript:insertConfig(' + device_node.ID + ');">Add Configuration</a>';
	it = 'Device'; //': ' + this.number;
	device_node.setText('Device Descriptor',it,ot);
	node.add(device_node);

	var attrib_node;
	attrib_node = new TreeListControlNode(false, null, '');
	attrib_node.setText('Device Attributes');
	device_node.add(attrib_node);

	var child;
	child = new TreeListControlNode(true, null, '');
	it = loadTextEditZ(this.idVendor, 'validateVendorID', child.ID, 'text_'+child.ID);
	child.setText('Vendor ID',it);
	attrib_node.add(child);	

	child = new TreeListControlNode(true, null, '');
	it = loadTextEditZ(this.idProduct, 'validateProductID', child.ID, 'text_'+child.ID);
	child.setText('Product ID',it);
	attrib_node.add(child);	

	child = new TreeListControlNode(true, null, '');
	it = loadTextEditZ(this.bcdDevice, 'validateDeviceRelease', child.ID, 'text_'+child.ID);
	child.setText('Device Release (bcdDevice)',it);
	attrib_node.add(child);	

	child = new TreeListControlNode(false, null, '');
	it = makeSelectP(g_DeviceClassMap, this.USBclass, 'onDeviceClassChange(this, ' + child.ID + ')', 'select_'+child.ID);
	child.setText('Device Class',it);
	attrib_node.add(child);	

	child = new TreeListControlNode(true, null, '');
	it = makeSelectP(g_InterfaceSubClassMap, this.subclass,   'onSubClassChange(this, ' + child.ID + ')', 'select_'+child.ID);
	child.setText('Subclass',it);
	attrib_node.add(child);	

	if (this.subclass != "No Subclass")
	{
		child = new TreeListControlNode(true, null, '');
		it = makeSelectP(g_InterfaceProtocolMap, this.protocol, 'onProtocolChange(this, ' + child.ID + ')', 'select_'+child.ID);
		child.setText('Protocol',it);
		attrib_node.add(child);	
	}

	child = new TreeListControlNode(true, null, '');
	it = makeSelectP(g_StringMap, this.iManufacturer, 'onStringChange(this, ' + child.ID + ')', 'select_'+child.ID);
	child.setText('Manufacturer String',it);
	var observer = new Observer("USBSTRINGS", updateUSBStringSelect, child.ID);
	attrib_node.add(child);	

	child = new TreeListControlNode(true, null, '');
	it = makeSelectP(g_StringMap, this.iProduct,   'onStringChange(this, ' + child.ID + ')', 'select_'+child.ID);
	child.setText('Product String',it);
	observer = new Observer("USBSTRINGS", updateUSBStringSelect, child.ID);
	attrib_node.add(child);	

	child = new TreeListControlNode(true, null, '');
	it = makeSelectP(g_StringMap, this.iSerialNumber, 'onStringChange(this, ' + child.ID + ')', 'select_'+child.ID);
	child.setText('Serial Number String',it);
	observer = new Observer("USBSTRINGS", updateUSBStringSelect, child.ID);
	attrib_node.add(child);	

	for (var i = 0; i < this.configs.length; ++i)
	{
		this.configs[i].loadUI(device_node);
	}
}
/********************************************************************
 * Function: USBDevice.prototype.unloadUI
 *
 ********************************************************************/
USBDevice.prototype.unloadUI = function (node)
{
//debugBox("USBDevice.prototype.unloadUI::"+node.columnText[0]);
	var attribs = node.firstChild;  // Attributes
	var child	= attribs.nextSibling;
	
	attribs = attribs.firstChild;
	this.idVendor = document.getElementById('text_'+attribs.ID).value;
	attribs = attribs.nextSibling;
	this.idProduct = document.getElementById('text_'+attribs.ID).value;
	attribs = attribs.nextSibling;
	this.bcdDevice = document.getElementById('text_'+attribs.ID).value;
	attribs = attribs.nextSibling;
	this.USBclass = document.getElementById('select_'+attribs.ID).value;
	attribs = attribs.nextSibling;
	this.subclass = document.getElementById('select_'+attribs.ID).value;
	if (this.subclass != "No Subclass")
	{
		attribs = attribs.nextSibling;
		this.protocol = document.getElementById('select_'+attribs.ID).value;
	}
	attribs = attribs.nextSibling;
	this.iManufacturer = document.getElementById('select_'+attribs.ID).value;
	attribs = attribs.nextSibling;
	this.iProduct = document.getElementById('select_'+attribs.ID).value;
	attribs = attribs.nextSibling;
	this.iSerialNumber = document.getElementById('select_'+attribs.ID).value;

	while (child != null)
	{
		this.addConfig();
		this.configs[this.configs.length - 1].unloadUI(child);
		child = child.nextSibling;
	}
}
/********************************************************************
 * Function: removeDevice
 *
 ********************************************************************/
function removeDevice(node_id)
{
	g_tree.allNodes[node_id].remove();
}
/********************************************************************
 * Function: validateVendorID
 *
 ********************************************************************/
function validateVendorID(object, node_id)
{
	if (isHex(object.value) && (object.value.length == 4))
	{
		object.value = object.value.toUpperCase();
	}
//TODO: Revert to previous value, on bad data
}
/********************************************************************
 * Function: validateProductID
 *
 ********************************************************************/
function validateProductID(object, node_id)
{
	if (isHex(object.value) && (object.value.length == 4))
	{
		object.value = object.value.toUpperCase();
	}
//TODO: Revert to previous value, on bad data
}
/********************************************************************
 * Function: validateDeviceRelease
 *
 ********************************************************************/
function validateDeviceRelease(object, node_id)
{
	var input = parseInt(object.value, 10);
	if (!isNaN(input) && (object.value.length == 4))
	{
		object.value = object.value;
	}
//TODO: Revert to previous value, on bad data
}
/********************************************************************
 * Function: onStringChange
 *
 ********************************************************************/
function onStringChange(object, node_id)
{
}
/********************************************************************
 * Function: updateUSBStringSelect
 *
 * This function is an observer callback when the USB Strings change
 *
 ********************************************************************/
function updateUSBStringSelect(node_id)
{
	var node = g_tree.allNodes[node_id];
    if (node == null)
    {
        throw 0;
    }
    
    var select = document.getElementById('select_'+node_id);
	var value = select.value;
    var value_index = select.selectedIndex;

    var len = 0;
    for (var j in g_StringMap.pairs)
        len++;

	if (select.options.length == len)// one was edited (no adds or removes)
	{
		// Search for the one that changed and see if it was the selected one
		// Should be in the same order
    	var i = 0;
		for (j in g_StringMap.pairs)
		{
			if (i == value_index)
			{
			    break;
			}
			i++;
		}
		value = g_StringMap.pairs[j].first
	}

	var it = makeSelectP(g_StringMap, value, 'onStringChange(this, ' + node_id + ')', 'select_'+node_id);
	node.setColumnText(1,it);
}
/********************************************************************
 * Function: onDeviceClassChange
 *
 ********************************************************************/
function onDeviceClassChange(object, node_id)
{
//	debugBox("onDeviceClassChange::Changing:" + object.value);
}
/********************************************************************
 * Function: onClassChange
 *
 ********************************************************************/
function onClassChange(object, node_id)
{
//	debugBox("onClassChange::Changing:" + object.value);
}
/********************************************************************
 * Function: onSubClassChange
 *
 ********************************************************************/
function onSubClassChange(object, node_id)
{
	if (object.value == g_InterfaceSubClassMap.pairs["No Subclass"].first)
	{
		// is the next node Protocol?
		var next = g_tree.allNodes[node_id].nextSibling;
		if (next != null)
		{
			if (next.columnText[0] == 'Protocol')
			{
				next.remove();
			}
		}
	}
	else
	{
		var next = g_tree.allNodes[node_id].nextSibling;
		if ((next == null) || (next.columnText[0] != 'Protocol'))
		{
			var child = new TreeListControlNode(true, null, '');
			it = makeSelectP(g_InterfaceProtocolMap, this.protocol, 'onProtocolChange(this, ' + child.ID + ')', 'select_'+child.ID);
			child.setText('Protocol',it);
			g_tree.allNodes[node_id].insertAfter(child);	
		}
	}
}
/********************************************************************
 * Function: onProtocolChange
 *
 ********************************************************************/
function onProtocolChange(object, node_id)
{
//	debugBox("onProtocolChange::Changing:" + object.value);
}
/********************************************************************
 * Method: USBConfiguration.prototype.loadUI
 *
 ********************************************************************/
USBConfiguration.prototype.loadUI = function (node)
{
//debugBox("USBConfiguration.prototype.loadUI::Enter");
	var it = new String();
	var ot = new String();

	var config_node = new TreeListControlNode(true, null, '');
	it = 'Configuration'; //: ' + this.number;
	ot = '<a href="javascript:removeConfig(' + config_node.ID + ');">Remove</a>' + ' | ' +
	     '<a href="javascript:addInterface(' + config_node.ID + ');">Add Interface</a>';
	
	config_node.setText('Configuration Descriptor',it,ot);
	node.add(config_node);

	var attrib_node;
	attrib_node = new TreeListControlNode(false, null, '');
	attrib_node.setText('Configuration Attributes');
	config_node.add(attrib_node);

	var child;
	child = new TreeListControlNode(true, null, '');
	it = makeSelectP(g_StringMap, this.iConfiguration, 'onStringChange(this, ' + child.ID + ')', 'select_'+child.ID);
	var observer = new Observer("USBSTRINGS", updateUSBStringSelect, child.ID);
	child.setText('Configuration String',it);
	attrib_node.add(child);

	child = new TreeListControlNode(true, null, '');
	it = loadTextEditZ(this.max_power, 'ValidateConfigPower', child.ID, 'text_'+child.ID);
	child.setText('Max Power (mA)',it);
	attrib_node.add(child);

	child = new TreeListControlNode(true, null, '');
	it = makeSelectP(g_PowerMap, this.power_source, 'onPoweredChange(this, ' + child.ID + ')', 'select_'+child.ID);
	child.setText('Device Power',it);
	attrib_node.add(child);

	child = new TreeListControlNode(true, null, '');
	it = makeSelectP(g_DisableEnableMap, this.remote_wakeup, 'onRWUChange(this, ' + child.ID + ')', 'select_'+child.ID);
	child.setText('Remote Wakeup',it);
	attrib_node.add(child);

    if( this.interfaces.length == 0 )
    {
        this.interfaces[0] = new USBInterface("");
    }

	for (var i = 0; i < this.interfaces.length; i++)
	{
		this.interfaces[i].loadUI(config_node);
	}	
//debugBox("USBConfiguration.prototype.loadUI::Exit");
	return node;
}
/********************************************************************
 * Method: USBConfiguration.prototype.unloadUI
 *
 ********************************************************************/
USBConfiguration.prototype.unloadUI = function (node)
{
	var attribs = node.firstChild;  // Attributes
	var child	= attribs.nextSibling;

	attribs = attribs.firstChild;
	this.iConfiguration = document.getElementById('select_'+attribs.ID).value;
	attribs = attribs.nextSibling;
	this.max_power = document.getElementById('text_'+attribs.ID).value;
	attribs = attribs.nextSibling;
	this.power_source = document.getElementById('select_'+attribs.ID).value;
	attribs = attribs.nextSibling;
	this.remote_wakeup = document.getElementById('select_'+attribs.ID).value;

	while (child != null)
	{
		this.addInterface();
		this.interfaces[this.interfaces.length - 1].unloadUI(child);
		child = child.nextSibling;
	}
}
/********************************************************************
 * Function: removeConfig
 *
 ********************************************************************/
function removeConfig(node_id)
{
	g_tree.allNodes[node_id].remove();
}
/********************************************************************
 * Function: insertConfig
 *
 ********************************************************************/
function insertConfig(node_id)
{
// TODO: Need better default name
	var config = new USBConfiguration(0);
	config.loadUI(g_tree.allNodes[node_id]);
}
/********************************************************************
 * Function: ValidateConfigPower
 *
 ********************************************************************/
function ValidateConfigPower(object, node_id)
{
	if ((object.value > 0) && (object.value < 501))
	{
	}
	else
	{
		object.value = 100; 
		alert("Max power must be between 0 and 500 mA");
	}
}
/********************************************************************
 * Function: onPoweredChange
 *
 ********************************************************************/
function onPoweredChange(object, node_id)
{
}
/********************************************************************
 * Function: onRWUChange
 *
 ********************************************************************/
function onRWUChange(object, node_id)
{
}
/********************************************************************
 * Method: USBInterface.prototype.loadUI
 *
 ********************************************************************/
USBInterface.prototype.loadUI = function (node)
{
	var it = new String();
	var ot = new String();

	var branch_node = new TreeListControlNode(true, null, '');
	it = 'Interface'; // + this.number;
	ot = '<a href="javascript:removeInterface(' + branch_node.ID + ');">Remove</a>' + ' | ' +
	     '<a href="javascript:addEndpoint(' + branch_node.ID + ');">Add Endpoint</a>';
	if (g_class == 'HID')
    {
        if (this.class_descr == null)
	    {
		    ot += ' | ' + '<a href="javascript:insertClassDescr(' + branch_node.ID + ');">Insert Class Descriptor</a>';
	    }
    }
	branch_node.setText('Interface Descriptor',it,ot);
	node.add(branch_node);

	var attrib_node;
	attrib_node = new TreeListControlNode(false, null, '');
	attrib_node.setText('Interface Attributes');
	branch_node.add(attrib_node);

	var child;

	child = new TreeListControlNode(true, null, '');
	it = makeSelectP(g_StringMap, this.iInterface, 'onStringChange(this, ' + child.ID + ')',  'select_'+child.ID);
	var observer = new Observer("USBSTRINGS", updateUSBStringSelect, child.ID);
	child.setText('Interface String',it);
	attrib_node.add(child);	

	child = new TreeListControlNode(true, null, '');
	it = makeSelectP(g_InterfaceClassMap, this.USBclass, 'onClassChange(this, ' + child.ID + ')', 'select_'+child.ID);
	child.setText('Class',it);
	attrib_node.add(child);	

	child = new TreeListControlNode(true, null, '');
	it = makeSelectP(g_InterfaceSubClassMap, this.subclass, 'onSubClassChange(this, ' + child.ID + ')', 'select_'+child.ID);
	child.setText('Subclass',it);
	attrib_node.add(child);	

	if (this.subclass != "No Subclass")
	{
		child = new TreeListControlNode(true, null, '');
		it = makeSelectP(g_InterfaceProtocolMap, this.protocol, 'onProtocolChange(this, ' + child.ID + ')', 'select_'+child.ID);
		child.setText('Protocol',it);
		attrib_node.add(child);	
	}

	if (this.class_descr != null)
	{
		this.class_descr.loadUI(branch_node);
	}

	for (var i = 0; i < this.endpoints.length; i++) // >
	{
		this.endpoints[i].loadUI(branch_node);
	} 	

	return node;
}
/********************************************************************
 * Method: USBInterface.prototype.unloadUI
 *
 ********************************************************************/
USBInterface.prototype.unloadUI = function (node)
{
	var attribs = node.firstChild;  // Attributes
	var child = attribs.nextSibling;

	attribs = attribs.firstChild;
	this.iInterface = document.getElementById('select_'+attribs.ID).value;
	attribs = attribs.nextSibling;
	this.USBclass = document.getElementById('select_'+attribs.ID).value;
	attribs = attribs.nextSibling;
	this.subclass = document.getElementById('select_'+attribs.ID).value;
	attribs = attribs.nextSibling;

	if (this.subclass != "No Subclass")
	{
		this.protocol = document.getElementById('select_'+attribs.ID).value;
		attribs = attribs.nextSibling;
	}
	if (child != null)
	{
		if (child.columnText[0] != 'Endpoint Descriptor')
		{
			this.class_descr = new USBClassHID('DEFAULT');
			this.class_descr.unloadUI(child);
			child = child.nextSibling;
		}
	}
	while (child != null)
	{
		this.addEndpoint();
		this.endpoints[this.endpoints.length - 1].unloadUI(child);
		child = child.nextSibling;
	}
}
/********************************************************************
 * Function: removeInterface
 *
 ********************************************************************/
function removeInterface(node_id)
{
	g_tree.allNodes[node_id].remove();
}
/********************************************************************
 * Function: addInterface
 *
 ********************************************************************/
function addInterface(node_id)
{
	var interface = new USBInterface(0);
	interface.loadUI(g_tree.allNodes[node_id]);
}
/********************************************************************
 * Function: onInterfaceStringChange
 *
 ********************************************************************/
function onInterfaceStringChange(object, node_id)
{
}
/********************************************************************
 * Method: USBClassHID.prototype.loadUI
 *
 ********************************************************************/
USBClassHID.prototype.loadUI = function (node)
{
	var it = new String();
	var ot = new String();

	var branch_node = new TreeListControlNode(false, null, '');
	it = '';
	ot = '<a href="javascript:removeClassDescr(' + branch_node.ID + ');">Remove</a>';
	branch_node.setText('HID Class Descriptor',it,ot);
	node.firstChild.insertAfter(branch_node);

	var child;
	child = new TreeListControlNode(false, null, '');
	it = makeSelectP(g_HIDDescriptorTypeMap, this.descr_type, 'onDescriptorTypeChange(this, ' + child.ID + ')', 'select_'+child.ID);
	child.setText('Descriptor Type',it);
	branch_node.add(child);	

	child = new TreeListControlNode(false, null, '');
	it = makeSelectP(g_CountryCodeMap, this.country_code, 'onCountryCodeChange(this, ' + child.ID + ')', 'select_'+child.ID);
	child.setText('Country Code',it);
	branch_node.add(child);	
	
	child = new TreeListControlNode(false, null, '');
	it = makeSelectP(g_HIDReportMap, this.hid_report, 'onHIDReportChange(this, ' + child.ID + ')', 'select_'+child.ID);
	var observer = new Observer("USBREPORTS", updateHIDReportSelect, child.ID);
	child.setText('HID Report',it);
	branch_node.add(child);	
}	
/********************************************************************
 * Method: USBClassHID.prototype.unloadUI
 *
 ********************************************************************/
USBClassHID.prototype.unloadUI = function (node)
{
	var child = node.firstChild;  // Attributes
	this.descr_type = document.getElementById('select_'+child.ID).value;
	child = child.nextSibling;
	this.country_code = document.getElementById('select_'+child.ID).value;
	child = child.nextSibling;
	this.hid_report = document.getElementById('select_'+child.ID).value;
}
/********************************************************************
 * Function: removeClassDescr
 *
 ********************************************************************/
function removeClassDescr(node_id)
{
	var node = g_tree.allNodes[node_id];
	var parent = node.parentNode;
	node.remove();
	var ot = '<a href="javascript:removeInterface(' + node_id + ');">Remove</a>' + ' | ' +
	     '<a href="javascript:addEndpoint(' + node_id + ');">Add Endpoint</a>';
	ot += ' | ' + '<a href="javascript:insertClassDescr(' + parent.ID + ');">Insert Class Descriptor</a>';
	parent.setColumnText(2,ot);
}
/********************************************************************
 * Function: insertClassDescr
 *
 ********************************************************************/
function insertClassDescr(node_id)
{
	var node = g_tree.allNodes[node_id];
	var class_descr = new USBClassHID(0);
	class_descr.loadUI(node);
	var ot = '<a href="javascript:removeInterface(' + node_id + ');">Remove</a>' + ' | ' +
	     '<a href="javascript:addEndpoint(' + node_id + ');">Add Endpoint</a>';
	node.setColumnText(2,ot);
}
/********************************************************************
 * Method: USBEndpoint.prototype.loadUI
 *
 ********************************************************************/
USBEndpoint.prototype.loadUI = function (node)
{
	var it = new String();
	var ot = new String();

	var branch_node = new TreeListControlNode(false, null, '');
	it = 'Endpoint'; //: ' + this.number;
	ot = '<a href="javascript:removeEndpoint(' + branch_node.ID + ');">Remove</a>'
	branch_node.setText('Endpoint Descriptor',it,ot);
	node.add(branch_node);

	var attrib_node;
	attrib_node = new TreeListControlNode(false, null, '');
	attrib_node.setText('Endpoint Attributes');
	branch_node.add(attrib_node);

	var child;

	child = new TreeListControlNode(true, null, '');
	it = makeSelectP(g_EndpointNumberMap, this.address, 'ValidateEndpointNumber(this, ' + child.ID + ')', 'select_'+child.ID);
	child.setText('Endpoint Number',it);
	attrib_node.add(child);

	child = new TreeListControlNode(true, null, '');
	it = makeSelectP(g_EndpointDirectionMap, this.direction, 'onDirectionChange(this, ' + child.ID + ')', 'select_'+child.ID);
	child.setText('Direction',it);
	attrib_node.add(child);	

	child = new TreeListControlNode(true, null, '');
	it = makeSelectP(g_EndpointTransferTypeMap, this.transfer_type, 'onTransferTypeChange(this, ' + child.ID + ')', 'select_'+child.ID);
	child.setText('Transfer Type',it);
	attrib_node.add(child);	

	if (this.transfer_type == "ISOC")
	{
		child = new TreeListControlNode(true, null, '');
		it = makeSelectP(g_EndpointSyncTypeMap, this.sync_type, 'onSyncTypeChange(this, ' + child.ID + ')', 'select_'+child.ID);
		child.setText('Sync Type',it);
		attrib_node.add(child);	

		child = new TreeListControlNode(true, null, '');
		it = makeSelectP(g_EndpointUsageTypeMap, this.usage_type, 'onUsageTypeChange(this, ' + child.ID + ')', 'select_'+child.ID);
		child.setText('Usage Type',it);
		attrib_node.add(child);	
	}
	child = new TreeListControlNode(true, null, '');
	it = loadTextEditZ(this.interval, 'ValidateEndpointInterval', child.ID, 'text_'+child.ID);
	child.setText('Interval',it);
	attrib_node.add(child);
	
	child = new TreeListControlNode(true, null, '');
	it = loadTextEditZ(this.max_packet_size, 'ValidateEndpointMaxPacketSize', child.ID, 'text_'+child.ID);
	child.setText('Max Packet Size',it);
	attrib_node.add(child);

	return node;
}
/********************************************************************
 * Method: USBEndpoint.prototype.unloadUI
 *
 ********************************************************************/
USBEndpoint.prototype.unloadUI = function (node)
{
	var attribs = node.firstChild;  // Attributes
	var child = attribs.firstChild;
	this.address = document.getElementById('select_'+child.ID).value;
	child = child.nextSibling;
	this.direction = document.getElementById('select_'+child.ID).value;
	child = child.nextSibling;
	this.transfer_type = document.getElementById('select_'+child.ID).value;
	child = child.nextSibling;
	if (this.transfer_type == "ISOC")
	{
		this.sync_type = document.getElementById('select_'+child.ID).value;
		child = child.nextSibling;
		this.usage_type = document.getElementById('select_'+child.ID).value;
		child = child.nextSibling;
	}
	this.interval = document.getElementById('text_'+child.ID).value;
	child = child.nextSibling;
	this.max_packet_size = document.getElementById('text_'+child.ID).value;
}
/********************************************************************
 * Function: removeEndpoint
 *
 ********************************************************************/
function removeEndpoint(node_id)
{
	g_tree.allNodes[node_id].remove();
}
/********************************************************************
 * Function: insertEndpoint
 *
 ********************************************************************/
function addEndpoint(node_id)
{
	var ep = new USBEndpoint(null);
	ep.loadUI(g_tree.allNodes[node_id]);
}
/********************************************************************
 * Function: onTransferTypeChange
 *
 ********************************************************************/
function onTransferTypeChange(object, node_id)
{
//	debugBox("onTransferTypeChange::Changing:" + object.value);
	var node = g_tree.allNodes[node_id];
	
	if (object.value != "ISOC")
	{
		if (node.nextSibling != null)
		{
			if (node.nextSibling.columnText[0] == 'Sync Type')
			{
				node.nextSibling.remove();	
			}
			if (node.nextSibling.columnText[0] == 'Usage Type')
			{
				node.nextSibling.remove();	
			}
		}
	}
	else
	{
		// Add a ISOC nodes
		var child = new TreeListControlNode(true, null, '');
		it = makeSelectP(g_EndpointSyncTypeMap, 'No Synchronization', 'onSyncTypeChange(this, ' + child.ID + ')', 'select_'+child.ID);
		child.setText('Sync Type',it);
		node.insertAfter(child);	
		node = child;
		
		child = new TreeListControlNode(true, null, '');
		it = makeSelectP(g_EndpointUsageTypeMap, 'Data Endpoint', 'onUsageTypeChange(this, ' + child.ID + ')', 'select_'+child.ID);
		child.setText('Usage Type',it);
		node.insertAfter(child);	
	}
}
/********************************************************************
 * Function: onDescriptorTypeChange
 *
 ********************************************************************/
function onDescriptorTypeChange(object, node_id)
{
}
/********************************************************************
 * Function: onCountryCodeChange
 *
 ********************************************************************/
function onCountryCodeChange(object, node_id)
{
}
/********************************************************************
 * Function: onHIDReportChange
 *
 ********************************************************************/
function onHIDReportChange(object, node_id)
{
}
/********************************************************************
 * Function: onProtocolChange
 *
 ********************************************************************/
function onProtocolChange(object, node_id)
{
}
/********************************************************************
 * Function: onDirectionChange
 *
 ********************************************************************/
function onDirectionChange(object, node_id)
{
}
/********************************************************************
 * Function: onSyncTypeChange
 *
 ********************************************************************/
function onSyncTypeChange(object, node_id)
{
}
/********************************************************************
 * Function: onUsageTypeChange
 *
 ********************************************************************/
function onUsageTypeChange(object, node_id)
{
}
/********************************************************************
 * Function: ValidateEndpointNumber
 *
 ********************************************************************/
function ValidateEndpointNumber(object, node_id)
{
}
/********************************************************************
 * Function: ValidateEndpointInterval
 *
 ********************************************************************/
function ValidateEndpointInterval(object, node_id)
{
	if ((object.value >= 0) && (object.value < 256))	// >
	{
	}
	else
	{
		alert("Invalid Interval value (must be 0-255)");
		object.value = 10;
	}
}
/********************************************************************
 * Function: ValidateEndpointMaxPacketSize
 *
 ********************************************************************/
function ValidateEndpointMaxPacketSize(object, node_id)
{
	if ((object.value >= 0) && (object.value < 1024))	// >
	{
	}
	else
	{
		alert("Invalid Interval value (must be 0-1023)");
		object.value = 8;
	}
}
/********************************************************************
 * Function: updateHIDReportSelect
 *
 * This function is an observer callback when the HID Reports change
 *
 ********************************************************************/
function updateHIDReportSelect(node_id)
{
	var node = g_tree.allNodes[node_id];
    if (node == null)
    {
        throw 0;
    }
    
    var select = document.getElementById('select_'+node_id);
	var value = select.value;
    var value_index = select.selectedIndex;

    var len = 0;
    for (var j in g_HIDReportMap.pairs)
        len++;

	if (select.options.length == len)// one was edited (no adds or removes)
	{
		// Search for the one that changed and see if it was the selected one
		// Should be in the same order
    	var i = 0;
		for (j in g_HIDReportMap.pairs)
		{
			if (i == value_index)
			{
			    break;
			}
			i++;
		}
		value = g_HIDReportMap.pairs[j].first
	}

	var it = makeSelectP(g_HIDReportMap, value, 'onHIDReportChange(this, ' + node_id + ')', 'select_'+node_id);
	node.setColumnText(1,it);
}
