/********************************************************************
 * Global
 *
 ********************************************************************/

var sUMName                 = "USB";
var g_DescrStyleSheet       = "GenDescr.xsl";
var g_ParamStyleSheet       = "GenParams.xsl";
var g_StringStyleSheet      = "GenString.xsl";
var g_HIDStyleSheet			= "GenHIDRpt.xsl";


/*********************************************************************
 * Function:	USBGenerateCode
 *
 * This function is an entry point for code generation of the USB
 * user module.
 *
 *********************************************************************/

function USBGenerateCode(sInstanceName)
{
    var xmlDOM       = PS.GetXMLDOM();
	var deviceEditor = PS.GetDeviceEditor();
    var UMPath       = deviceEditor.GetUserModulePath(sUMName);
	var raw_descrs   = new String();

    // Get the current overlay
    xmlDOM.setProperty("SelectionLanguage","XPath");
    xmlDOM.loadXML( deviceEditor.EnumerateConfigs2() );
    var configs = xmlDOM.documentElement.selectNodes("//PSOC_CONFIG");
    var sOverlay = configs.item(0).getAttribute("NAME");

    // Generate code for descriptor
    var xmlDESCR = getPSoCParameter(deviceEditor, sOverlay, sInstanceName, 
                                    'DSHID_xmlDESCR');
	try {
        xmlDOM.loadXML(xmlDESCR);
        // Before we gen code, update the instance name in case the UM was renamed
        var workElements=xmlDOM.getElementsByTagName("USB_ROOT");
        workElements.item(0).setAttribute("INSTANCE_NAME", sInstanceName);
	    raw_descrs += genDescriptors(xmlDOM, UMPath + "\\" + g_DescrStyleSheet);

        // Generate code for HID class if exists
        var USBClass = getPSoCParameter(deviceEditor, sOverlay, sInstanceName, 
                                        'DSHID_Class');

        if( USBClass == 'HID' )
        {
            var xmlHID = getPSoCParameter(deviceEditor, sOverlay, sInstanceName, 
                                          'DSHID_xmlHID');
            xmlDOM.loadXML(xmlHID);
            // Before we gen code, update the instance name in case the UM was renamed
            workElements=xmlDOM.getElementsByTagName("HID_REPORT_ROOT");
            workElements.item(0).setAttribute("NAME", sInstanceName);

            raw_descrs += genDescriptors(xmlDOM, UMPath + "\\" + g_HIDStyleSheet);
        }

        // Generate strings
        var xmlSTRING = getPSoCParameter(deviceEditor, sOverlay, sInstanceName, 
                                        'DSHID_xmlSTRING');
        xmlDOM.loadXML(xmlSTRING);
        // Before we gen code, update the instance name in case the UM was renamed
        workElements=xmlDOM.getElementsByTagName("STRING_ROOT");
        workElements.item(0).setAttribute("INSTANCE_NAME", sInstanceName);

	    raw_descrs += genDescriptors(xmlDOM, UMPath + "\\" + g_StringStyleSheet);

        // Fix end of line
	    var regex = /\r/g;
	    raw_descrs = raw_descrs.replace(regex,'');

	}
	catch (e) {
		raw_descrs = "PSOC_ERROR Please run the USB Setup Wizard.  Device Editor, Right Click the USB User Module\n"
+ "; This message will only appear if the USB Setup Wizard has not be run and the descriptors\n"
+ "; and associated data structures have been created.\n"
+ "; After running the USB Setup Wizard, you must also select the Config/Generate Application\n"
+ "; menu item from PSoC Designer in order to generate USB User Module data structures and\n"
+ "; descriptors."; // If any of the code generation fails, clear the descriptors\n"
	}
    // Save code to parameter for file substitution
	setPSoCParameter(deviceEditor, sOverlay, sInstanceName, 
                     "DSHID_hfDescriptors", raw_descrs);
}


/*********************************************************************
 * Function:	genDescriptors
 *
 * This function applies the specified stylesheet to the xmlDocument
 * and returns the rendered data as the return value.
 *
 *********************************************************************/

function genDescriptors(xmlDOM, stylesheet)
{
	var stylesheetXSL = PS.GetXMLDOM();
	stylesheetXSL.async="false"
	stylesheetXSL.load(stylesheet);
	var descriptors = xmlDOM.transformNode(stylesheetXSL);
 	return descriptors;
}


/*********************************************************************
 * Function:	setPSoCParameter
 *
 * This function provide the means to update a User Module Parameter
 * inside the PSoC Designer Context
 *
 *********************************************************************/

function setPSoCParameter(deviceEditor, sOverlay, sInstanceName,
                          paramName, paramValue)
{
    deviceEditor.lSetUserModuleParameter2(sOverlay, sInstanceName, 
                                          paramName, paramValue);
}


/*********************************************************************
 * Function:    getPSoCParameter
 *
 * This function provide the means to get the current value of a
 * User Module Parameter inside the PSoC Designer Context
 *
 *********************************************************************/

function getPSoCParameter(deviceEditor, sOverlay, sInstanceName, paramName)
{
    return deviceEditor.GetUserModuleParameter2(sOverlay, sInstanceName, 
                                                paramName);
}



