/********************************************************************
 * Global
 *
 ********************************************************************/
var g_ps					= null;
var g_pDlg					= null;
var g_DeviceEditor			= null;
var g_ProjectManager		= null;
var g_xmlDoc				= null;
var g_sXMLPath				= null;
var g_UMPath                = null;

var g_sOverLay				= null;
var g_sUMName				= null;
var g_sInstanceName		    = null;

var g_wizardRan             = 0;
var g_Root			        = null;
var g_Strings			    = null;
var g_Reports			    = null;

var g_tree					= null;
var g_StringTree			= null;
var g_ReportTree			= null;

var g_StringMap			    = null;
var g_HIDReportMap		    = null;

var g_DescrStyleSheet       = "GenDescr.xsl";
var g_ParamStyleSheet       = "GenParams.xsl";
var g_StringStyleSheet      = "GenString.xsl";
var g_HIDStyleSheet			= "GenHIDRpt.xsl";
var g_RAMStyleSheet			= "GenRAM.xsl";

var g_xmlDESCR              = null;
var g_xmlSTRING             = null;
var g_xmlHID                = null;

var g_HIDTemplateDirectory  = 'TEMPLATE_DIR.XML';
var g_HIDTemplateBase       = 'HIDTemplate_';

var g_class;
var g_INTTransfer;
var g_BulkTransfer;
var g_ISOCTransfer;
var g_NumDataEndpoints;

var deviceHeadings = new Array(new pair(300, 'Descriptor'), new pair(250, 'Data'), new pair(250, 'Operations'));
var stringHeadings = new Array(new pair(300, 'String/LANGID'), new pair(250, ''), new pair(250, 'Operations'));

/*********************************************************************
 * Function:	setPSoCParameter
 *
 * This function provide the means to update a User Module Parameter
 * inside the PSoC Designer Context
 *
 *********************************************************************/
function setPSoCParameter(paramName, paramValue)
{
    g_DeviceEditor.lSetUserModuleParameter2(g_sOverLay, g_sInstanceName, paramName, paramValue);
}
/*********************************************************************
 * Function:	getPSoCParameter
 *
 * This function provide the means to get the current value of a
 * User Module Parameter inside the PSoC Designer Context
 *
 *********************************************************************/
function getPSoCParameter(paramName)
{
    return g_DeviceEditor.GetUserModuleParameter2(g_sOverLay, g_sInstanceName, paramName);
}
/*********************************************************************
 * Function:	setPSoC
 *
 * This function is invoked by PSoC Designer to give the wizard the
 * necessary context handles for communication between the PSoC
 * Designer User Module instance and the wizard.
 *
 *********************************************************************/
function setPSoC(ps, pDlg, sOverLay, sUMName, sInstanceName)
{
	g_ps			= ps;
	g_pDlg			= pDlg;
	g_sOverLay	= sOverLay;
	g_sUMName	= sUMName;
	g_sInstanceName = sInstanceName;

	g_xmlDoc        = g_ps.GetXMLDOM();
	g_DeviceEditor  = g_ps.GetDeviceEditor();
	g_ProjectManager = g_ps.GetProjectManager();
    g_UMPath = g_DeviceEditor.GetUserModulePath(sUMName);
	
    GetAllUMParameters();
    // Create two data maps to control the pull downs for # of endpoints and transfer types
    //   g_EndpointTransferTypeMap and g_EndpointNumber exist in another file and we are going
    //   to override them by deleting them and creating new objects
    //
    if (g_EndpointTransferTypeMap != null)
    {
        delete g_EndpointTransferTypeMap;
    }
	g_EndpointTransferTypeMap = new map();    
	if (g_INTTransfer == 'Enabled')
        g_EndpointTransferTypeMap.add("INT", 3);
    if (g_BulkTransfer == 'Enabled')
        g_EndpointTransferTypeMap.add("ISOC", 1);
    if (g_ISOCTransfer == 'Enabled')
        g_EndpointTransferTypeMap.add("BULK", 2);

    if (g_EndpointNumberMap != null)
    {
        delete g_EndpointNumberMap;
    }
	g_EndpointNumberMap = new map();    

    for (var j = 1; j < g_NumDataEndpoints + 1; j++)
    {
        g_EndpointNumberMap.add(String("EP"+j), j);
    }

	if (g_DeviceEditor != null)
	{
		if(g_xmlDoc != null)
		{
			g_xmlDoc.onreadystatechange = xmlDocOnReadyStateChange;

			var sPath = g_ProjectManager.GetProjectPath();
			// Load the strings first
			if (g_wizardRan != 0)
			{
			    g_xmlDoc.loadXML(g_xmlSTRING);
				g_Strings = new USBStrings();
				g_Strings.serializeIn(g_xmlDoc.documentElement);
			}
			else
			{
				g_Strings = new USBStrings();
                g_Strings.addLANGID();
			}
            g_Strings.instance_name = g_sInstanceName;  // Update the instance name
                                                        //  in case the UM was renamed

			g_StringMap = loadStringMap(g_Strings.strings, g_StringMap);

			if (g_class == 'HID')
            {
			    if (g_wizardRan != 0)
			    {
			        g_xmlDoc.loadXML(g_xmlHID);
				    g_Reports = new Reports();
				    g_Reports.serializeIn(g_xmlDoc.documentElement);
			    }
			    else
			    {
				    g_Reports = new Reports();
			    }
                g_Reports.name = g_sInstanceName;   // Update the instance name
                                                    //  in case the UM was renamed
			    g_HIDReportMap = loadHIDReportMap(g_Reports.reports, g_HIDReportMap);
            }			
			
			
			if (g_wizardRan != 0)
			{
			    g_xmlDoc.loadXML(g_xmlDESCR);
			    g_Root = new USBRoot();
                g_Root.serializeIn(g_xmlDoc.documentElement);
			}
			else
			{
				g_Root = new USBRoot();
				g_Root.addDevice();
				g_Root.devices[0].addConfig();
				g_Root.devices[0].configs[0].addInterface();
				g_Root.devices[0].configs[0].interfaces[0].addEndpoint();
				g_Root.devices[0].configs[0].interfaces[0];
			    if (g_class == 'HID')
			    {
				    g_Root.devices[0].configs[0].interfaces[0].addClass();
			    }				
			}
            g_Root.instance_name = g_sInstanceName; // Update the instance name
                                                    //  in case the UM was renamed
		}
	}
    if(g_pDlg != null)
    {
        g_pDlg.SetCaption("USB Setup Wizard");
    }
    
    refreshApp();
}
/*********************************************************************
 * Function:	GetAllUMParameters
 *
 * This function gets all of the UM parameter values from PSoC Designer.
 *********************************************************************/
function GetAllUMParameters()
{
    // The paramenters are API_HIDDEN and limit the options provided
    // to the user based on the resources availble in the device.
    g_wizardRan         = getPSoCParameter('DSHID_bWizardRan');
    g_class             = getPSoCParameter('DSHID_Class');
    g_INTTransfer       = getPSoCParameter('DSHID_SupportsINT');  
    g_BulkTransfer      = getPSoCParameter('DSHID_SupportsBULK'); 
    g_ISOCTransfer      = getPSoCParameter('DSHID_SupportsISOC');
    g_NumDataEndpoints  = new Number(getPSoCParameter('DSHID_NumDataEndpoints'));

    g_xmlDESCR          = getPSoCParameter('DSHID_xmlDESCR');
    g_xmlSTRING         = getPSoCParameter('DSHID_xmlSTRING');
    g_xmlHID            = getPSoCParameter('DSHID_xmlHID');
    g_xmlVENDOR         = getPSoCParameter('DSHID_xmlVENDOR');
}

/*********************************************************************
 * Function:	onSaveGenerateDescriptors
 *
 * This function makes backup the previous XML data files that
 * represent the descriptors.  It saves the current state of the
 * XML data files to include any updated done by the user.  It generates
 * all of the descriptor code and finally updates the descriptors
 * in the PSoC Designer/User Module context.
 *
 *********************************************************************/
function onSaveGenerateDescriptors()
{
	var xmlDoc;

	xmlDoc = g_ps.GetXMLDOM();
// Device data
	var d = new USBRoot();
	d.unloadUI(g_tree.rootNode);
	xmlDoc.documentElement = d.serializeOut(xmlDoc);
    setPSoCParameter("DSHID_xmlDESCR", xmlDoc.xml);	// 

// HID Reports
    if (g_class == 'HID')
    {
	    var r = new Reports();
	    r.unloadUI(g_ReportTree.rootNode);
	    xmlDoc.documentElement = r.serializeOut(xmlDoc);
        setPSoCParameter("DSHID_xmlHID", xmlDoc.xml);	// 
    }	
// Back up the original, serializeOut the current, the generate code
// Strings
	var s = new USBStrings();
	s.unloadUI(g_StringTree.rootNode);
	xmlDoc.documentElement = s.serializeOut(xmlDoc);
    setPSoCParameter("DSHID_xmlSTRING", xmlDoc.xml);	// 

// This section updates the various User Module Parameters
	setPSoCParameter("DSHID_bNumStringDescrs", s.strings.length);	// The number of string descriptors
	setPSoCParameter("DSHID_bNumDevices", d.devices.length);	// The number of Devices

	var max_interfaces = ExtractParameter('MAX_NUM_INTERFACES', 0, d, g_UMPath + "\\" + g_ParamStyleSheet);
	setPSoCParameter("DSHID_bNumInterfaces", max_interfaces);
    if (g_class == 'HID')
    	setPSoCParameter("DSHID_bNumHIDReports", r.reports.length);	// The number of HID Report descriptors

	setPSoCParameter("DSHID_bWizardRan", 1);				// Set the assembler value showing the wizard ran

}
/*********************************************************************
 * Function:	genDescriptors
 *
 * This function applies the specified stylesheet to the xmlDocument
 * and returns the rendered data as the return value.
 *
 *********************************************************************/
function genDescriptors(xmlDoc, stylesheet)
{
	var stylesheetXSL = g_ps.GetXMLDOM();
	stylesheetXSL.async="false"
	stylesheetXSL.load(stylesheet);
	var descriptors = xmlDoc.transformNode(stylesheetXSL);
 	return descriptors;
 }
/********************************************************************
 * Function: ExtractParameter
 *
 ********************************************************************/
function ExtractParameter(element, param, root, stylesheet)
{
    var xmlDoc = g_ps.GetXMLDOM();
    var xmlElement = xmlDoc.createElement(element);
    if (param.constructor == Array)
    {
        debugBox("ExtractParameter::Array");
        for (var i = 0; i < param.length; ++i)
        {
            xmlElement.setAttribute('PARAM'+i, param[i]);
        }
    }
    else
    {
        xmlElement.setAttribute('PARAM0', param);
    }
    xmlElement.appendChild(root.serializeOut (xmlDoc));
	xmlDoc.documentElement = xmlElement;

	var stylesheetXSL = g_ps.GetXMLDOM();
	stylesheetXSL.async="false"
	stylesheetXSL.load(stylesheet);
	var value = xmlDoc.transformNode(stylesheetXSL);
	return value;
}
/*********************************************************************
 * Function:	refreshApp
 *
 * This function refreshes all of the tree controls in the application.
 *
 *********************************************************************/
function refreshApp()
{
	if (g_Reports != null)
        g_ReportTree = loadTreeX('HID_LOC',    g_Reports, deviceHeadings);
	g_tree       = loadTreeX('TREE_LOC',   g_Root,    deviceHeadings);
	g_StringTree = loadTreeX('STRING_LOC', g_Strings, stringHeadings);
}
/*********************************************************************
 * The following functions may need to be refactored into other function
 * or at least moved to other locations.
 *********************************************************************/
function xmlDocOnReadyStateChange()
{
	//debugBox("xmlDocOnReadyStateChange::Entering: " + g_xmlDoc.readyState );
}
/*********************************************************************
 * Function:	onCancel
 *
 *********************************************************************/
function onCancel()
{
    g_pDlg.Close(0);
}
/*********************************************************************
 * Function:	onOK
 *
 *********************************************************************/
function onOK()
{
    onSaveGenerateDescriptors();
    g_pDlg.Close(0);
}
/*********************************************************************
 * Function:	onApply
 *
 *********************************************************************/
function onApply()
{
    onSaveGenerateDescriptors();
}
function checkDuplicateName(list,name)
{
	bDup = false;
	
	for (var i = 0; i < list.length; i++) // >
	{
		if (list[i].name == name)
		{
			bDup = true;
		}
	}
	return bDup;
}
// Data validation routines
function checkDuplicateAttribute(objList, attrib, value)
{
	var bDup = false;
	
	debugBox("checkDuplicateAttribute::Entered");
	for (var i = 0; i < objList.length; i++) //>
	{
		var l_value = objList[i].getAttribute(attrib);
		if (l_value == value)
		{
			bDup = true;
		}
	}
//	debugBox("checkDuplicateAttribute::Exiting");
	return bDup;
}

