/********************************************************************
 * Globals
 *
 ********************************************************************/

var g_MainItemTypeMap = new map();
g_MainItemTypeMap.add("", 0);
g_MainItemTypeMap.add("Input", 1);
g_MainItemTypeMap.add("Output", 2);
g_MainItemTypeMap.add("Feature", 3);
g_MainItemTypeMap.add("Collection", 4);
g_MainItemTypeMap.add("End Collection", 5);

var g_HIDItemMap = new map();
g_HIDItemMap.add('Undefined',			0x00);
g_HIDItemMap.add('Input',				0x80);
g_HIDItemMap.add('Output',				0x90);
g_HIDItemMap.add('Feature',				0xB0);
g_HIDItemMap.add('Collection',			0xA0);
g_HIDItemMap.add('Usage Page',			0x04);
g_HIDItemMap.add('Logical Minimum',		0x14);
g_HIDItemMap.add('Logical Maximum',		0x24);
g_HIDItemMap.add('Physical Minimum',	0x34);
g_HIDItemMap.add('Physical Maximum',	0x44);
g_HIDItemMap.add('Unit Exponent',		0x54);
g_HIDItemMap.add('Unit',				0x64);
g_HIDItemMap.add('Report Size',			0x74);
g_HIDItemMap.add('Report ID',			0x84);
g_HIDItemMap.add('Report Count',		0x94);
g_HIDItemMap.add('Push',				0xA4);
g_HIDItemMap.add('Pop',					0xB4);
g_HIDItemMap.add('Usage',				0x08);
g_HIDItemMap.add('Usage Minimum',		0x18);
g_HIDItemMap.add('Usage Maximum',		0x28);
g_HIDItemMap.add('Designator Index',	0x38);
g_HIDItemMap.add('Designator Minimum',	0x48);
g_HIDItemMap.add('Designator Maximum',	0x58);
g_HIDItemMap.add('String Index',		0x78);
g_HIDItemMap.add('String Minimum',		0x88);
g_HIDItemMap.add('String Maximum',		0x98);
g_HIDItemMap.add('Delimiter',			0xA8);

var g_HIDItemSelectMap = new map();
for (var j in g_HIDItemMap.pairs)
{
	g_HIDItemSelectMap.add(g_HIDItemMap.pairs[j].first, g_HIDItemMap.pairs[j].second);
}
g_HIDItemMap.add('End Collection',	0xC0);


var g_HIDItemOverrideMap = new map();
// This violates encapsalation slightly.  editForm shouldn't be done here.  So, refactor it later
//														  toString, editForm
g_HIDItemOverrideMap.add('Undefined',			new Array(null, null));
g_HIDItemOverrideMap.add('Input',				new Array(toStringInput, null));
g_HIDItemOverrideMap.add('Output',				new Array(toStringOutput, null));
g_HIDItemOverrideMap.add('Feature',				new Array(toStringOutput, null));
g_HIDItemOverrideMap.add('Collection',			new Array(toStringCollection, null));
g_HIDItemOverrideMap.add('End Collection',	    new Array(null, null));
g_HIDItemOverrideMap.add('Usage Page',			new Array(null, null));
g_HIDItemOverrideMap.add('Logical Minimum',		new Array(null, null));
g_HIDItemOverrideMap.add('Logical Maximum',		new Array(null, null));
g_HIDItemOverrideMap.add('Physical Minimum',	new Array(null, null));
g_HIDItemOverrideMap.add('Physical Maximum',	new Array(null, null));
g_HIDItemOverrideMap.add('Unit Exponent',		new Array(null, null));
g_HIDItemOverrideMap.add('Unit',				new Array(null, null));
g_HIDItemOverrideMap.add('Report Size',			new Array(null, null));
g_HIDItemOverrideMap.add('Report ID',			new Array(null, null));
g_HIDItemOverrideMap.add('Report Count',		new Array(null, null));
g_HIDItemOverrideMap.add('Push',				new Array(null, null));
g_HIDItemOverrideMap.add('Pop',					new Array(null, null));
g_HIDItemOverrideMap.add('Usage',				new Array(null, null));
g_HIDItemOverrideMap.add('Usage Minimum',		new Array(null, null));
g_HIDItemOverrideMap.add('Usage Maximum',		new Array(null, null));
g_HIDItemOverrideMap.add('Designator Index',	new Array(null, null));
g_HIDItemOverrideMap.add('Designator Minimum',	new Array(null, null));
g_HIDItemOverrideMap.add('Designator Maximum',	new Array(null, null));
g_HIDItemOverrideMap.add('String Index',		new Array(null, null));
g_HIDItemOverrideMap.add('String Minimum',		new Array(null, null));
g_HIDItemOverrideMap.add('String Maximum',		new Array(null, null));
g_HIDItemOverrideMap.add('Delimiter',			new Array(null, null));


var s_HID_REPORT_ROOT		= "HID_REPORT_ROOT";
var s_HID_REPORT			= "HID_REPORT";
var s_HID_ITEM				= "HID_ITEM";
var s_ITEM_DATA				= "ITEM_DATA";
var s_ATTRIB_NAME           = "NAME";
var s_ATTRIB_HASH           = "HASH";
var s_ATTRIB_ITEM_TYPE		= "ITEM_TYPE";
var s_ATTRIB_ITEM_CODE		= "ITEM_CODE";
var s_ATTRIB_SIZE			= "SIZE";
var s_ATTRIB_VALUE			= "VALUE";
var s_ATTRIB_ORDER			= "ORDER";
s_ATTRIB_HASH

/*********************************************************************
 * Class: Reports (Container)
 *
 *********************************************************************/
function Reports()
{
	this.name = '';
	this.next_hash  = 0;
    this.reports    = new Array();
}
/*********************************************************************
 * Method: Reports.prototype.serializeIn
 *
 *********************************************************************/
Reports.prototype.serializeIn = function (xmlElement)
{
	this.name  = serializeInFromString(xmlElement.getAttribute(s_ATTRIB_NAME), '');
	this.next_hash  = serializeInFromString(xmlElement.getAttribute(s_ATTRIB_NEXT_HASH), 0);
	var reports = xmlElement.getElementsByTagName(s_HID_REPORT);
	for (var i = 0; i < reports.length; i++)	// >
	{
		this.reports[i] = new HIDReport(0);
		this.reports[i].serializeIn(reports[i]);
	}
}
/*********************************************************************
 * Method: Reports.prototype.serializeOut
 *
 *********************************************************************/
Reports.prototype.serializeOut = function (xmlDoc)
{
	var xmlElement = xmlDoc.createElement(s_HID_REPORT_ROOT);

	xmlElement.setAttribute(s_ATTRIB_NAME,				this.name);
	xmlElement.setAttribute(s_ATTRIB_NEXT_HASH,			this.next_hash);
	for (var i = 0; i < this.reports.length; i++)	// >
	{
		xmlElement.appendChild(this.reports[i].serializeOut (xmlDoc));	
	}
	return xmlElement;
}
/*********************************************************************
 * Method: Reports.prototype.addReport
 *
 *********************************************************************/
Reports.prototype.addReport = function ()
{
	this.reports[this.reports.length] = new HIDReport(++this.next_hash);
}
/*********************************************************************
 * Method: Reports.prototype.deleteReport
 *
 *********************************************************************/
Reports.prototype.deleteReport = function (num)
{
	if ((num >= 0) && (num < this.reports.length))	// >
	{
		this.reports.splice(num, 1);
	}
}

/*********************************************************************
 * Class: HIDReport
 *
 *********************************************************************/
function HIDReport(hash)
{
	this.name = "HID_RPT_"+hash;
	this.hash = hash;
	this.items = new Array();
}
/*********************************************************************
 * Method: HIDReport.prototype.serializeIn
 *
 *********************************************************************/
HIDReport.prototype.serializeIn = function (xmlElement)
{
	this.name   = serializeInFromString(xmlElement.getAttribute(s_ATTRIB_NAME), "");
	this.hash   = serializeInFromString(xmlElement.getAttribute(s_ATTRIB_HASH), "");
	var items = xmlElement.getElementsByTagName(s_HID_ITEM);
	var i,j;
	for (i=0,j=0; i < items.length; i++)	// >
	{
		if (items[i].parentNode == xmlElement)
		{
			this.items[j] = new HIDItem();
			this.items[j].serializeIn(items[i]);
			j++;
		}
	}
}
/*********************************************************************
 * Method: HIDReport.prototype.serializeOut
 *
 *********************************************************************/
HIDReport.prototype.serializeOut = function (xmlDoc)
{
	var xmlElement = xmlDoc.createElement(s_HID_REPORT);
	// Add default attributes here
	xmlElement.setAttribute(s_ATTRIB_NAME,				this.name);
	xmlElement.setAttribute(s_ATTRIB_HASH,				this.hash);

	for (var i = 0; i < this.items.length; i++)	// >
	{
		xmlElement.appendChild(this.items[i].serializeOut (xmlDoc));	
	}
	return xmlElement;
}
/*********************************************************************
 * Method: HIDReport.prototype.addHIDItem
 *
 *********************************************************************/
HIDReport.prototype.addHIDItem = function (loc)
{
	if ((loc == null) || (loc >= this.items.length))
	{
		this.items[this.items.length] = new HIDItem();
	}
	else
	{
		this.items.splice(loc, 0, new HIDItem());
	}
}
/*********************************************************************
 * Method: HIDReport.prototype.deleteHIDItem
 *
 *********************************************************************/
HIDReport.prototype.deleteHIDItem = function (loc)
{
	if ((loc >= 0) && (loc < this.items.length))	// >
	{
		this.items.splice(loc, 1);
	}
}
/*********************************************************************
 * Method: HIDReport.prototype.isCollection
 *
 *********************************************************************/
HIDReport.prototype.isCollection = function ()
{
	return true;
}
/*********************************************************************
 * Method: HIDReport.prototype.isEndCollection
 *
 *********************************************************************/
HIDReport.prototype.isEndCollection = function ()
{
	return false;
}

/*********************************************************************
 * Class: HIDItem
 *
 *********************************************************************/
function HIDItem()
{
	this.item_type		= g_HIDItemMap.pairs['Undefined'].first;
	this.item_code		= g_HIDItemMap.pairs[this.item_type].second;
	this.size			= 0;
	this.data			= new Array();
	for (var i = 0; i < 4; ++i)
	{
		this.data[i] = new HIDData();
		this.data[i].order = i;
	}
	this.items			= new Array();
	this.editForm		= 'itemEditForm';
}
/*********************************************************************
 * Method: HIDItem.prototype.serializeIn
 *
 *********************************************************************/
HIDItem.prototype.serializeIn = function (xmlElement)
{
	this.item_type  = serializeInFromString(xmlElement.getAttribute(s_ATTRIB_ITEM_TYPE), g_HIDItemMap.pairs['Undefined'].first);
	
	this.item_code  = g_HIDItemMap.pairs[this.item_type].second;
	this.size       = serializeInFromString(xmlElement.getAttribute(s_ATTRIB_SIZE),  '0');

	var data = xmlElement.getElementsByTagName(s_ITEM_DATA);

	for (var i = 0; i < data.length; ++i)
	{
		this.data[i] = new HIDData();
		this.data[i].serializeIn(data[i]);
	}

	if (g_HIDItemOverrideMap.pairs[this.item_type].second[0] != null)
		this.toString = g_HIDItemOverrideMap.pairs[this.item_type].second[0];
	if (g_HIDItemOverrideMap.pairs[this.item_type].second[1] != null)
		this.editForm = g_HIDItemOverrideMap.pairs[this.item_type].second[1];

	if (this.isCollection())
	{
		var items = xmlElement.getElementsByTagName(s_HID_ITEM);
		var i,j;
		for (i=0,j=0; i < items.length; i++)	// >
		{
			if (items[i].parentNode == xmlElement)
			{
				this.items[j] = new HIDItem("");
				this.items[j].serializeIn(items[i]);
				j++;
			}
		}
	}
}
/*********************************************************************
 * Method: HIDItem.prototype.serializeOut
 *
 *********************************************************************/
HIDItem.prototype.serializeOut = function (xmlDoc)
{
	var xmlElement = xmlDoc.createElement(s_HID_ITEM);
	xmlElement.setAttribute(s_ATTRIB_ITEM_TYPE,		g_HIDItemMap.pairs[this.item_type].first);
	xmlElement.setAttribute(s_ATTRIB_ITEM_CODE,		g_HIDItemMap.pairs[this.item_type].second);
	xmlElement.setAttribute(s_ATTRIB_SIZE,			this.size);

	// Only serialize out as many data bytes as are specified by the size attrib
	//  (Note: size (0->0) (1->1) (2->2) (3->4)
	var s = ((this.size == 3) ? 4 : this.size);
	for (var i = 0; i < s; ++i)
	{
		xmlElement.appendChild(this.data[i].serializeOut (xmlDoc));	
	}

	if (this.isCollection)
	{
		for (var i = 0; i < this.items.length; i++)	// >
		{
			xmlElement.appendChild(this.items[i].serializeOut (xmlDoc));	
		}
	}
	return xmlElement;
}
/*********************************************************************
 * Method: HIDItem.prototype.isCollection
 *
 *********************************************************************/
HIDItem.prototype.isCollection = function ()
{
	var bRet = false;
	if (this.item_type == 'Collection')
		bRet =  true;
	return bRet;
}
/*********************************************************************
 * Method: HIDItem.prototype.isEndCollection
 *
 *********************************************************************/
HIDItem.prototype.isEndCollection = function ()
{
	var bRet = false;
	if (this.item_type == 'End Collection')
		bRet =  true;
	return bRet;
}
/*********************************************************************
 * Method: HIDItem.prototype.isDefined
 *
 *********************************************************************/
HIDItem.prototype.isDefined = function ()
{
	var bRet = false;
	if (this.item_type != 'Undefined')
		bRet =  true;
	return bRet;
}
/*********************************************************************
 * Method: HIDItem.prototype.changeType
 *
 *********************************************************************/
HIDItem.prototype.changeType = function(type)
{
	this.item_type   = g_HIDItemMap.pairs[type].first;
	this.item_code   = g_HIDItemMap.pairs[this.item_type].second;
	this.size = 0;
	delete this.data;
	this.data = new Array(4);
	for (var i =0; i < 4; ++i)
	{
		this.data[i] = new HIDData();
		this.data[i].order = i;
	}
		
	if (g_HIDItemOverrideMap.pairs[this.item_type].second[0] != null)
		this.toString = g_HIDItemOverrideMap.pairs[this.item_type].second[0];
	if (g_HIDItemOverrideMap.pairs[this.item_type].second[1] != null)
		this.editForm = g_HIDItemOverrideMap.pairs[this.item_type].second[1];

	delete this.items;
	this.items = new Array();
}
/*********************************************************************
 * Method: HIDItem.prototype.addHIDItem
 *
 *********************************************************************/
HIDItem.prototype.addHIDItem = function (loc)
{
	if ((loc == null) || (loc >= this.items.length))
	{
		this.items[this.items.length] = new HIDItem();
	}
	else
	{
		this.items.splice(loc, 0, new HIDItem());
	}
}
/*********************************************************************
 * Method: HIDItem.prototype.deleteHIDItem
 *
 *********************************************************************/
HIDItem.prototype.deleteHIDItem = function (loc)
{
	if ((loc >= 0) && (loc < this.items.length))	// >
	{
		this.items.splice(loc, 1);
	}
}
/*********************************************************************
 * Method: HIDItem.prototype.toString
 *
 *********************************************************************/
HIDItem.prototype.toString = function ()
{
	return (this.item_type + ' ' +this.itemCode());
}
/*********************************************************************
 * Method: HIDItem.prototype.itemCode
 *
 *********************************************************************/
HIDItem.prototype.itemCode = function ()
{
	var rStr = String();
	rStr += toHex(this.item_code | this.size);
	var size = this.size;
	if (size == 3)	// Adjust size value (0->0, 1->1, 2->2, 3->4) 
		size = 4;
	for (var i = 0; i < size ; ++i)
	{
		rStr += ' ' + (this.data[i]);
	}
	return rStr;
}
/*********************************************************************
 * Method: HIDItem.prototype.setData
 *
 *********************************************************************/
HIDItem.prototype.setData = function (index, value)
{
	this.data[index].value = value;
	this.data[index].order = index;
}

/*********************************************************************
 * Class: HIDData
 *
 *********************************************************************/
function HIDData()
{
	this.value = '00';
	this.order = 0;
}
/*********************************************************************
 * Method: HIDData.prototype.serializeIn
 *
 *********************************************************************/
HIDData.prototype.serializeIn = function (xmlElement)
{
	this.value  = serializeInFromString(xmlElement.getAttribute(s_ATTRIB_VALUE), '00');
	this.order  = serializeInFromString(xmlElement.getAttribute(s_ATTRIB_ORDER), 0);
}
/*********************************************************************
 * Method: HIDData.prototype.serializeOut
 *
 *********************************************************************/
HIDData.prototype.serializeOut = function (xmlDoc)
{
	var xmlElement = xmlDoc.createElement(s_ITEM_DATA);
	// Add default attributes here
	xmlElement.setAttribute(s_ATTRIB_VALUE,				this);
	xmlElement.setAttribute(s_ATTRIB_ORDER,				this.order);
	return xmlElement;
}
/*********************************************************************
 * Method: HIDData.prototype.toString
 *
 *********************************************************************/
HIDData.prototype.toString = function ()
{
	return this.value;
}
/*********************************************************************
 * Helpers
 *
 *********************************************************************/
/*********************************************************************
 * Function: toStringInput
 *
 *********************************************************************/
function toStringInput()
{
	var str = this.item_type + ' (' +inputItem2Text(this) + ')';
	return str;
}
/*********************************************************************
 * Function: toStringInput
 *
 *********************************************************************/
function toStringOutput()
{
	var str = this.item_type + ' (' +outputItem2Text(this) + ')';
	return str;
}
/*********************************************************************
 * Function: toStringCollection
 *
 *********************************************************************/
function toStringCollection()
{
	var str = this.item_type + ' (' + collectionItem2Text(this) + ')';
	return str;
}

/*********************************************************************
 * Function: inputItem2Text
 *
 *********************************************************************/
function inputItem2Text(item)
{
	sRet = "";
	sRet += ((item.data[0] & 0x01) ? "Constant"         : "Data");
	if ((item.data[0] & 0x01) == 0)
	{
		sRet += ((item.data[0] & 0x02) ? ", Variable"         : ", Array");
		sRet += ((item.data[0] & 0x04) ? ", Relative"         : ", Absolute");
		sRet += ((item.data[0] & 0x08) ? ", Wrap"             : ""/*"No-Wrap "*/);
		sRet += ((item.data[0] & 0x10) ? ", Non-Linear"       : ""/*"Linear "*/);
		sRet += ((item.data[0] & 0x20) ? ", No-Prefered"      : ""/*"Prefered-State "*/);
		sRet += ((item.data[0] & 0x40) ? ", Null-State"       : ""/*"No-Null-State "*/);
		sRet += ((item.data[1] & 0x01) ? ", Buffered-Bytes"   : ""/*"Bit-Field"*/);
	}
	sRet += ' ' + item.data[0];
	return sRet;
}
/*********************************************************************
 * Function: outputItem2Text
 *
 *********************************************************************/
function outputItem2Text(item)
{
	sRet = "";
	sRet += ((item.data[0] & 0x01) ? "Constant"         : "Data");
	if ((item.data[0] & 0x01) == 0)
	{
		sRet += ((item.data[0] & 0x02) ? ", Variable"         : ", Array");
		sRet += ((item.data[0] & 0x04) ? ", Relative"         : ", Absolute");
		sRet += ((item.data[0] & 0x08) ? ", Wrap"             : ""/*"No-Wrap "*/);
		sRet += ((item.data[0] & 0x10) ? ", Non-Linear"       : ""/*"Linear "*/);
		sRet += ((item.data[0] & 0x20) ? ", No-Prefered"      : ""/*"Prefered-State "*/);
		sRet += ((item.data[0] & 0x40) ? ", Null-State"       : ""/*"No-Null-State "*/);
		sRet += ((item.data[0] & 0x80) ? ", Volatile"         : ""/*"Non-Volatile "*/);
		sRet += ((item.data[1] & 0x01) ? ", Buffered-Bytes"   : ""/*"Bit-Field"*/);
	}
	sRet += ' ' + item.data[0];
	return sRet;
}

/*********************************************************************
 * Function: collectionItem2Text
 *
 *********************************************************************/
var g_CollectionMap = new map();
g_CollectionMap.add('Physical', 0x00);
g_CollectionMap.add('Application', 0x01);
g_CollectionMap.add('Logical', 0x02);
g_CollectionMap.add('Report', 0x03);
g_CollectionMap.add('Named Array', 0x04);
g_CollectionMap.add('Usage Switch', 0x05);
g_CollectionMap.add('Usage Modifier', 0x06);
g_CollectionMap.add('Reserved', 0x07);
g_CollectionMap.add('User Defined', 0x80);

function collectionItem2Text(item)
{
	sRet = "";
	if (item.size != 1)
	{
		sRet += "Size Undefined";
	}
	else
	{
		if ((item.data[0] >= 0) && (item.data[0] <= 6))
		{
			sRet += g_CollectionMap.reverse_lookup(item.data[0]);
		}
		else if (item.data[0] <= 0x7f)
		{
			sRet += g_CollectionMap.pairs['Reserved'].first;
		}
		else
		{
			sRet += g_CollectionMap.pairs['User Defined'].first;
		}
		sRet += ' ' + item.data[0];
	}
	return sRet;
}

/*********************************************************************
 * Function: toHex
 *
 *********************************************************************/
function toHex(dec)
{
   var hexChars = "0123456789ABCDEF";
   var result = new String();
   if (dec > 255) {
      return null;
   }
   var i = dec % 16;
   var j = (dec - i) / 16;
   //result = "0x";
   result += hexChars.charAt(j);
   result += hexChars.charAt(i);
   return result;
}
