/********************************************************************
 * Global
 *
 ********************************************************************/
var g_ReportSubject = new Subject("USBREPORTS");  // Subject of an observer update
var g_DirMap = null;
/********************************************************************
 * Method: Reports.prototype.loadUI
 *
 ********************************************************************/
Reports.prototype.loadUI = function (node)
{
	var child, ot;
	child = new TreeListControlNode(true, null, '');
	var ot = '<a href="javascript:addHIDReport(' + child.ID + ');">Add HID Report</a>';
    ot += '  |  ' + '<a href="javascript:importHIDReport(' + child.ID + ');">Import HID Report Template</a>';
	child.setText('HID Report Descriptor Root', this.name, ot);
	node.add(child);	
	for (var i = 0; i < this.reports.length; ++i)
	{
		this.reports[i].loadUI(child);
	}
}
/********************************************************************
 * Method: Reports.prototype.unloadUI
 *
 ********************************************************************/
Reports.prototype.unloadUI = function (node)
{
	var child = node.firstChild;
	this.name = child.columnText[1];
	child = child.firstChild;
	while (child != null)
	{
		// Little hack.  If an HTML element (hashed) text_ID does not exist, then
		//  a template add was in process but not committed.  So we skip it.
		if (document.getElementById('text_'+child.ID) != null)
		{
		    this.addReport();
		    this.reports[this.reports.length - 1].unloadUI(child);
		}
		child = child.nextSibling;
	}
}
/*********************************************************************
 * Function: addHIDReport
 *
 *********************************************************************/
function addHIDReport(node_id)
{
// TODO: I want an incrementing number instead of node_id for the default name
	var rpt = new HIDReport(node_id);
	// Application layer coupling using the tree root name.
	rpt.loadUI(g_ReportTree.allNodes[node_id]);
	var r = new Reports();
	r.unloadUI(g_ReportTree.rootNode);
	g_HIDReportMap = loadHIDReportMap(r.reports, g_HIDReportMap);
	g_ReportSubject.notify();
}
/*********************************************************************
 * Function: importHIDReport
 *
 *********************************************************************/
function importHIDReport(node_id)
{
    // Open a modal dialog with the menu of templates
	var node = g_ReportTree.allNodes[node_id];
    if (g_ps != null)
    {
        var xmlDoc      = g_ps.GetXMLDOM();
        xmlDoc.async    = "false";
        var umPath      = g_DeviceEditor.GetUserModulePath(g_sUMName);
	    
        var regex       = new RegExp('\\\\'+g_sUMName+'\\\\');
        var umTemplatePath= umPath.substring(0, umPath.search(regex) + g_sUMName.length + 2) + 'WIZARD\\templates\\';
        bRet = xmlDoc.load(umTemplatePath + g_HIDTemplateDirectory);
        DirRoot = new HIDTemplateRoot();
        if (bRet)
        {
	    	if (g_DirMap == null)
	    	{
	    	    DirRoot.serializeIn(xmlDoc.documentElement);
	            g_DirMap = new map();
	        
                g_DirMap.add("Select a HID Report Template",0);
	            var i;
	            for (i = 0; i < DirRoot.templates.length; i++)
	            {
                    g_DirMap.add(DirRoot.templates[i].description,DirRoot.templates[i].ID);
	            }
	        }
	        var child, it, ot;
	        child = new TreeListControlNode(false, null, '');
	        it = makeSelectP(g_DirMap, "Select a HID Report Template", 'onHIDTemplateChange(this, ' + child.ID + ')', 'select_' + child.ID);
	        ot = '<a href="javascript:ApplyTemplate(' + child.ID + ');">Apply</a>';
	        ot += ' | ';
	        ot += '<a href="javascript:CancelTemplate(' + child.ID + ');">Cancel</a>';
	
	        child.setText('HID Report Template',it,ot);
	        node.add(child);
        }
	}
}
/*********************************************************************
 * Function: onHIDTemplateChange
 *
 *********************************************************************/
function onHIDTemplateChange(object, node_id)
{
}
/*********************************************************************
 * Function: ApplyTemplate
 *
 *********************************************************************/
function ApplyTemplate(node_id)
{
	var node = g_ReportTree.allNodes[node_id];
    var select = document.getElementById('select_'+node.ID).value;
	var template_number = g_DirMap.pairs[select].second;
	if (template_number != 0)
	{
	// Serialize in the selected template
        var xmlDoc      = g_ps.GetXMLDOM();
        xmlDoc.async    = "false";
        var umPath      = g_DeviceEditor.GetUserModulePath(g_sUMName);
        var regex       = new RegExp('\\\\'+g_sUMName+'\\\\');
        var umTemplatePath= umPath.substring(0, umPath.search(regex) + g_sUMName.length + 2) + 'WIZARD\\templates\\';

        bRet = xmlDoc.load(umTemplatePath + g_HIDTemplateBase + template_number + '.xml');
        if (bRet)
        {
	        var parent = g_ReportTree.allNodes[node_id].parentNode;
	        g_ReportTree.allNodes[node_id].remove();
            var HIDTemplate = new HIDReport();
            HIDTemplate.serializeIn(xmlDoc.documentElement);
	        HIDTemplate.loadUI(parent);
            var r = new Reports();
    		r.unloadUI(g_ReportTree.rootNode);
	        g_HIDReportMap = loadHIDReportMap(r.reports, g_HIDReportMap);
	        g_ReportSubject.notify();
        }
	}
}
/*********************************************************************
 * Function: CancelTemplate
 *
 *********************************************************************/
function CancelTemplate(node_id)
{
	g_ReportTree.allNodes[node_id].remove();
}
/*********************************************************************
 * Function: removeHIDReport
 *
 *********************************************************************/
function removeHIDReport(node_id)
{
	// Application layer coupling using the tree root name.
	g_ReportTree.allNodes[node_id].remove();
	var r = new Reports();
	r.unloadUI(g_ReportTree.rootNode);
	g_HIDReportMap = loadHIDReportMap(r.reports, g_HIDReportMap);
	g_ReportSubject.notify();
}
/********************************************************************
 * Function: onReportNameChange
 *
 ********************************************************************/
function onReportNameChange(object, node_id)
{
	var r = new Reports();
//	if (validateLabelName(object, node_id))
//	{
		r.unloadUI(g_ReportTree.rootNode);
		if (g_HIDReportMap != null)
		{
		    g_HIDReportMap = loadHIDReportMap(r.reports, g_HIDReportMap);
		}
		g_ReportSubject.notify();
//	}
}
/*********************************************************************
 * Function: loadHIDReportMap
 *
 *********************************************************************/
function loadHIDReportMap(list, theMap)
{
	if (theMap != null)
	{
		delete theMap;
	}
	theMap = new map();
	theMap.add('None', 0);
	for (var i=0; i < list.length; ++i)
	{
		theMap.add(list[i].name, list[i].hash);
	}
	return theMap;
}
/********************************************************************
 * Method: HIDReport.prototype.loadUI
 *
 ********************************************************************/
HIDReport.prototype.loadUI = function (node)
{
	var child, it, ot;
	child = new TreeListControlNode(false, null, '');
	it = loadTextEditZ(this.name, 'onReportNameChange', child.ID, 'text_'+child.ID);
	
	ot = '<a href="javascript:insertUnderHIDItem(' + child.ID + ');">Insert HID Item</a>';
	ot += ' | ';
	ot += '<a href="javascript:removeHIDReport(' + child.ID + ');">Delete HID Report</a>';
	child.setText('HID Report Descriptor',it,ot);
	node.add(child);
	for (var i = 0; i < this.items.length; ++i)
	{
		this.items[i].loadUI(child);
	}
}
/********************************************************************
 * Method: HIDReport.prototype.unloadUI
 *
 ********************************************************************/
HIDReport.prototype.unloadUI = function (node)
{
	var child = node.firstChild;
	this.name = document.getElementById('text_'+node.ID).value;
	
	while (child != null)
	{
		this.addHIDItem();
		this.items[this.items.length - 1].unloadUI(child);
		child = child.nextSibling;
	}
}
/********************************************************************
 * Function: moveItemUp
 *
 ********************************************************************/
function moveItemUp(node_id)
{
	var node = g_ReportTree.allNodes[node_id];

	var item = new HIDItem();
    var ID = null;
    var movedID = null;
    
    if (node.firstChild != null) // Collection
    {
        if (node.previousSibling != null) // 
        {
            if (node.previousSibling.columnText[0] != 'End Collection' &&
                node.previousSibling.columnText[0] != 'Collection' )
            {
                item.unloadUI(node.previousSibling);
                movedID = node.previousSibling.ID;
                item.loadBeforeUI(node.firstChild);
                g_ReportTree.allNodes[movedID].remove();
                node.recalcChildImages();
            }
        }
    }
    else if (node.columnText[0] == 'End Collection')
    {
        if (node.previousSibling != null)
        {
            if (node.previousSibling.columnText[0] != 'Collection' &&
                node.previousSibling.columnText[0] != 'End Collection')
            {
                item.unloadUI(node.previousSibling);
                item.loadAfterUI(node.parentNode);
                node.previousSibling.remove();
	            node.parentNode.recalcChildImages();
            }
        }
    }
    else    // Anything else
    {
	    movedID = node.ID;
        item.unloadUI(node);
	    if (node.previousSibling != null)
	    {
            if (node.previousSibling.columnText[0] == 'Collection') // Collection
	        {
	            ID = node.previousSibling.lastChild.ID;
	            item.loadBeforeUI(g_ReportTree.allNodes[ID]);
	        }
	        else if (node.previousSibling.columnText[0] != 'HID Report Descriptor') // Top
	        {
	            ID = node.previousSibling.ID;
	            item.loadBeforeUI(g_ReportTree.allNodes[ID]);
	        }
	    }
	    else
	    {
	        ID = node.parentNode.ID;
	        item.loadBeforeUI(g_ReportTree.allNodes[ID]);
	    }
	    g_ReportTree.allNodes[movedID].remove();
	    if (ID != null)
	        g_ReportTree.allNodes[ID].parentNode.recalcChildImages();
    }
}
/********************************************************************
 * Function: moveItemDown
 *
 ********************************************************************/
function moveItemDown(node_id)
{
	var node = g_ReportTree.allNodes[node_id];

	var item = new HIDItem();
    var ID = null;
    var movedID = null;
    
    if (node.firstChild != null) // Collection
    {
        if (node.firstChild.firstChild == null) // child is not a collection
        {
            if (node.firstChild.columnText[0] != 'End Collection')
            {
                item.unloadUI(node.firstChild);
                movedID = node.firstChild.ID;
                if (node.previousSibling == null)
                {
                	item.loadBeforeUI(node);
                }
                else
                {
                	item.loadAfterUI(node.previousSibling);
                }
                g_ReportTree.allNodes[movedID].remove();
                node.recalcChildImages();
            }
        }
    }
    else if (node.columnText[0] == 'End Collection')
    {
        if (node.parentNode.nextSibling != null)
        {
            if (node.parentNode.nextSibling.columnText[0] != 'Collection' &&
                node.parentNode.nextSibling.columnText[0] != 'End Collection')
            {
                item.unloadUI(node.parentNode.nextSibling);
                item.loadBeforeUI(node);
                node.parentNode.nextSibling.remove();
	            node.parentNode.recalcChildImages();
            }
        }
    }
    else    // Anything else
    {
	    movedID = node.ID;
        item.unloadUI(node);
	    if (node.nextSibling == null)
	    {
	        ID = null;
	    }
	    else if (node.nextSibling.firstChild != null) // Collection
	    {
	        ID = node.nextSibling.firstChild.ID;
	        item.loadBeforeUI(g_ReportTree.allNodes[ID]);
	    }
	    else if (node.nextSibling.columnText[0] == 'End Collection')
	    {
	        ID = node.parentNode.ID;
	        item.loadAfterUI(g_ReportTree.allNodes[ID]);
	    }
	    else
	    {
	        ID = node.nextSibling.ID;
	        item.loadAfterUI(g_ReportTree.allNodes[ID]);
	    }
	    g_ReportTree.allNodes[movedID].remove();
	    if (ID != null)
	        g_ReportTree.allNodes[ID].parentNode.recalcChildImages();
    }
}
/********************************************************************
 * Function: insertHIDItem
 *
 ********************************************************************/
function insertHIDItem(node_id)
{
	var item = new HIDItem();
	// Application layer coupling using the tree root name.
	item.loadAfterUI(g_ReportTree.allNodes[node_id]);
}
/********************************************************************
 * Function: insertUnderHIDItem
 *
 ********************************************************************/
function insertUnderHIDItem(node_id)
{
	var item = new HIDItem();
	// Application layer coupling using the tree root name.
	var node = g_ReportTree.allNodes[node_id];
	if (node.firstChild == null)
	{
		item.loadUI(node);
	}
	else
	{
		item.loadBeforeUI(node.firstChild);
	}
}
/********************************************************************
 * Function: removeHIDItem
 *
 ********************************************************************/
function removeHIDItem(node_id)
{
	// Application layer coupling using the tree root name.
	g_ReportTree.allNodes[node_id].remove();
}
/********************************************************************
 * Function: itemEditForm
 *
 ********************************************************************/
function itemEditForm(node_id)
{
	var node = g_ReportTree.allNodes[node_id];
	var args = node.d_data.concat(new Array(node.d_size, node.d_type, node.d_code));
	var looks = 'unadorned: "YES"; help: "no"; status: "NO"; dialogHeight: 170px; dialogWidth: 375px;';
	var editform = 'DefaultItemEdit.htm';
	if (g_HIDItemOverrideMap.pairs[node.d_type].second[1] != null)
		editForm = g_HIDItemOverrideMap.pairs[node.d_type].second[1];
	var new_vals = window.showModalDialog(editform, args, looks);//, args, null);
	var item = new HIDItem();
	item.changeType(node.d_type);
	if (new_vals != null)
	{
		for (var i = 0; i < 4; ++i)
		{
			item.setData(i, new_vals[i]);
		}
		item.size = new_vals[4];
		item.loadItemData(node);
		item.fillUI(node);
	}
}
/********************************************************************
 * Function: onHIDReportItemChange
 *
 ********************************************************************/
function onHIDReportItemChange(object, node_id)
{
	var item = new HIDItem();
	item.changeType(object.value);
	if (item.isCollection())
	{
		item.addHIDItem();
		item.items[0].changeType('End Collection');
	}
	item.reloadUI(g_ReportTree.allNodes[node_id]);
}
/********************************************************************
 * Method: HIDItem.prototype.loadUI
 *
 ********************************************************************/
HIDItem.prototype.loadUI = function (node)
{
	var child;
	child = new TreeListControlNode(false, null, '');
	this.loadItemData(child);
	node.add(child)
	this.fillUI(child);
	if (this.isCollection())
	{
		for (var i = 0; i < this.items.length; ++i)
		{
			this.items[i].loadUI(child);
		}
	}
}
/********************************************************************
 * Method: HIDItem.prototype.loadItemData
 *
 ********************************************************************/
HIDItem.prototype.loadItemData = function (node)
{
	node.d_type = this.item_type;
	node.d_size = this.size;
	node.d_code = this.item_code;
	for (var i = 0; i < 4; ++i)
	{
		node.d_data[i] = this.data[i].value;
	}
}
/********************************************************************
 * Method: HIDItem.prototype.loadAfterUI
 *
 ********************************************************************/
HIDItem.prototype.loadAfterUI = function (node)
{
	var child;
	child = new TreeListControlNode(false, null, '');
	this.loadItemData(child);
	node.insertAfter(child)
	this.fillUI(child);
	if (this.isCollection())
	{
		for (var i = 0; i < this.items.length; ++i)
		{
			this.items[i].loadUI(child);
		}
	}
}
/********************************************************************
 * Method: HIDItem.prototype.loadBeforeUI
 *
 ********************************************************************/
HIDItem.prototype.loadBeforeUI = function (node)
{
	var child;
	child = new TreeListControlNode(false, null, '');
	this.loadItemData(child);
	node.insertBefore(child)
	this.fillUI(child);
	if (this.isCollection())
	{
		for (var i = 0; i < this.items.length; ++i)
		{
			this.items[i].loadUI(child);
		}
	}
}
/********************************************************************
 * Method: HIDItem.prototype.reloadUI
 * Note: This should only be invoked when an Item type has changed
 ********************************************************************/
HIDItem.prototype.reloadUI = function (node)
{
	this.loadItemData(node);
	this.fillUI(node);
	if (this.isCollection())
	{
		for (var i = 0; i < this.items.length; ++i)
		{
			this.items[i].loadUI(node);
		}
	}
}
/********************************************************************
 * Method: HIDItem.prototype.fillUI
 *
 ********************************************************************/
HIDItem.prototype.fillUI = function (node)
{
	var it = new String('');
	var ot = new String('');
	var mt = new String('');
	var in_s = '<a href="javascript:insertHIDItem(' + node.ID + ');">Insert HID Item</a>';
	var ed   = '<a href="javascript:itemEditForm( ' + node.ID + ');">' + this + '</a>';

    var rm = '<a href="javascript:removeHIDItem(' + node.ID + ');">Remove</a>';
	var mv = '<a href="javascript:moveItemUp(' + node.ID + ');">Move Up</a>';
	mv += ' | ';
	mv += '<a href="javascript:moveItemDown(' + node.ID + ');">Move Down</a>';


	if (this.isDefined())
	{
		it = this.item_type;
		if (this.isEndCollection())
		{
			var in_e = '<a href="javascript:insertHIDItem(' + node.parentNode.ID + ');">Insert HID Item</a>';
			mt = this;
			ot = in_e + ' | ' + mv;
		}
		else if (this.isCollection())
		{
			var in_c = '<a href="javascript:insertUnderHIDItem(' + node.ID + ');">Insert HID Item</a>';
			ot = in_c + ' | ' + rm + ' | ' + mv;
			mt = ed;
		}
		else
		{
			ot = in_s + ' | ' + rm + ' | ' + mv;
			mt = ed;
		}
	}
	else
	{
		it = makeSelectP(g_HIDItemSelectMap, this.item_type, 'onHIDReportItemChange(this, ' + node.ID + ')', 'select_' + node.ID);
		ot = in_s + ' | ' + rm + ' | ' + mv;
	}
	node.setText(it, mt, ot);
}
/********************************************************************
 * Method: HIDItem.prototype.unloadUI
 *
 ********************************************************************/
HIDItem.prototype.unloadUI = function (node)
{
	this.changeType(node.d_type);
	this.size = node.d_size;
	for (var i = 0; i < 4; ++i)
	{
		this.setData(i, node.d_data[i]);
	}
	var child = node.firstChild;
	while (child != null)
	{
		this.addHIDItem();
		this.items[this.items.length - 1].unloadUI(child);
		child = child.nextSibling;
	}
}