/********************************************************************
 * Global
 *
 ********************************************************************/
var g_EditorDirty = false;
var g_DirectoryDirty = false;
var deviceHeadings = new Array(new pair(300, 'Descriptor'),
                               new pair(250, 'Data'), 
                               new pair(250, 'Operations'));
var g_DescriptionTitles = new Subject('DESCRIPTIONS');  // Subject of an observer update
/********************************************************************
 * Function: HIDTemplateRoot.prototype.loadUI
 *
 ********************************************************************/
HIDTemplateRoot.prototype.loadUI = function (node)
{
	var child, ot;
	child = new TreeListControlNode(true, null, '');

    ot = '';
    if (HIDTemplateRootHandler.enableEditor)
        ot = '<a href="javascript:insertTemplate(' + child.ID + ');">Add a Report Template</a>';
    
	child.setText('HID Report Template Directory', '', ot);
	node.add(child);	

    for (var i = 0; i < this.templates.length; ++i)
    {
        this.templates[i].loadUI(child);
    }
}
/********************************************************************
 * Method: HIDTemplateRoot.prototype.unloadUI
 *
 ********************************************************************/
HIDTemplateRoot.prototype.unloadUI = function (node)
{
	var child = node.firstChild.firstChild;

	this.instance_name = node.firstChild.columnText[1];
    while (child != null)
    {
        this.addHIDTemplate();
        this.templates[this.templates.length - 1].unloadUI(child);
        child = child.nextSibling;
    }
}
/********************************************************************
 * Method: HIDTemplate.prototype.loadUI
 *
 ********************************************************************/
HIDTemplate.prototype.loadUI = function (node)
{
	var child;
	var mt = new String('');
	var it = new String('');
	var ot = new String('');

	child = new TreeListControlNode(true, null, '');
	//var observer = new Observer('DESCRIPTIONS', updateDescription, child.ID);
	var observer = new Observer('USBREPORTS', updateDescription, child.ID);
	
	mt = 'HID Report Template: ' + this.ID;
	it = this.description;

    if (HIDTemplateRootHandler.enableEditor)
    {
	    if (this.allow_delete == 'Enable')
	        ot +=  '<a href="javascript:removeTemplate(' + child.ID + ');">Remove</a>';
	    if (this.allow_modify == 'Enable')
	    {
	    if (ot.length > 0)
	            ot += ' | ';
	        ot += '<a href="javascript:modifyReport(' + child.ID + ');">Modify</a>';
	    }
	    if (ot.length > 0)
	        ot += ' | ';
	    ot +=  '<a href="javascript:copyTemplate(' + child.ID + ');">Copy</a>';
	}
	else if (HIDTemplateRootHandler.enableExport)
	{
	    if (ot.length > 0)
	        ot += ' | ';
	    ot +=  '<a href="javascript:onSelectTemplate(' + child.ID + ');">Import to Project</a>';
	}
	child.setText(mt, it, ot);
	node.add(child);
}
/********************************************************************
 * Method: HIDTemplate.prototype.unloadUI
 *
 ********************************************************************/
HIDTemplate.prototype.unloadUI = function (node)
{
    this.description   = node.columnText[1];
    this.ID            = node.columnText[0].split(': ')[1];
	this.allow_delete  = g_Enable.pairs["Enable"].first;
	this.allow_modify  = g_Enable.pairs["Enable"].first;
    this.date_saved    = new Date();	
}
/********************************************************************
 * Function: updateDescription
 *
 * This function is an observer callback when the Description Changes
 *
 ********************************************************************/
function updateDescription(node_id)
{
	var dir_node = g_DirTree.allNodes[node_id];
    if (dir_node == null)
    {
        throw 0;
    }
    // New description is in the g_EditorTree, but we must match
    //  the report id 
    var desc_node = g_ReportTree.rootNode.firstChild;
    if (dir_node.columnText[0] == desc_node.columnText[0])
    {
        var name_node = desc_node.firstChild;
        dir_node.setColumnText(1, document.getElementById('text_'+name_node.ID).value);
        g_DirectoryDirty = true;    
    }
}
/********************************************************************
 * Function: insertTemplate
 *
 ********************************************************************/
function insertTemplate(node_id)
{
	var t = g_DirRoot.addHIDTemplate();
	t.loadUI(g_DirTree.allNodes[node_id]);
    document.getElementById('Apply').disabled = false;
	g_DirectoryDirty = true;
}
/********************************************************************
 * Function: removeTemplate
 *
 ********************************************************************/
function removeTemplate(node_id)
{
	g_DirTree.allNodes[node_id].remove();
    document.getElementById('Apply').disabled = false;
	g_DirectoryDirty = true;
}
/********************************************************************
 * Function: modifyReport
 *
 ********************************************************************/
function modifyReport(node_id)
{
    if (g_EditorDirty)
    {
	    document.getElementById('EDITOR').innerHTML = '';
    }
    var dir_node = g_DirTree.allNodes[node_id];
    var ed = null;
    // Check to see if another template is already in the editor
    //   If so, ask if they want to save that template
    // Load the specified template
    var id = dir_node.columnText[0].split(': ')[1];
    var bRet = g_xmlDoc.load(g_umTemplatePath + g_HIDTemplateBase + id + '.xml');
    ed = new HIDReport(0);
    if (bRet)
    {
        ed.serializeIn(g_xmlDoc.documentElement);
    }
    // Force the name to be the same as the directory.
    ed.name = dir_node.columnText[1];
    g_EditorCtrl = new TreeEditor(ed);
    g_EditorCtrl.setEditColumn(0, dir_node.columnText[0], false, '');
    //g_EditorCtrl.setEditColumn(1, dir_node.columnText[1], true, 'validateDescription');
    g_ReportTree = loadTreeX('EDITOR', g_EditorCtrl, deviceHeadings);

    // Add (or enable) buttons that let them save the template
    document.getElementById('Apply').disabled = false;

    g_EditorDirty = true;
    // Done
}

/********************************************************************
 * Function: onApply
 *
 ********************************************************************/
function onApply()
{
    var bRet = null;
    if (g_EditorDirty)
    {
	    // Save the old template in the backup directory
        var id = g_ReportTree.rootNode.firstChild.columnText[0].split(': ')[1];
	    bRet = g_xmlDoc.load(g_umTemplatePath + g_HIDTemplateBase + id + '.xml');
	    if (bRet)
	        g_xmlDoc.save(g_umTemplatePath + 'backup\\' + g_HIDTemplateBase + id + '.xml');
        delete g_EditorCtrl.edit;
        g_EditorCtrl.edit = new HIDReport(id);
        g_EditorCtrl.unloadUI(g_ReportTree.rootNode); 
        g_xmlDoc.documentElement = g_EditorCtrl.edit.serializeOut(g_xmlDoc);
	    bRet = g_xmlDoc.save(g_umTemplatePath + g_HIDTemplateBase + id + '.xml');
    }
    if (g_EditorDirty || g_DirectoryDirty)
    {
        // Save the directory
	    bRet = g_xmlDoc.load(g_umTemplatePath + g_HIDTemplateDirectory);
	    if (bRet)
	        g_xmlDoc.save(g_umTemplatePath + 'backup\\' + g_HIDTemplateDirectory);
        var d = new HIDTemplateRoot();
        d.unloadUI(g_DirTree.rootNode);
        g_xmlDoc.documentElement = d.serializeOut(g_xmlDoc);
        g_xmlDoc.save(g_umTemplatePath + g_HIDTemplateDirectory);
    }
}
/********************************************************************
 * Function: copyTemplate
 *
 ********************************************************************/
function copyTemplate(node_id)
{
    // Load the source
    var dir_node = g_DirTree.allNodes[node_id];
    var id = dir_node.columnText[0].split(': ')[1];
    var bRet = g_xmlDoc.load(g_umTemplatePath + g_HIDTemplateBase + id + '.xml');
    // Add a directory entry (we need the destination id)
	var t = g_DirRoot.addHIDTemplate();
    // Save the destination
    var bRet = g_xmlDoc.save(g_umTemplatePath + g_HIDTemplateBase + t.ID + '.xml');
    // Update the description with "Copy of "
    t.description = 'Copy of ' + dir_node.columnText[1];
	t.loadUI(g_DirTree.allNodes[node_id].parentNode);
    g_EditorDirty = true;
    document.getElementById('Apply').disabled = false;
}
/********************************************************************
 * Class: TreeEditor
 *
 ********************************************************************/
function TreeEditor(ed)
{
    this.columnText       = new Array();
    this.columnEditEnable = new Array();
    this.columnValidator  = new Array();
    this.edit = null;
    if (ed != null)
    {
        this.edit = ed;
    }
}
/********************************************************************
 * Method: TreeEditor.prototype.setEditColumn
 *
 ********************************************************************/
TreeEditor.prototype.setEditColumn = function (col, text, enable, validator)
{
    this.columnText[col]       = text;
    this.columnEditEnable[col] = enable;
    this.columnValidator[col]  = validator;
}
/********************************************************************
 * Method: TreeEditor.prototype.loadUI
 *
 ********************************************************************/
TreeEditor.prototype.loadUI = function (node)
{
	var child = new TreeListControlNode(true, null, '');
    for (var i = 0; i < this.columnText.length; i++)
    {
        if (this.columnEditEnable[i])
        {
            child.setColumnText(i, loadTextEditZ(this.columnText[i], this.columnValidator[i], ((child.ID * 10) + i), 'text_'+((child.ID * 10) + i)));
        }
        else
        {
            child.setColumnText(i, this.columnText[i]);
        } 
    }
    node.add(child);
    if (this.edit !=null)
    {
        this.edit.loadUI(child);
    }
}
/********************************************************************
 * Method: TreeEditor.prototype.unloadUI
 *
 ********************************************************************/
TreeEditor.prototype.unloadUI = function (node)
{
    var child = node.firstChild;
    for (var i = 0; i < this.columnText.length; i++)
    {
        if (this.columnEditEnable[i])
        {
            this.columnText[i] = document.getElementById('text_'+((child.ID * 10) + i)).value;
        }
    }
    child = child.firstChild;
    // This is a little tricky.  Not sure of another way to do this at runtime
    if (this.edit != null)
    {
        this.edit.unloadUI(child);
    }
}
/********************************************************************
 * Function: validateDescription
 *
 ********************************************************************/
function validateDescription(object, edit_id)
{
    g_DescriptionTitles.notify();
}
/********************************************************************
 * Function: onOK
 *
 ********************************************************************/
function onOK()
{
    onApply();
    g_pDlg.Close(0);
}
/********************************************************************
 * Function: onCancel
 *
 ********************************************************************/
function onCancel()
{
    var bExit = true;
    if (g_EditorDirty || g_DirectoryDirty)
    {    
        bExit = window.confirm('Your changes have not been saved.  Exit?');
    }
    
    if (bExit)
        g_pDlg.Close(0);
}