//*****************************************************************************
//     FILENAME: ArchitectureVis.js
//      VERSION: 2.0, Aug 15, 2005
//  DESCRIPTION: Reduces information in a user module datasheet HTML file
//               so it covers only one register architecture.
//
//  Copyright (c) Cypress MicroSystems 2005. All Rights Reserved.
//*****************************************************************************

var g_sOverLay                  = '';
var g_sUMName                   = '';
var g_sInstanceName             = '';
var g_DeviceEditor              = null;

// var g_ProjectManager         = null;
// var g_xmlDoc                 = null;


function setPSoC(ps, pDlg, sOverLay, sUMName, sInstanceName)
{
   // Called from outside
   // Adds Name PSoC Designer components to scripting engine's namespace

   g_sOverLay       = sOverLay;
   g_sUMName        = sUMName;
   g_sInstanceName  = sInstanceName;
   g_DeviceEditor   = ps.GetDeviceEditor();

   // g_ProjectManager = ps.GetProjectManager();
   // g_xmlDoc         = ps.GetXMLDOM();

   if(g_DeviceEditor != null)
   {
      customizeDatasheet( g_DeviceEditor.GetBaseDevice() );
   }
}


function customizeDatasheet( sFamilyName )
{
   var reArch27xxx = /CY8C2(1|2|3|4|7|8|9)/;
   var reArch26xxx = /CY8C2(5|6)/;
   var reE3 = /CYWUSB/;
   var reE2 = /CY7C6/;

   if ( sFamilyName.match(reArch27xxx) || sFamilyName.match(reE3) || sFamilyName.match(reE2))
   {
      visibleByID('CY8C26', 0);  // hide 26xxx content
      visibleByID('CY8C00', 0);  // hide connective material
      visibleByID('CY8C27', 1);  // show 27xxx content
   }
   else if ( sFamilyName.match(reArch26xxx) )
   {
      visibleByID('CY8C27', 0);  // hide 27xxx content
      visibleByID('CY8C00', 0);  // hide connective material
      visibleByID('CY8C26', 1);  // show 26xxx content
   }
   else
   {
      //don't hide anything
      visibleByID('CY8C27', 1);  // show 26xxx content
      visibleByID('CY8C00', 1);  // show connective material
      visibleByID('CY8C26', 1);  // show 27xxx content
   }
}


function visibleByID( sBasisName, bMakeVisible )
{
   var i = 0;
   while( 1 )
   {
      var sID = 'PSoC_' + sBasisName + '_' + i.toString();
      var El  = document.getElementById( sID );

      if(El == null)
      {
         // Because the numbering sequence is contiguous, this
         // means there are no more IDs with this basis
         break
      }
      else
      {
         if ( bMakeVisible )
         {
            El.style.visibility = 'visible';
            if ( El.tagName == 'SPAN' )
               El.style.display = 'inline';
            else
               El.style.display = 'block';  // used for <div> tags
         }
         else
         {
            El.style.visibility = 'hidden';
            El.style.display    = 'none';
         }
      }
      i++;  // next member of sequence...
   }
}
